

% ------------------------------
% |        S. digitalis        |
% | Yeast Evolution Simulation |
% |  Version 1.79, 2005-2009   |
% |     Philipp J. Keller      |
% |      EMBL Heidelberg       |
% ------------------------------
% 
% "Evolution of essential gene clusters
%  and coevolution of essential gene distributions
%  and distributions of meiotic recombination hot spots"
% 
% Theoretical model by Philipp J. Keller and Michael Knop
%
% European Molecular Biology Laboratory
% Cell Biology and Biophysics Unit
% Meyerhofstrasse 1, 69117 Heidelberg
% 
% Email: keller@embl.de, knop@embl.de


function results = evolve(...
    nGenesInput, nEssentialGenesInput, nRPIEInput, verticalParametersInput,...
    verticalParametersSpacingInput, horizontalParametersInput, statisticalAveragingInput, ...
    seedingSetupInput, clusterFrequencyInput, clusterFrequencyArrayInput, seedingKnockoutInput,...
    seedingKnockoutArrayInput, fittingSchemeInput, performSurvivalScreenInput,...
    screeningArrayInput, positiveSurvivalRunsInput, totalSimulationStepsInput,...
    inbreedingFractionInput, requiredHomologousFractionInput, rearrangementModeInput,...
    recombinationProbabilityInput, recombinationUnitFactorInput, recombinationModeInput,...
    erlangApplicationFlagInput, randomApplicationFlagInput, matingLocusFlagInput,...
    matingLocusPositionInput, externalLocusInput, matingSwitchingInput, adaptiveMutationRateInput,...
    relativeFeedbackFlagInput, feedbackPopulationSizeInput, feedbackPopulationSizeDerivativeInput,...
    derivativeDistanceInput, populationCapSizeInput, initialOrMaxGenomesInput,...
    protocolSplittingSizeInput, databaseResolutionInput, numericalCorrectionFactorInput,...
    skipMeiosisInput, randomizeCyclesInput, mutationApplicationInput, mutationModeInput, ...
    subPopulationFlagInput, displayInfoInput, readRandomGeneratorInput, silentIOInput, ...
    analyzeClusteringInput, frontendFlagInput, clusterModeInput, centralRepositoryInput, ...
    centralSeedingInput, touchInput)

% Invoke conventional test:  evolve(500, 100, 166, 0.010, 0, 0.0100, 1, 13,   1,   1, 0.00, 0.00, 1, 0, 2, 1,  10, 1.0, 0.9, 1, 1, 1, 0, 1, 0, 1, 99, 0, 0.5, 0, 1, 0.0, 0.00, 3, 1,  200, 100, 100, 0.682075, 0, 1, 1, 0, 0, 1, 0, 0, 1, 0, 0, '.', '.', '.')
% Invoke feedback test:      evolve(500, 100, 166, 0.015, 0, 0.0000, 1, 13, 100, 100, 0.05, 0.05, 1, 0, 2, 1, 100, 0.5, 0.9, 0, 1, 1, 1, 1, 1, 1, 99, 0, 0.5, 1, 1, 1.0, 0.00, 3, 0, 1000, 100, 100, 0.682075, 0, 1, 1, 0, 0, 1, 0, 0, 1, 0, 0, '.', '.', '.')
% Invoke competition test:   evolve(207,  35,  58, 0.010, 0, 0.0000, 1, 16,   1,   1, 0.00, 0.00, 1, 0, 2, 1, 100, 0.5, 0.9, 1, 1, 1, 0, 1, 0, 1, 99, 0, 0.5, 0, 1, 0.0, 0.00, 3, 1,  100, 100, 100, 0.397950, 0, 1, 1, 0, 0, 1, 0, 0, 1, 0, 0, '.', '.', '.')
% Invoke evolution test:     evolve(207,  35,  58, 0.010, 0, 0.0001, 1,  5,   1,   1, 0.00, 0.00, 1, 0, 2, 1, 100, 0.5, 0.9, 1, 1, 1, 0, 1, 0, 1, 99, 0, 0.5, 0, 1, 0.0, 0.00, 3, 1,  100,  10,  10, 0.397950, 0, 1, 1, 0, 1, 1, 0, 0, 1, 0, 0, '.', '.', '.')
% Invoke batch test:       ./evolve 500 100 166 0.005 0 0.0001 1 13 50 50 0.00 0.00 1 0 2 1 20 1 0.9 0 1 1 0 1 0 1 199 0 0.5 0 1 0.0 0.00 3 1 200 100 100 0.682075 0 1 1 0 0 1 0 0 1 0 0 '.' '.' '.'

% Invoke command line test:

% nGenesInput = 207;
% nEssentialGenesInput = 35;
% nRPIEInput = 58;
% verticalParametersInput = 1/70;
% verticalParametersSpacingInput = 0;
% horizontalParametersInput = 0;
% statisticalAveragingInput = 1;
% seedingSetupInput = 16;
% clusterFrequencyInput = 1;
% clusterFrequencyArrayInput = 1;
% seedingKnockoutInput = 0;
% seedingKnockoutArrayInput = 0;
% fittingSchemeInput = 1;
% performSurvivalScreenInput = 0;
% screeningArrayInput = 2;
% positiveSurvivalRunsInput = 1;
% totalSimulationStepsInput = 200;
% inbreedingFractionInput = 0;
% requiredHomologousFractionInput = 0.9;
% rearrangementModeInput = 0;
% recombinationProbabilityInput = 1;
% recombinationUnitFactorInput = 1;
% recombinationModeInput = 0;
% erlangApplicationFlagInput = 1;
% randomApplicationFlagInput = 0;
% matingLocusFlagInput = 1;
% matingLocusPositionInput = 333;
% externalLocusInput = 1;
% matingSwitchingInput = 0.5;
% adaptiveMutationRateInput = 0;
% relativeFeedbackFlagInput = 1;
% feedbackPopulationSizeInput = 0;
% feedbackPopulationSizeDerivativeInput = 0;
% derivativeDistanceInput = 3;
% populationCapSizeInput = 1;
% initialOrMaxGenomesInput = 20000;
% protocolSplittingSizeInput = 100;
% databaseResolutionInput = 100;
% numericalCorrectionFactorInput = 0.39795;
% skipMeiosisInput = 0;
% randomizeCyclesInput = 1;
% mutationApplicationInput = 1;
% mutationModeInput = 0;
% subPopulationFlagInput = 0;
% displayInfoInput = 0;
% readRandomGeneratorInput = 0;
% silentIOInput = 1;
% analyzeClusteringInput = 0;
% frontendFlagInput = 0;
% clusterModeInput = 0;
% centralRepositoryInput = '/tmp/home/keller/test';
% centralSeedingInput = '/grid/fzk.de/mounts/nfs/home/keller/genomes';
% touchInput = '/tmp/home/keller';

simulationVersion = 1.79;

% ---------------------------
% | Solution space settings |
% ---------------------------

% numericalCorrectionFactor templates:
% 500 genes, erlangApplicationFlag = 0, 7.5 events: 0.671715
% 500 genes, erlangApplicationFlag = 1, 7.5 events: 0.682075
% 207 genes, erlangApplicationFlag = 0, 4.31 events: 0.38721
% 207 genes, erlangApplicationFlag = 1, 4.31 events: 0.39795

if (isdeployed)
    nGenesInput = str2num(nGenesInput);
    nEssentialGenesInput = str2num(nEssentialGenesInput);
    nRPIEInput = str2num(nRPIEInput);
    verticalParametersInput = str2num(verticalParametersInput);
    verticalParametersSpacingInput = str2num(verticalParametersSpacingInput);
    horizontalParametersInput = str2num(horizontalParametersInput);
    statisticalAveragingInput = str2num(statisticalAveragingInput);
    seedingSetupInput = str2num(seedingSetupInput);
    clusterFrequencyInput = str2num(clusterFrequencyInput);
    clusterFrequencyArrayInput = str2num(clusterFrequencyArrayInput);
    seedingKnockoutInput = str2num(seedingKnockoutInput);
    seedingKnockoutArrayInput = str2num(seedingKnockoutArrayInput);
    fittingSchemeInput = str2num(fittingSchemeInput);
    performSurvivalScreenInput = str2num(performSurvivalScreenInput);
    screeningArrayInput = str2num(screeningArrayInput);
    positiveSurvivalRunsInput = str2num(positiveSurvivalRunsInput);
    totalSimulationStepsInput = str2num(totalSimulationStepsInput);
    inbreedingFractionInput = str2num(inbreedingFractionInput);
    requiredHomologousFractionInput = str2num(requiredHomologousFractionInput);
    rearrangementModeInput = str2num(rearrangementModeInput);
    recombinationProbabilityInput = str2num(recombinationProbabilityInput);
    recombinationUnitFactorInput = str2num(recombinationUnitFactorInput);
    recombinationModeInput = str2num(recombinationModeInput);
    erlangApplicationFlagInput = str2num(erlangApplicationFlagInput);
    randomApplicationFlagInput = str2num(randomApplicationFlagInput);
    matingLocusFlagInput = str2num(matingLocusFlagInput);
    matingLocusPositionInput = str2num(matingLocusPositionInput);
    externalLocusInput = str2num(externalLocusInput);
    matingSwitchingInput = str2num(matingSwitchingInput);
    adaptiveMutationRateInput = str2num(adaptiveMutationRateInput);
    relativeFeedbackFlagInput = str2num(relativeFeedbackFlagInput);
    feedbackPopulationSizeInput = str2num(feedbackPopulationSizeInput);
    feedbackPopulationSizeDerivativeInput = str2num(feedbackPopulationSizeDerivativeInput);
    derivativeDistanceInput = str2num(derivativeDistanceInput);
    populationCapSizeInput = str2num(populationCapSizeInput);
    initialOrMaxGenomesInput = str2num(initialOrMaxGenomesInput);
    protocolSplittingSizeInput = str2num(protocolSplittingSizeInput);
    databaseResolutionInput = str2num(databaseResolutionInput);
    numericalCorrectionFactorInput = str2num(numericalCorrectionFactorInput);
    skipMeiosisInput = str2num(skipMeiosisInput);
    randomizeCyclesInput = str2num(randomizeCyclesInput);
    mutationApplicationInput = str2num(mutationApplicationInput);
    mutationModeInput = str2num(mutationModeInput);
    subPopulationFlagInput = str2num(subPopulationFlagInput);
    displayInfoInput = str2num(displayInfoInput);
    readRandomGeneratorInput = str2num(readRandomGeneratorInput);
    silentIOInput = str2num(silentIOInput);
    analyzeClusteringInput = str2num(analyzeClusteringInput);
    frontendFlagInput = str2num(frontendFlagInput);
    clusterModeInput = str2num(clusterModeInput);
end;

jobCompleted = 0;

frontendFlag = frontendFlagInput;
clusterMode = clusterModeInput;

if clusterMode
    seedingDirectory = centralSeedingInput;
end;

touchDirectory = touchInput;

if readRandomGeneratorInput
    load randomGenerator;
else
    currentClock = clock;
    randomGenerator = sum(currentClock .* [1 1 1 1 1 10 ^ 7]);
end;
rand('twister', randomGenerator);

if verticalParametersSpacingInput ~= 0
    verticalParametersInput = verticalParametersInput:-verticalParametersSpacingInput:0;
end;

verticalParametersList = verticalParametersInput;
horizontalParametersList = 2 .* horizontalParametersInput;

verticalParameters = length(verticalParametersList);
horizontalParameters = length(horizontalParametersList);

% Notes:
% - verticalParameters implements mitotic mutation rates
% - horizontalParameters implements mitotic rearrangement rates,
%   for a row index independently of chromosome specificity (thus factor 2)

statisticalAveraging = statisticalAveragingInput;

seedingSetup = seedingSetupInput;

clusterFrequency = clusterFrequencyInput;
clusterFrequencyArray = clusterFrequencyArrayInput;
clusterFrequencyArrayLength = length(clusterFrequencyArray);

seedingKnockout = seedingKnockoutInput;
seedingKnockoutArray = seedingKnockoutArrayInput;
seedingKnockoutArrayLength = length(seedingKnockoutArray);

fittingScheme = fittingSchemeInput;

% Note: fittingScheme settings are 0 for linear fit, 1 for exponential fit

performSurvivalScreen = performSurvivalScreenInput;

% Note: performSurvivalScreen settings are 0 for no screen, 1 for mutation
% rate or rearrangement rate screen while keeping clusterFrequency and
% seedingKnockout constant, 2 for seedingKnockout screen applied to a
% series of clusterFrequency parameters

screeningArray = screeningArrayInput;
positiveSurvivalRuns = positiveSurvivalRunsInput;

% ------------------------------------------------------------------------
% seedingSetup configuration:
% 0:  one single random genome
% 1:  one single predefined unclustered genome
% 2:  one single predefined clustered genome
% 3:  two single predefined genomes, each one unclustered and one clustered
% 4:  maxGenomes different random genomes
% 5:  maxGenomes indentical random genomes
% 6:  maxGenomes predefined unclustered genomes
% 7:  maxGenomes predefined clustered genomes
% 8:  maxGenomes predefined genomes, half of them unclustered, the other
%     half clustered
% 9:  maxGenomes predefined unclustered genomes, with a fraction of
%     seedingKnockout mutated essential genes per chromosome at random
%     positions
% 10: maxGenomes predefined clustered genomes, with a fraction of
%     seedingKnockout mutated essential genes per chromosome at random
%     positions
% 11: read-in of seedingGenome from file (seedingGenome.mat) that must be
%     available from the same directory as evolve.m
% 12: maxGenomes predefined clustered genomes, with a cluster frequency
%     that is given by the parameter clusterFrequency
% 13: maxGenomes predefined clustered genomes, with a cluster frequency
%     that is given by the parameter clusterFrequency and a fraction of
%     seedingKnockout mutated essential genes per chromosome at random
%     positions
% 14: deathmatch mode 1, starting with read-in of seedingGenome from file
%     (seedingGenome.mat) that must be available from the same directory as
%     evolve.m. In mode 1, maxGenomes/2 are defined according to the first
%     two columns of seedingGenome. Another maxGenomes/2 are defined
%     according to the third/fourth columns of seedingGenome.
% 15: deathmatch mode 2, starting with read-in of seedingGenome from file
%     (seedingGenome.mat) that must be available from the same directory as
%     evolve.m. In mode 2, maxGenomes/2 are defined according to the first
%     two columns of seedingGenome. Another maxGenomes/2 are defined as
%     different random genomes according to the structural input settings.
% 16: deathmatch mode 3, starting with read-in of seedingGenome from file
%     (seedingGenome.mat) that must be available from the same directory as
%     evolve.m. In mode 3, maxGenomes/2 are defined according to the first
%     two columns of seedingGenome. Another maxGenomes/2 are defined as
%     identical random genomes according to the structural input settings.
% 17: deathmatch mode 4. In mode 4, maxGenomes/2 are defined as different
%     random genomes ccording to the structural input settings.
%     Additionally, a seeding knockout fraction is applied according to the
%     first parameter in seedingKnockout. Another maxGenomes/2 are defined
%     as different random genomes according to the structural input
%     settings, but with a seeding knockout fraction according to the
%     second parameter in seedingKnockout.
% 18: deathmatch mode 5. In mode 5, maxGenomes/2 are defined as different
%     random genomes according to the structural input settings. In every
%     second haploid all essential genes are knocked out. Another
%     maxGenomes/2 are defined as different random genomes according to the
%     structural input settings, but without any knockouts.
% 19: deathmatch mode 6. In mode 6, maxGenomes are defined as different
%     random genomes according to the structural input settings. In every
%     second haploid all essential genes are knocked out.
% 20: deathmatch mode 7. In mode 5, maxGenomes/2 are defined as identical
%     random genomes according to the structural input settings. In every
%     second haploid all essential genes are knocked out. Another
%     maxGenomes/2 are defined as identical random genomes according to the
%     structural input settings, but without any knockouts.
% 21: deathmatch mode 8. In mode 6, maxGenomes/2 + maxGenomes/2 are defined
%     as identical random genomes according to the structural input
%     settings. In every second haploid all essential genes are knocked
%     out.
% -------------------------------------------------------------------------

% deathmatchWinner notes:
% column 1: 0 for success of first population
%           1 for success of second/random population
%           2 for survival of both populations
%           3 for extinction of both populations before end of simulation
% column 2: time point (generation) of deathmatch evaluation;
%           this will be totalSimulationSteps in case of scenario 2
% column 3: # of haploids in the population at the time point of deathmatch
%           evaluation
% column 4: # of type 1 haploids in the population at the time point of
%           deathmatch evaluation
% column 5: # of type 2 haploids in the population at the time point of
%           deathmatch evaluation

% --------------------------------------
% | Fixed parameters of the simulation |
% --------------------------------------

totalSimulationSteps = totalSimulationStepsInput;

% Note: totalSimulationSteps should be a multiple of protocolSplittingSize

nGenes = nGenesInput;
nRPIE = nRPIEInput;

nIntergenicElements = nGenes - 1;
nGenomicElements = 2 * nGenes - 1;
nEssentialGenes = nEssentialGenesInput;
nNonEssentialGenes = nGenes - nEssentialGenes;
essentialFraction = nEssentialGenesInput / nGenes;

maxGenomes = initialOrMaxGenomesInput;
        
doubleRecombinationSiteSelectionAdvantage = 2;

% Note: doubleRecombinationSiteSelectionAdvantage is the statistical
%       selection factor in comparison to single recombination sites

inbreedingFlag = length(inbreedingFractionInput);
inbreedingFraction = inbreedingFractionInput;
if inbreedingFlag == 1
    if inbreedingFraction == 1
        breedingSituation = 1;
    else
        breedingSituation = 0;
    end;
elseif inbreedingFlag == 2
    if inbreedingFraction(1, 1) == 1 && inbreedingFraction(1, 2) == 1
        breedingSituation = 1;
    else
        breedingSituation = 0;
    end;
end;
requiredHomologousFraction = requiredHomologousFractionInput;

rearrangementMode = rearrangementModeInput;

% Note: rearrangementMode settings are 0 for swapping and 1 for flipping,
%       flipping is only implemented for inbreeding-only situations

matingLocusFlag = matingLocusFlagInput;
matingLocusPosition = matingLocusPositionInput;
matingLocusPositionFlag = length(matingLocusPosition);
if matingLocusPositionFlag == 2
    matingLocusPosition1 = matingLocusPosition(1, 1);
    matingLocusPosition2 = matingLocusPosition(1, 2);
end;
externalLocus = externalLocusInput;
matingSwitching = matingSwitchingInput;
matingSwitchingFlag = length(matingSwitching);
if matingSwitchingFlag == 2
	matingSwitching1 = matingSwitching(1, 1);
	matingSwitching2 = matingSwitching(1, 2);
end;

skipMeiosis = skipMeiosisInput;
randomizeCycles = randomizeCyclesInput;
mutationApplication = mutationApplicationInput;
mutationMode = mutationModeInput; % 0 for mitosis only, 1 for meiosis only, 2 for both
mutationModeFlag = length(mutationMode);
if mutationModeFlag == 2
	mutationMode1 = mutationMode(1, 1);
	mutationMode2 = mutationMode(1, 2);
end;
subPopulationFlag = subPopulationFlagInput;
displayInfo = displayInfoInput;

printoutSeedingGenome = 0;
saveASCIIFiles = 0;
printoutMitoticGenome = 0;
printoutMeioticGenome = 0;
createGenomicDatabase = 1;
minimalDiskSpaceMode = 1;
silentIO = silentIOInput; % 0 for standard protocolling, 1 for no protocolling, 2 for genome snapshots only

analyzeClustering = analyzeClusteringInput;
clusterAnalysisPower = 2;
visualizeProtocols = 1;

recombinationSites = 2 .* (1:(nGenes - 1))';
geneSites = 2 .* (1:(nGenes))' - 1;

minClusteringValue = 0;
maxClusteringValue = nEssentialGenes * nRPIE;

fullFitnessIdentifier = 1000;
noFitnessIdentifier = 2000;
matingLocusIdentifier = 3000;

recombinationMode = recombinationModeInput;
recombinationModeFlag = length(recombinationModeInput);
if recombinationModeFlag == 2
    recombinationMode1 = recombinationMode(1, 1);
    recombinationMode2 = recombinationMode(1, 2);
end;
recombinationUnitFactor = recombinationUnitFactorInput;
recombinationUnitFactorFlag = length(recombinationUnitFactor);
recombinationProbability = recombinationProbabilityInput;
recombinationProbabilityFlag = length(recombinationProbability);
if recombinationProbabilityFlag == 2
    recombinationProbability1 = recombinationProbability(1, 1);
    recombinationProbability2 = recombinationProbability(1, 2);
else
    recombinationProbability1 = recombinationProbability;
    recombinationProbability2 = recombinationProbability;
end;
erlangApplicationFlag = erlangApplicationFlagInput;
randomApplicationFlag = randomApplicationFlagInput;

% Notes:
% 1) recombinationMode settings are 0 for Erlang mode and 1 for random mode
% 2) erlangApplicationFlag settings are 0 for physical distance and 1 for
%    genetic distance
% 3) randomApplicationFlag settings are 0 for Erlang-based event comupation
%    and 1 for fixed formula-based computation

% --- feedback parameters ---
adaptiveMutationRate = adaptiveMutationRateInput;
feedbackPopulationSize = feedbackPopulationSizeInput;
feedbackPopulationSizeDerivative = feedbackPopulationSizeDerivativeInput;
feedbackDerivativeDistance = derivativeDistanceInput;
relativeFeedback = relativeFeedbackFlagInput;

populationCapSize = populationCapSizeInput;
derivativeFeedbackCap = 0.1;

protocolSplittingSize = protocolSplittingSizeInput;
databaseResolution = databaseResolutionInput;

% Note: protocolSplittingSize must be an even number

slidingWindowSize = 10;
slidingWindowPlotSize = 20;
if totalSimulationSteps >= 100
    parameterAveragingRange = 99;
else
    parameterAveragingRange = 9;
end;

% -----------------------------------
% | Setting-up of global parameters |
% -----------------------------------

if performSurvivalScreen == 2
    verticalClass = 'seedingsKnockout class';
    horizontalClass = 'clusterFrequency class';
    verticalParameters = seedingKnockoutArrayLength;
    horizontalParameters = clusterFrequencyArrayLength;
else
    verticalClass = 'mitoticMutationRate class';
    horizontalClass = 'mitoticRearrangementRate class';
end;

statisticalAnalysisMatrix = zeros(verticalParameters + 1, horizontalParameters + 1, statisticalAveraging);
statisticalAnalysisMatrixUnfitted = zeros(verticalParameters + 1, horizontalParameters + 1, statisticalAveraging);
statisticalAnalysisRecombinationSites = zeros(verticalParameters + 1, horizontalParameters + 1, statisticalAveraging);
statisticalAnalysisFunctionalGenes = zeros(verticalParameters + 1, horizontalParameters + 1, statisticalAveraging);
statisticalAnalysisGrouping = zeros(verticalParameters + 1, horizontalParameters + 1, statisticalAveraging);

if fittingScheme == 0
    statisticalClusterAnalysisResults = zeros(verticalParameters * horizontalParameters, 9, statisticalAveraging);
elseif fittingScheme == 1
    statisticalClusterAnalysisResults = zeros(verticalParameters * horizontalParameters, 12, statisticalAveraging);
end;

statisticalSlidingWindowResults = zeros(2 * verticalParameters * horizontalParameters, 13, statisticalAveraging);
statisticalSlidingWindowDeviationResults = zeros(2 * verticalParameters * horizontalParameters, 13, statisticalAveraging);

slidingWindowEssentialGenesHistogramRandomCase = ...
    [0.104874847250509,    0.268282790224033,    0.305364256619145,...
     0.202902342158860,    0.0873153767820774,   0.0254066191446029,...
     0.00510091649694501,  0.000693584521384929, 5.71283095723014e-05,...
     2.13849287169043e-06, 0;];

slidingWindowRecombinationSitesHistogramRandomCase = ...
    [0.0168515306122449,   0.0859145918367347,  0.195891122448980,...
     0.262351326530612,    0.228961224489796,   0.136103571428571,...
     0.0554992857142857,   0.0153715306122449,  0.00275346938775510,...
     0.000289081632653061, 1.32653061224490e-05;];
 
% -----------------------------------
% | Assembly of Erlang distribution |
% -----------------------------------

% -------------------------------------------------------------------------
% Notes:
% 1) Maximum of Erlang distribution with shape factor 4 is at 3/8
% 2) The maximum is supposed to be located at 50 centi-Morgans
% 3) 1 Morgan equals the genomic distance that covers
%    (nGenes - 1) * 12/90 intergenic elements (physical implementation)
%    (nRPIE - 1) * 12/90 intergenic elements (genetical implementation)
% 4) The distribution unit therefore is 0.5 * 6/8 * 90/12 * 1/(nGenes - 1)
%    or 0.5 * 6/8 * 90/12 * 1/nRPIE respectively
% 5) The probability of an event taking place at IE site x therefore equals
%    the integral over the Erlang distribution from x * unit to
%    (x + 1) * unit.
% 6) A recombinationUnitFactor > 1 will in average decrease the distance
%    between two adjacent recombination events, while
%    recombinationUnitFactor < 1 will increase the distance.
% -------------------------------------------------------------------------

numericalCorrectionFactor = numericalCorrectionFactorInput;

if recombinationUnitFactorFlag == 1
    
    if erlangApplicationFlag == 0
        erlangUnit = recombinationUnitFactor * numericalCorrectionFactor * 6/8 * 90/12 * 1/(nGenes - 1);
    else
        erlangUnit = recombinationUnitFactor * numericalCorrectionFactor * 6/8 * 90/12 * 1/nRPIE;
    end;

    if erlangUnit > 0
        erlangDistribution = zeros(nGenes, 1);
        for n = 1:(nGenes - 1)
            erlangDistribution(n, 1) = quad(@erlang, (n - 1) * erlangUnit, n * erlangUnit);
        end;
        erlangDistribution(nGenes, 1) = 1 - sum(erlangDistribution(1:(nGenes - 1), 1));
        erlangIntervals = zeros(nGenes, 1);
        for n = 1:(nGenes)
            erlangIntervals(n, 1) = sum(erlangDistribution(1:n, 1));
        end;
        mostLikelyDistance = sum(erlangDistribution(:, 1) .* (1:nGenes)') ./ sum(erlangDistribution(:, 1));
    elseif erlangUnit == 0
        erlangIntervals = zeros(nGenes, 1);
        erlangIntervals(nGenes, 1) = 1;
        mostLikelyDistance = Inf;
    elseif recombinationUnitFactor == -1
        erlangIntervals = ones(nGenes, 1);
        mostLikelyDistance = 1;
    end;

    if erlangApplicationFlag == 0
        mostLikelyEventNumber = (nGenes - 1) / mostLikelyDistance;
    else
        mostLikelyEventNumber = nRPIE / mostLikelyDistance;
    end;
    
elseif recombinationUnitFactorFlag == 2
    
    if erlangApplicationFlag == 0
        erlangUnit1 = recombinationUnitFactor(1, 1) * numericalCorrectionFactor * 6/8 * 90/12 * 1/(nGenes - 1);
        erlangUnit2 = recombinationUnitFactor(1, 2) * numericalCorrectionFactor * 6/8 * 90/12 * 1/(nGenes - 1);
    else
        erlangUnit1 = recombinationUnitFactor(1, 1) * numericalCorrectionFactor * 6/8 * 90/12 * 1/nRPIE;
        erlangUnit2 = recombinationUnitFactor(1, 2) * numericalCorrectionFactor * 6/8 * 90/12 * 1/nRPIE;
    end;

    if erlangUnit1 > 0
        erlangDistribution1 = zeros(nGenes, 1);
        for n = 1:(nGenes - 1)
            erlangDistribution1(n, 1) = quad(@erlang, (n - 1) * erlangUnit1, n * erlangUnit1);
        end;
        erlangDistribution1(nGenes, 1) = 1 - sum(erlangDistribution1(1:(nGenes - 1), 1));
        erlangIntervals1 = zeros(nGenes, 1);
        for n = 1:(nGenes)
            erlangIntervals1(n, 1) = sum(erlangDistribution1(1:n, 1));
        end;
        mostLikelyDistance1 = sum(erlangDistribution1(:, 1) .* (1:nGenes)') ./ sum(erlangDistribution1(:, 1));
    elseif erlangUnit1 == 0
        erlangIntervals1 = zeros(nGenes, 1);
        erlangIntervals1(nGenes, 1) = 1;
        mostLikelyDistance1 = Inf;
    elseif recombinationUnitFactor(1, 1) == -1
        erlangIntervals1 = ones(nGenes, 1);
        mostLikelyDistance1 = 1;
    end;
    
    if erlangUnit2 > 0
        erlangDistribution2 = zeros(nGenes, 1);
        for n = 1:(nGenes - 1)
            erlangDistribution2(n, 1) = quad(@erlang, (n - 1) * erlangUnit2, n * erlangUnit2);
        end;
        erlangDistribution2(nGenes, 1) = 1 - sum(erlangDistribution2(1:(nGenes - 1), 1));
        erlangIntervals2 = zeros(nGenes, 1);
        for n = 1:(nGenes)
            erlangIntervals2(n, 1) = sum(erlangDistribution2(1:n, 1));
        end;
        mostLikelyDistance2 = sum(erlangDistribution2(:, 1) .* (1:nGenes)') ./ sum(erlangDistribution2(:, 1));
    elseif erlangUnit2 == 0
        erlangIntervals2 = zeros(nGenes, 1);
        erlangIntervals2(nGenes, 1) = 1;
        mostLikelyDistance2 = Inf;
    elseif recombinationUnitFactor(1, 2) == -1
        erlangIntervals2 = ones(nGenes, 1);
        mostLikelyDistance2 = 1;
    end;

    if erlangApplicationFlag == 0
        mostLikelyEventNumber1 = (nGenes - 1) / mostLikelyDistance1;
    else
        mostLikelyEventNumber1 = nRPIE / mostLikelyDistance1;
    end;
    
    if erlangApplicationFlag == 0
        mostLikelyEventNumber2 = (nGenes - 1) / mostLikelyDistance2;
    else
        mostLikelyEventNumber2 = nRPIE / mostLikelyDistance2;
    end;
end;

% ----------------------------------------
% | Generation of the initial population |
% ----------------------------------------

if (seedingSetup ~= 0) && (seedingSetup ~= 4) && (seedingSetup ~= 5)
    
    % Definition of the unclustered and the clustered genome
    % Attention: Does only work with certain values of
    %            nGenes, nEssentialGenes, nRPIE
    
    unclusteredGenome = zeros(nGenomicElements, 2, 'uint16');
    clusteredGenome = zeros(nGenomicElements, 2, 'uint16');

    for n = 1:nGenes
        unclusteredGenome(2 * n - 1, 1:2) = 2;
        clusteredGenome(2 * n - 1, 1:2) = 2;
    end;

    for n = 1:nRPIE
        unclusteredGenome(6 * n, 1:2) = 1;
    end;

    for n = 1:(nEssentialGenes / 2)
        unclusteredGenome(6 * (3 * n - 2 + 8) + 3, 1:2) = 2 * n - 1 + fullFitnessIdentifier;
        unclusteredGenome(6 * (3 * n - 1 + 8) + 3, 1:2) = 2 * n + fullFitnessIdentifier;
    end;

    for n = 1:(nRPIE / 2)
        clusteredGenome(2 * n, 1:2) = 1;
        clusteredGenome(nGenomicElements - (2 * n - 1), 1:2) = 1;
    end;

    for n = 1:nEssentialGenes
        clusteredGenome(nGenes - 2 - (2 * nEssentialGenes / 2) + 2 * n - 1, 1:2) = n + fullFitnessIdentifier;
    end;
    
end;

if seedingSetup == 0
    
    seedingGenome = zeros(nGenomicElements, 2, 'uint16');

    for n = 1:nGenes
        seedingGenome(2 * n - 1, 1) = 2;
        seedingGenome(2 * n - 1, 2) = 2;
    end;

    nonAssignedEssentialGenes = nEssentialGenes;
    while nonAssignedEssentialGenes > 0
        testSite = ceil(nGenes .* rand([1 1]));
        if seedingGenome(2 * testSite - 1, 1) == 2
            seedingGenome(2 * testSite - 1, 1) = fullFitnessIdentifier + nonAssignedEssentialGenes;
            seedingGenome(2 * testSite - 1, 2) = fullFitnessIdentifier + nonAssignedEssentialGenes;
            nonAssignedEssentialGenes = nonAssignedEssentialGenes - 1;
        end;
    end;

    nonAssignedProficientRecombinationSites = nRPIE;
    while nonAssignedProficientRecombinationSites > 0
        testSite = ceil((nGenes - 1) .* rand([1 1]));
        if seedingGenome(2 * testSite, 1) == 0
            seedingGenome(2 * testSite, 1) = 1;
            seedingGenome(2 * testSite, 2) = 1;
            nonAssignedProficientRecombinationSites = nonAssignedProficientRecombinationSites - 1;
        end;
    end;
    
elseif seedingSetup == 4
    
    seedingGenome = zeros(nGenomicElements, 2 * maxGenomes, 'uint16');
    
    for n = 1:nGenes
        seedingGenome(2 * n - 1, :) = 2;
    end;
    
    for n = 1:maxGenomes
        nonAssignedEssentialGenes = nEssentialGenes;
        while nonAssignedEssentialGenes > 0
            testSite = ceil(nGenes .* rand([1 1]));
            if seedingGenome(2 * testSite - 1, 2 * n - 1) == 2
                seedingGenome(2 * testSite - 1, (2 * n - 1):(2 * n)) = fullFitnessIdentifier + nonAssignedEssentialGenes;
                nonAssignedEssentialGenes = nonAssignedEssentialGenes - 1;
            end;
        end;
    end;
    
    for n = 1:maxGenomes
        nonAssignedProficientRecombinationSites = nRPIE;
        while nonAssignedProficientRecombinationSites > 0
            testSite = ceil((nGenes - 1) .* rand([1 1]));
            if seedingGenome(2 * testSite, 2 * n - 1) == 0
                seedingGenome(2 * testSite, (2 * n - 1):(2 * n)) = 1;
                nonAssignedProficientRecombinationSites = nonAssignedProficientRecombinationSites - 1;
            end;
        end;
    end;

elseif seedingSetup == 5
    
    seedingGenome = zeros(nGenomicElements, 2 * maxGenomes, 'uint16');
    
    for n = 1:nGenes
        seedingGenome(2 * n - 1, :) = 2;
    end;
    
    nonAssignedEssentialGenes = nEssentialGenes;
    while nonAssignedEssentialGenes > 0
    	testSite = ceil(nGenes .* rand([1 1]));
        if seedingGenome(2 * testSite - 1, 1) == 2
            seedingGenome(2 * testSite - 1, :) = fullFitnessIdentifier + nonAssignedEssentialGenes;
            nonAssignedEssentialGenes = nonAssignedEssentialGenes - 1;
        end;
    end;
    
    
    nonAssignedProficientRecombinationSites = nRPIE;
    while nonAssignedProficientRecombinationSites > 0
    	testSite = ceil((nGenes - 1) .* rand([1 1]));
        if seedingGenome(2 * testSite, 1) == 0
        	seedingGenome(2 * testSite, :) = 1;
            nonAssignedProficientRecombinationSites = nonAssignedProficientRecombinationSites - 1;
        end;
    end;
    
elseif seedingSetup == 1
    
    seedingGenome = unclusteredGenome;
    
elseif seedingSetup == 2
    
    seedingGenome = clusteredGenome;
    
elseif seedingSetup == 3
    
    seedingGenome = cat(2, unclusteredGenome, clusteredGenome);
    
elseif seedingSetup == 6
    
    seedingGenome = zeros(nGenomicElements, 2 * maxGenomes, 'uint16');
    for n = 1:maxGenomes
        seedingGenome(:, (2 * n - 1):(2 * n)) = unclusteredGenome;
    end;
    
elseif seedingSetup == 7
    
    seedingGenome = zeros(nGenomicElements, 2 * maxGenomes, 'uint16');
    for n = 1:maxGenomes
        seedingGenome(:, (2 * n - 1):(2 * n)) = clusteredGenome;
    end;
    
elseif seedingSetup == 8
    
    seedingGenome = zeros(nGenomicElements, 2 * maxGenomes, 'uint16');
    for n = 1:(maxGenomes / 2)
        seedingGenome(:, (2 * n - 1):(2 * n)) = unclusteredGenome;
        seedingGenome(:, ((2 * n - 1):(2 * n)) + 2 * maxGenomes / 2) = clusteredGenome;
    end;

elseif seedingSetup == 9
    
    nEssentialGeneKnockouts = seedingKnockout * nEssentialGenes;
    seedingGenome = zeros(nGenomicElements, 2 * maxGenomes, 'uint16');
    seedingFitness = ones(nEssentialGenes, 2 * maxGenomes, 'uint8');
    
    for n = 1:maxGenomes
        seedingGenome(:, (2 * n - 1):(2 * n)) = unclusteredGenome;
        knockoutAvailabilityList = (1:nEssentialGenes)';
        for m = 1:(2 * nEssentialGeneKnockouts)
            currentKnockoutItem = ceil((nEssentialGenes - m + 1) .* rand([1 1]));
            currentKnockout = knockoutAvailabilityList(currentKnockoutItem, 1);
            knockoutAvailabilityList(currentKnockoutItem, :) = [];
            if mod(currentKnockout, 2) == 0
                currentKnockoutPosition = 6 * (3 * currentKnockout / 2 - 1 + 8) + 3;
            else
                currentKnockoutPosition = 6 * (3 * (currentKnockout + 1) / 2 - 2 + 8) + 3;
            end;
            if m <= nEssentialGeneKnockouts
                seedingGenome(currentKnockoutPosition, 2 * n - 1) = noFitnessIdentifier + currentKnockout;
                seedingFitness(currentKnockout,  2 * n - 1) = 0;
            else
                seedingGenome(currentKnockoutPosition, 2 * n) = noFitnessIdentifier + currentKnockout;
                seedingFitness(currentKnockout,  2 * n) = 0;
            end;
        end;
    end;

elseif seedingSetup == 10
    
    nEssentialGeneKnockouts = seedingKnockout * nEssentialGenes;
    seedingGenome = zeros(nGenomicElements, 2 * maxGenomes, 'uint16');
    seedingFitness = ones(nEssentialGenes, 2 * maxGenomes, 'uint8');
    
    for n = 1:maxGenomes
        seedingGenome(:, (2 * n - 1):(2 * n)) = clusteredGenome;
        knockoutAvailabilityList = (1:nEssentialGenes)';
        for m = 1:(2 * nEssentialGeneKnockouts)
            currentKnockoutItem = ceil((nEssentialGenes - m + 1) .* rand([1 1]));
            currentKnockout = knockoutAvailabilityList(currentKnockoutItem, 1);
            knockoutAvailabilityList(currentKnockoutItem, :) = [];
            currentKnockoutPosition = nGenes - 2 - (2 * nEssentialGenes / 2) + 2 * currentKnockout - 1;
            if m <= nEssentialGeneKnockouts
                seedingGenome(currentKnockoutPosition, 2 * n - 1) = noFitnessIdentifier + currentKnockout;
                seedingFitness(currentKnockout,  2 * n - 1) = 0;
            else
                seedingGenome(currentKnockoutPosition, 2 * n) = noFitnessIdentifier + currentKnockout;
                seedingFitness(currentKnockout,  2 * n) = 0;
            end;
        end;
    end;
    
elseif seedingSetup == 11
    
    if ~clusterMode
        load seedingGenome;
    else
        load([seedingDirectory '/seedingGenome.mat']);
    end;
    nSeedingGenomes = size(seedingGenome, 2) / 2;
    seedingFitness = ones(nEssentialGenes, 2 * nSeedingGenomes, 'uint8');
    for n = 1:(2 * nSeedingGenomes)
        currentChromosome = seedingGenome(:, n);
        for m = 1:nEssentialGenes
           if isempty(find(currentChromosome == (fullFitnessIdentifier + m), 1))
               seedingFitness(m, n) = 0;
           end;
        end;
    end;
    
elseif seedingSetup == 12
    
    nGenePackSize = floor(nEssentialGenes / clusterFrequency);
    nRPIEPackSize = floor(nRPIE / clusterFrequency);
    
    remainingGenes = nEssentialGenes - clusterFrequency * nGenePackSize;
    remainingRPIEs = nRPIE - clusterFrequency * nRPIEPackSize;
    
    genesToDistribute = nEssentialGenes;
    RPIEsToDistribute = nRPIE;
    
    seedingGenome = zeros(nGenomicElements, 2 * maxGenomes, 'uint16');
    seedingFitness = ones(nEssentialGenes, 2 * maxGenomes, 'uint8');
    
    for k = 1:nGenes
        seedingGenome(2 * k - 1, :) = 2;
    end;
    
    geneAbortFlag = 0;
    RPIEAbortFlag = 0;
    
    for k = 1:clusterFrequency
        
        % the gene cluster starts at currentClusterStart
        
        currentClusterStart = floor(1 + nGenomicElements * (k - 1) / clusterFrequency);
        if mod(currentClusterStart, 2) == 0
            currentClusterStart = currentClusterStart + 1;
        end;
        
        for m = 1:nGenePackSize
            seedingGenome(currentClusterStart + 2 * (m - 1), :) = genesToDistribute + fullFitnessIdentifier;
            genesToDistribute = genesToDistribute - 1;
            if genesToDistribute == 0
                geneAbortFlag = 1;
                break;
            end;
        end;
        
        if geneAbortFlag == 1
            break;
        end;
        
        if remainingGenes > 0
            seedingGenome(currentClusterStart + 2 * nGenePackSize, :) = genesToDistribute + fullFitnessIdentifier;
            genesToDistribute = genesToDistribute - 1;
            remainingGenes = remainingGenes - 1;
            if genesToDistribute == 0
                geneAbortFlag = 1;
                break;
            end;
        end;
        
        if geneAbortFlag == 1
            break;
        end;
        
    end;
    
    for k = 1:clusterFrequency
        
        % the RPIE cluster starts at currentClusterCenter
        
        currentClusterStart = floor(1 + nGenomicElements * (k - 1) / clusterFrequency);
        currentClusterEnd = floor(1 + nGenomicElements * k / clusterFrequency) - 1;
        currentClusterCenter = round((currentClusterEnd + currentClusterStart) / 2);
        if mod(currentClusterStart, 2) == 0
            currentClusterStart = currentClusterStart + 1;
        end;
        if mod(currentClusterCenter, 2) == 1
            currentClusterCenter = currentClusterCenter + 1;
        end;
        
        for m = 1:nRPIEPackSize
            seedingGenome(currentClusterCenter + 2 * (m - 1), :) = 1;
            RPIEsToDistribute = RPIEsToDistribute - 1;
            if RPIEsToDistribute == 0
                RPIEAbortFlag = 1;
                break;
            end;
        end;
        
        if RPIEAbortFlag == 1
            break;
        end;
        
        if remainingRPIEs > 0
            seedingGenome(currentClusterCenter + 2 * nRPIEPackSize, :) = 1;
            RPIEsToDistribute = RPIEsToDistribute - 1;
            remainingRPIEs = remainingRPIEs - 1;
            if RPIEsToDistribute == 0
                RPIEAbortFlag = 1;
                break;
            end;
        end;
        
        if RPIEAbortFlag == 1
            break;
        end;
        
    end;

elseif (seedingSetup == 13) && (performSurvivalScreen ~= 2)
    
    nGenePackSize = floor(nEssentialGenes / clusterFrequency);
    nRPIEPackSize = floor(nRPIE / clusterFrequency);
    
    remainingGenes = nEssentialGenes - clusterFrequency * nGenePackSize;
    remainingRPIEs = nRPIE - clusterFrequency * nRPIEPackSize;
    
    genesToDistribute = nEssentialGenes;
    RPIEsToDistribute = nRPIE;
    
    seedingGenome = zeros(nGenomicElements, 2 * maxGenomes, 'uint16');
    seedingFitness = ones(nEssentialGenes, 2 * maxGenomes, 'uint8');
    
    for k = 1:nGenes
        seedingGenome(2 * k - 1, :) = 2;
    end;
    
    nEssentialGeneKnockouts = 2 * seedingKnockout * nEssentialGenes;
    
    if nEssentialGeneKnockouts > 0
        for k = 1:maxGenomes
            availableGenes = (1:nEssentialGenes)';
            nAvailableGenes = nEssentialGenes;
            knockoutsToDistribute = nEssentialGeneKnockouts;
            while knockoutsToDistribute > 0
                currentKnockout = ceil(nAvailableGenes .* rand([1 1]));
                seedingFitness(availableGenes(currentKnockout, 1), 2 * (k - 1) + ceil(2 .* rand([1 1]))) = 0;
                availableGenes(currentKnockout, :) = [];
                nAvailableGenes = nAvailableGenes - 1;
                knockoutsToDistribute = knockoutsToDistribute - 1;
            end;
        end;
    end;        
    
    geneAbortFlag = 0;
    RPIEAbortFlag = 0;
    
    for k = 1:clusterFrequency
        
        % the gene cluster starts at currentClusterStart
        
        currentClusterStart = floor(1 + nGenomicElements * (k - 1) / clusterFrequency);
        if mod(currentClusterStart, 2) == 0
            currentClusterStart = currentClusterStart + 1;
        end;
        
        for m = 1:nGenePackSize
            for p = 1:(2 * maxGenomes)
                if seedingFitness(genesToDistribute, p) == 1
                    seedingGenome(currentClusterStart + 2 * (m - 1), p) = genesToDistribute + fullFitnessIdentifier;
                else
                    seedingGenome(currentClusterStart + 2 * (m - 1), p) = genesToDistribute + noFitnessIdentifier;
                end;
            end;
            genesToDistribute = genesToDistribute - 1;
            if genesToDistribute == 0
                geneAbortFlag = 1;
                break;
            end;
        end;
        
        if geneAbortFlag == 1
            break;
        end;
        
        if remainingGenes > 0
            seedingGenome(currentClusterStart + 2 * nGenePackSize, :) = genesToDistribute + fullFitnessIdentifier;
            genesToDistribute = genesToDistribute - 1;
            remainingGenes = remainingGenes - 1;
            if genesToDistribute == 0
                geneAbortFlag = 1;
                break;
            end;
        end;
        
        if geneAbortFlag == 1
            break;
        end;
        
    end;
    
    for k = 1:clusterFrequency
        
        % the RPIE cluster starts at currentClusterCenter
        
        currentClusterStart = floor(1 + nGenomicElements * (k - 1) / clusterFrequency);
        currentClusterEnd = floor(1 + nGenomicElements * k / clusterFrequency) - 1;
        currentClusterCenter = round((currentClusterEnd + currentClusterStart) / 2);
        if mod(currentClusterStart, 2) == 0
            currentClusterStart = currentClusterStart + 1;
        end;
        if mod(currentClusterCenter, 2) == 1
            currentClusterCenter = currentClusterCenter + 1;
        end;
        
        for m = 1:nRPIEPackSize
            seedingGenome(currentClusterCenter + 2 * (m - 1), :) = 1;
            RPIEsToDistribute = RPIEsToDistribute - 1;
            if RPIEsToDistribute == 0
                RPIEAbortFlag = 1;
                break;
            end;
        end;
        
        if RPIEAbortFlag == 1
            break;
        end;
        
        if remainingRPIEs > 0
            seedingGenome(currentClusterCenter + 2 * nRPIEPackSize, :) = 1;
            RPIEsToDistribute = RPIEsToDistribute - 1;
            remainingRPIEs = remainingRPIEs - 1;
            if RPIEsToDistribute == 0
                RPIEAbortFlag = 1;
                break;
            end;
        end;
        
        if RPIEAbortFlag == 1
            break;
        end;
        
    end;
    
elseif seedingSetup == 14
    
    if ~clusterMode
        load seedingGenome;
    else
        load([seedingDirectory '/seedingGenome.mat']);
    end;
    
    backupGenome = seedingGenome;
    
    seedingGenome = zeros(nGenomicElements, 2 * maxGenomes, 'uint16');
    
    for i = 1:(maxGenomes / 2)
        seedingGenome(:, (2 * i - 1):(2 * i)) = backupGenome(:, 1:2);
        seedingGenome(:, (2 * i - 1 + maxGenomes):(2 * i + maxGenomes)) = ...
            backupGenome(:, 3:4);
    end;
    
    [dummy nSeedingGenomes] = size(seedingGenome);
    nSeedingGenomes = nSeedingGenomes / 2;
    seedingFitness = ones(nEssentialGenes, 2 * nSeedingGenomes, 'uint8');
    for n = 1:(2 * nSeedingGenomes)
        currentChromosome = seedingGenome(:, n);
        for m = 1:nEssentialGenes
           if isempty(find(currentChromosome == (fullFitnessIdentifier + m), 1))
               seedingFitness(m, n) = 0;
           end;
        end;
    end;
    
    populationIdentitiesMaster = ones(1, 2 * nSeedingGenomes, 'uint8');
    populationIdentitiesMaster(1, 1:maxGenomes) = 0;
    primaryTemplate = mod(seedingGenome(:, 1), fullFitnessIdentifier);
    
elseif seedingSetup == 15
    
    if ~clusterMode
        load seedingGenome;
    else
        load([seedingDirectory '/seedingGenome.mat']);
    end;
    
    backupGenome = seedingGenome;
    
    seedingGenome = zeros(nGenomicElements, 2 * maxGenomes, 'uint16');
    
    for i = 1:(maxGenomes / 2)
        seedingGenome(:, (2 * i - 1):(2 * i)) = backupGenome(:, 1:2);
    end;
    
    for n = 1:nGenes
        seedingGenome(2 * n - 1, (maxGenomes + 1):(2 * maxGenomes)) = 2;
    end;
    
    for n = (maxGenomes / 2 + 1):maxGenomes
        nonAssignedEssentialGenes = nEssentialGenes;
        while nonAssignedEssentialGenes > 0
            testSite = ceil(nGenes .* rand([1 1]));
            if seedingGenome(2 * testSite - 1, 2 * n - 1) == 2
                seedingGenome(2 * testSite - 1, (2 * n - 1):(2 * n)) = fullFitnessIdentifier + nonAssignedEssentialGenes;
                nonAssignedEssentialGenes = nonAssignedEssentialGenes - 1;
            end;
        end;
    end;
    
    for n = (maxGenomes / 2 + 1):maxGenomes
        nonAssignedProficientRecombinationSites = nRPIE;
        while nonAssignedProficientRecombinationSites > 0
            testSite = ceil((nGenes - 1) .* rand([1 1]));
            if seedingGenome(2 * testSite, 2 * n - 1) == 0
                seedingGenome(2 * testSite, (2 * n - 1):(2 * n)) = 1;
                nonAssignedProficientRecombinationSites = nonAssignedProficientRecombinationSites - 1;
            end;
        end;
    end;
    
    [dummy nSeedingGenomes] = size(seedingGenome);
    nSeedingGenomes = nSeedingGenomes / 2;
    seedingFitness = ones(nEssentialGenes, 2 * nSeedingGenomes, 'uint8');
    for n = 1:(2 * nSeedingGenomes)
        currentChromosome = seedingGenome(:, n);
        for m = 1:nEssentialGenes
           if isempty(find(currentChromosome == (fullFitnessIdentifier + m), 1))
               seedingFitness(m, n) = 0;
           end;
        end;
    end;
    
    populationIdentitiesMaster = ones(1, 2 * nSeedingGenomes, 'uint8');
    populationIdentitiesMaster(1, 1:maxGenomes) = 0;
    primaryTemplate = mod(seedingGenome(:, 1), fullFitnessIdentifier);
    
elseif seedingSetup == 16
    
    if ~clusterMode
        load seedingGenome;
    else
        load([seedingDirectory '/seedingGenome.mat']);
    end;
    
    backupGenome = seedingGenome;
    
    seedingGenome = zeros(nGenomicElements, 2 * maxGenomes, 'uint16');
    
    for i = 1:(maxGenomes / 2)
        seedingGenome(:, (2 * i - 1):(2 * i)) = backupGenome(:, 1:2);
    end;
    
    for n = 1:nGenes
        seedingGenome(2 * n - 1, (maxGenomes + 1):(2 * maxGenomes)) = 2;
    end;
    
    nonAssignedEssentialGenes = nEssentialGenes;
    while nonAssignedEssentialGenes > 0
        testSite = ceil(nGenes .* rand([1 1]));
        if seedingGenome(2 * testSite - 1, maxGenomes + 1) == 2
            seedingGenome(2 * testSite - 1, (maxGenomes + 1):(2 * maxGenomes)) = ...
                fullFitnessIdentifier + nonAssignedEssentialGenes;
            nonAssignedEssentialGenes = nonAssignedEssentialGenes - 1;
        end;
    end;
    
    nonAssignedProficientRecombinationSites = nRPIE;
    while nonAssignedProficientRecombinationSites > 0
        testSite = ceil((nGenes - 1) .* rand([1 1]));
        if seedingGenome(2 * testSite, maxGenomes + 1) == 0
            seedingGenome(2 * testSite, (maxGenomes + 1):(2 * maxGenomes)) = 1;
            nonAssignedProficientRecombinationSites = nonAssignedProficientRecombinationSites - 1;
        end;
    end;
    
    [dummy nSeedingGenomes] = size(seedingGenome);
    nSeedingGenomes = nSeedingGenomes / 2;
    seedingFitness = ones(nEssentialGenes, 2 * nSeedingGenomes, 'uint8');
    for n = 1:(2 * nSeedingGenomes)
        currentChromosome = seedingGenome(:, n);
        for m = 1:nEssentialGenes
           if isempty(find(currentChromosome == (fullFitnessIdentifier + m), 1))
               seedingFitness(m, n) = 0;
           end;
        end;
    end;
    
    populationIdentitiesMaster = ones(1, 2 * nSeedingGenomes, 'uint8');
    populationIdentitiesMaster(1, 1:maxGenomes) = 0;
    primaryTemplate = mod(seedingGenome(:, 1), fullFitnessIdentifier);
    
elseif seedingSetup == 17
    
    seedingGenome = zeros(nGenomicElements, 2 * maxGenomes, 'uint16');
    seedingFitness = ones(nEssentialGenes, 2 * maxGenomes, 'uint8');
    
    for n = 1:nGenes
        seedingGenome(2 * n - 1, 1:(2 * maxGenomes)) = 2;
    end;
    
    for n = 1:maxGenomes
        nonAssignedEssentialGenes = nEssentialGenes;
        while nonAssignedEssentialGenes > 0
            testSite = ceil(nGenes .* rand([1 1]));
            if seedingGenome(2 * testSite - 1, 2 * n - 1) == 2
                seedingGenome(2 * testSite - 1, (2 * n - 1):(2 * n)) = fullFitnessIdentifier + nonAssignedEssentialGenes;
                nonAssignedEssentialGenes = nonAssignedEssentialGenes - 1;
            end;
        end;
        if n > maxGenomes / 2
            nEssentialGeneKnockouts = seedingKnockout(1, 2) * nEssentialGenes;
        else
            nEssentialGeneKnockouts = seedingKnockout(1, 1) * nEssentialGenes;
        end;
        
        knockoutAvailabilityList = (1:nEssentialGenes)';
        for m = 1:(2 * nEssentialGeneKnockouts)
            currentKnockoutItem = ceil((nEssentialGenes - m + 1) .* rand([1 1]));
            currentKnockout = knockoutAvailabilityList(currentKnockoutItem, 1);
            knockoutAvailabilityList(currentKnockoutItem, :) = [];
            if m <= nEssentialGeneKnockouts
                currentKnockoutPosition = find(seedingGenome(:, 2 * n - 1) == ...
                    (fullFitnessIdentifier + currentKnockout));
                seedingGenome(currentKnockoutPosition, 2 * n - 1) = noFitnessIdentifier + currentKnockout;
                seedingFitness(currentKnockout,  2 * n - 1) = 0;
            else
                currentKnockoutPosition = find(seedingGenome(:, 2 * n) == ...
                    (fullFitnessIdentifier + currentKnockout));
                seedingGenome(currentKnockoutPosition, 2 * n) = noFitnessIdentifier + currentKnockout;
                seedingFitness(currentKnockout,  2 * n) = 0;
            end;
        end;
    end;
    
    for n = 1:maxGenomes
        nonAssignedProficientRecombinationSites = nRPIE;
        while nonAssignedProficientRecombinationSites > 0
            testSite = ceil((nGenes - 1) .* rand([1 1]));
            if seedingGenome(2 * testSite, 2 * n - 1) == 0
                seedingGenome(2 * testSite, (2 * n - 1):(2 * n)) = 1;
                nonAssignedProficientRecombinationSites = nonAssignedProficientRecombinationSites - 1;
            end;
        end;
    end;
    
    populationIdentitiesMaster = ones(1, 2 * maxGenomes, 'uint8');
    populationIdentitiesMaster(1, 1:maxGenomes) = 0;
    primaryTemplate = mod(seedingGenome(:, 1), fullFitnessIdentifier);
    
elseif seedingSetup == 18
    
    seedingGenome = zeros(nGenomicElements, 2 * maxGenomes, 'uint16');
    seedingFitness = ones(nEssentialGenes, 2 * maxGenomes, 'uint8');
    seedingFitness(:, 1:2:(maxGenomes - 1)) = 0;
    
    for n = 1:nGenes
        seedingGenome(2 * n - 1, 1:(2 * maxGenomes)) = 2;
    end;
    
    for n = 1:maxGenomes
        nonAssignedEssentialGenes = nEssentialGenes;
        while nonAssignedEssentialGenes > 0
            testSite = ceil(nGenes .* rand([1 1]));
            if seedingGenome(2 * testSite - 1, 2 * n - 1) == 2
                if n > (maxGenomes / 2)
                    seedingGenome(2 * testSite - 1, 2 * n - 1) = fullFitnessIdentifier + nonAssignedEssentialGenes;
                    seedingGenome(2 * testSite - 1, 2 * n) = fullFitnessIdentifier + nonAssignedEssentialGenes;
                else
                    seedingGenome(2 * testSite - 1, 2 * n - 1) = fullFitnessIdentifier + nonAssignedEssentialGenes;
                    seedingGenome(2 * testSite - 1, 2 * n) = noFitnessIdentifier + nonAssignedEssentialGenes;
                end;
                nonAssignedEssentialGenes = nonAssignedEssentialGenes - 1;
            end;
        end;
    end;
    
    for n = 1:maxGenomes
        nonAssignedProficientRecombinationSites = nRPIE;
        while nonAssignedProficientRecombinationSites > 0
            testSite = ceil((nGenes - 1) .* rand([1 1]));
            if seedingGenome(2 * testSite, 2 * n - 1) == 0
                seedingGenome(2 * testSite, (2 * n - 1):(2 * n)) = 1;
                nonAssignedProficientRecombinationSites = nonAssignedProficientRecombinationSites - 1;
            end;
        end;
    end;
    
    populationIdentitiesMaster = ones(1, 2 * maxGenomes, 'uint8');
    populationIdentitiesMaster(1, 1:maxGenomes) = 0;
    primaryTemplate = mod(seedingGenome(:, 1), fullFitnessIdentifier);
    
elseif seedingSetup == 19
    
    seedingGenome = zeros(nGenomicElements, 2 * maxGenomes, 'uint16');
    seedingFitness = ones(nEssentialGenes, 2 * maxGenomes, 'uint8');
    seedingFitness(:, 1:2:(2 * maxGenomes - 1)) = 0;
    
    for n = 1:nGenes
        seedingGenome(2 * n - 1, 1:(2 * maxGenomes)) = 2;
    end;
    
    for n = 1:maxGenomes
        nonAssignedEssentialGenes = nEssentialGenes;
        while nonAssignedEssentialGenes > 0
            testSite = ceil(nGenes .* rand([1 1]));
            if seedingGenome(2 * testSite - 1, 2 * n - 1) == 2
                seedingGenome(2 * testSite - 1, 2 * n - 1) = fullFitnessIdentifier + nonAssignedEssentialGenes;
                seedingGenome(2 * testSite - 1, 2 * n) = noFitnessIdentifier + nonAssignedEssentialGenes;
                nonAssignedEssentialGenes = nonAssignedEssentialGenes - 1;
            end;
        end;
    end;
    
    for n = 1:maxGenomes
        nonAssignedProficientRecombinationSites = nRPIE;
        while nonAssignedProficientRecombinationSites > 0
            testSite = ceil((nGenes - 1) .* rand([1 1]));
            if seedingGenome(2 * testSite, 2 * n - 1) == 0
                seedingGenome(2 * testSite, (2 * n - 1):(2 * n)) = 1;
                nonAssignedProficientRecombinationSites = nonAssignedProficientRecombinationSites - 1;
            end;
        end;
    end;
    
    populationIdentitiesMaster = ones(1, 2 * maxGenomes, 'uint8');
    populationIdentitiesMaster(1, 1:maxGenomes) = 0;
    primaryTemplate = mod(seedingGenome(:, 1), fullFitnessIdentifier);
    
elseif seedingSetup == 20
    
    seedingGenome = zeros(nGenomicElements, 2 * maxGenomes, 'uint16');
    seedingFitness = ones(nEssentialGenes, 2 * maxGenomes, 'uint8');
    seedingFitness(:, 1:2:(maxGenomes - 1)) = 0;
    
    for n = 1:nGenes
        seedingGenome(2 * n - 1, 1:(2 * maxGenomes)) = 2;
    end;
    
    nonAssignedEssentialGenes = nEssentialGenes;
    while nonAssignedEssentialGenes > 0
        testSite = ceil(nGenes .* rand([1 1]));
        if seedingGenome(2 * testSite - 1, 1) == 2
            seedingGenome(2 * testSite - 1, 1:2:(maxGenomes - 1)) = fullFitnessIdentifier + nonAssignedEssentialGenes;
            seedingGenome(2 * testSite - 1, 2:2:maxGenomes) = noFitnessIdentifier + nonAssignedEssentialGenes;
            nonAssignedEssentialGenes = nonAssignedEssentialGenes - 1;
        end;
    end;
    
    nonAssignedEssentialGenes = nEssentialGenes;
    while nonAssignedEssentialGenes > 0
        testSite = ceil(nGenes .* rand([1 1]));
        if seedingGenome(2 * testSite - 1, (maxGenomes + 1)) == 2
            seedingGenome(2 * testSite - 1, (maxGenomes + 1):2:(2 * maxGenomes - 1)) = fullFitnessIdentifier + nonAssignedEssentialGenes;
            seedingGenome(2 * testSite - 1, (maxGenomes + 2):2:(2 * maxGenomes)) = fullFitnessIdentifier + nonAssignedEssentialGenes;
            nonAssignedEssentialGenes = nonAssignedEssentialGenes - 1;
        end;
    end;
    
    nonAssignedProficientRecombinationSites = nRPIE;
    while nonAssignedProficientRecombinationSites > 0
        testSite = ceil((nGenes - 1) .* rand([1 1]));
        if seedingGenome(2 * testSite, 1) == 0
            seedingGenome(2 * testSite, 1:maxGenomes) = 1;
            nonAssignedProficientRecombinationSites = nonAssignedProficientRecombinationSites - 1;
        end;
    end;
    
    nonAssignedProficientRecombinationSites = nRPIE;
    while nonAssignedProficientRecombinationSites > 0
        testSite = ceil((nGenes - 1) .* rand([1 1]));
        if seedingGenome(2 * testSite, (maxGenomes + 1)) == 0
            seedingGenome(2 * testSite, (maxGenomes + 1):(2 * maxGenomes)) = 1;
            nonAssignedProficientRecombinationSites = nonAssignedProficientRecombinationSites - 1;
        end;
    end;
    
    populationIdentitiesMaster = ones(1, 2 * maxGenomes, 'uint8');
    populationIdentitiesMaster(1, 1:maxGenomes) = 0;
    primaryTemplate = mod(seedingGenome(:, 1), fullFitnessIdentifier);
    
elseif seedingSetup == 21
    
    seedingGenome = zeros(nGenomicElements, 2 * maxGenomes, 'uint16');
    seedingFitness = ones(nEssentialGenes, 2 * maxGenomes, 'uint8');
    seedingFitness(:, 1:2:(2 * maxGenomes - 1)) = 0;
    
    for n = 1:nGenes
        seedingGenome(2 * n - 1, 1:(2 * maxGenomes)) = 2;
    end;
    
    nonAssignedEssentialGenes = nEssentialGenes;
    while nonAssignedEssentialGenes > 0
        testSite = ceil(nGenes .* rand([1 1]));
        if seedingGenome(2 * testSite - 1, 1) == 2
            seedingGenome(2 * testSite - 1, 1:2:(maxGenomes - 1)) = fullFitnessIdentifier + nonAssignedEssentialGenes;
            seedingGenome(2 * testSite - 1, 2:2:maxGenomes) = noFitnessIdentifier + nonAssignedEssentialGenes;
            nonAssignedEssentialGenes = nonAssignedEssentialGenes - 1;
        end;
    end;
    
    nonAssignedEssentialGenes = nEssentialGenes;
    while nonAssignedEssentialGenes > 0
        testSite = ceil(nGenes .* rand([1 1]));
        if seedingGenome(2 * testSite - 1, (maxGenomes + 1)) == 2
            seedingGenome(2 * testSite - 1, (maxGenomes + 1):2:(2 * maxGenomes - 1)) = fullFitnessIdentifier + nonAssignedEssentialGenes;
            seedingGenome(2 * testSite - 1, (maxGenomes + 2):2:(2 * maxGenomes)) = noFitnessIdentifier + nonAssignedEssentialGenes;
            nonAssignedEssentialGenes = nonAssignedEssentialGenes - 1;
        end;
    end;
    
    nonAssignedProficientRecombinationSites = nRPIE;
    while nonAssignedProficientRecombinationSites > 0
        testSite = ceil((nGenes - 1) .* rand([1 1]));
        if seedingGenome(2 * testSite, 1) == 0
            seedingGenome(2 * testSite, 1:maxGenomes) = 1;
            nonAssignedProficientRecombinationSites = nonAssignedProficientRecombinationSites - 1;
        end;
    end;
    
    nonAssignedProficientRecombinationSites = nRPIE;
    while nonAssignedProficientRecombinationSites > 0
        testSite = ceil((nGenes - 1) .* rand([1 1]));
        if seedingGenome(2 * testSite, (maxGenomes + 1)) == 0
            seedingGenome(2 * testSite, (maxGenomes + 1):(2 * maxGenomes)) = 1;
            nonAssignedProficientRecombinationSites = nonAssignedProficientRecombinationSites - 1;
        end;
    end;
    
    populationIdentitiesMaster = ones(1, 2 * maxGenomes, 'uint8');
    populationIdentitiesMaster(1, 1:maxGenomes) = 0;
    primaryTemplate = mod(seedingGenome(:, 1), fullFitnessIdentifier);
    
end;

if clusterMode
    masterDirectory = centralRepositoryInput;
    mkdir(masterDirectory);
elseif strcmp(centralRepositoryInput, '.') == 0
    mkdir(centralRepositoryInput);
    cd(centralRepositoryInput);
end;

if frontendFlag == 0
    if silentIOInput == 0
        unix('mkdir averaged_results');
    end;
    if ~clusterMode
        unix('mkdir protocol_plots');
    end;
    if silentIOInput ~= 1
        unix('mkdir clustering_plots');
    end;
end;

if subPopulationFlag
    switch seedingSetup
        case {0, 1, 2}
            subPopulations = ones(1, 1, 'uint32');
        case {3}
            subPopulations = ones(1, 2, 'uint32');
            subPopulations(1, 2) = 2;
        case {5, 6, 7, 9, 10, 12, 13}
            subPopulations = ones(1, maxGenomes, 'uint32');
        case {4}
            subPopulations = ones(1, maxGenomes, 'uint32');
            subPopulations(1, 1:maxGenomes) = 1:maxGenomes;
        case {8}
            subPopulations = ones(1, maxGenomes, 'uint32');
            subPopulations(1, (maxGenomes / 2 + 1):maxGenomes) = 2;
        case {11}
            subPopulations = zeros(1, maxGenomes, 'uint32');
            subPopulations(1, 1) = 1;
            currentIndex = 1;
            if maxGenomes > 1
                for i = 2:maxGenomes
                    if sum(seedingGenome(:, 2 * i - 1) == seedingGenome(:, 2 * (i - 1) - 1)) == size(seedingGenome, 1) && ...
                            sum(seedingGenome(:, 2 * i) == seedingGenome(:, 2 * (i - 1))) == size(seedingGenome, 1)
                        subPopulations(1, i) = currentIndex;
                    else
                        currentIndex = currentIndex + 1;
                        subPopulations(1, i) = currentIndex;
                    end;
                end;
            end;
            
            save initialSubPopulations.mat subPopulations;
        otherwise
            subPopulationFlag = 0;
    end;
end;


% ---------------------------------------------------------
% Genomic code:
% 0: Intergenic element, recombination deficient
% 1: Intergenic element, recombination proficient
% 2: Gene, non-essential
% 1xxx: Gene, essential, fitness = 1
% 2xxx: Gene, essential, fitness = 0
% Note:
% xxx -> refers to the specific gene identifier and ranges
%        from 1 to nEssentialGenes, using three digits
% ---------------------------------------------------------

% ---------------------------------
% | Writing of configuration file |
% ---------------------------------

settings = zeros(54, 2);

settings(1, 1) = totalSimulationSteps;

settings(2, 1) = nGenes;
settings(3, 1) = essentialFraction;
settings(4, 1) = nRPIE;

settings(5, 1) = nIntergenicElements;
settings(6, 1) = nGenomicElements;
settings(7, 1) = nEssentialGenes;
settings(8, 1) = nNonEssentialGenes;
        
settings(9, 1) = maxGenomes;
        
settings(10, 1) = doubleRecombinationSiteSelectionAdvantage;
settings(11, 1) = inbreedingFlag;
if inbreedingFlag == 1
    settings(12, 1) = inbreedingFraction;
else
    settings(12, 1:2) = inbreedingFraction;
end;
settings(13, 1) = requiredHomologousFraction;

settings(14, 1) = clusterAnalysisPower;

settings(15, 1) = fullFitnessIdentifier;
settings(16, 1) = noFitnessIdentifier;

if recombinationModeFlag == 1
    settings(17, 1) = recombinationMode;
else
    settings(17, 1:2) = recombinationMode;
end;
if recombinationUnitFactorFlag == 1
    settings(18, 1) = recombinationUnitFactor;
else
    settings(18, 1:2) = recombinationUnitFactor;
end;
if recombinationProbabilityFlag == 1
    settings(19, 1) = recombinationProbability;
else
    settings(19, 1:2) = recombinationProbability;
end;
settings(20, 1) = erlangApplicationFlag;
settings(21, 1) = randomApplicationFlag;

settings(22, 1) = numericalCorrectionFactor;
if recombinationUnitFactorFlag == 1
    settings(23, 1) = mostLikelyDistance;
else
    settings(23, 1:2) = mostLikelyDistance;
end;
if recombinationUnitFactorFlag == 1
    settings(24, 1) = mostLikelyEventNumber;
else
    settings(24, 1:2) = mostLikelyEventNumber;
end;

settings(25, 1) = protocolSplittingSize;

settings(26, 1) = slidingWindowSize;
settings(27, 1) = slidingWindowPlotSize;
settings(28, 1) = parameterAveragingRange;

settings(29, 1) = seedingSetup;
settings(30, 1) = clusterFrequency;
if length(seedingKnockout) == 1
    settings(31, 1) = seedingKnockout;
else
    settings(31, 1:2) = seedingKnockout;
end;

settings(32, 1) = rearrangementMode;

settings(33, 1) = performSurvivalScreen;
settings(34, 1) = screeningArray;
settings(35, 1) = positiveSurvivalRuns;

settings(36, 1) = matingLocusFlag;
if matingLocusPositionFlag == 1
    settings(37, 1) = matingLocusPosition;
else
    settings(37, 1:2) = matingLocusPosition;
end;
settings(38, 1) = externalLocus;
if matingSwitchingFlag  == 1
	settings(39, 1) = matingSwitching;
else
	settings(39, 1:2) = matingSwitching;
end;

settings(40, 1) = adaptiveMutationRate;
settings(41, 1) = feedbackPopulationSize;
settings(42, 1) = feedbackPopulationSizeDerivative;
settings(43, 1) = feedbackDerivativeDistance;

settings(44, 1) = populationCapSize;
settings(45, 1) = derivativeFeedbackCap;

settings(46, 1) = skipMeiosis;
settings(47, 1) = randomizeCycles;
settings(48, 1) = mutationApplication;
if mutationModeFlag == 1
    settings(49, 1) = mutationMode;
else
    settings(49, 1:2) = mutationMode;
end;

settings(50, 1) = subPopulationFlag;
settings(51, 1) = displayInfo;
settings(52, 1) = analyzeClustering;

settings(53, 1) = readRandomGeneratorInput;

settings(54, 1) = simulationVersion;

if frontendFlag == 0 && silentIO == 0
    save averaged_results/settings settings;
    % save averaged_results/settings.txt settings -ascii;
    save averaged_results/randomGenerator randomGenerator;
    % save averaged_results/randomGenerator.txt randomGenerator -ascii;

    save averaged_results/verticalParametersList verticalParametersList;
    % save averaged_results/verticalParametersList.txt verticalParametersList -ascii;
    save averaged_results/horizontalParametersList horizontalParametersList;
    % save averaged_results/horizontalParametersList.txt horizontalParametersList -ascii;
    save averaged_results/clusterFrequencyArray clusterFrequencyArray;
    % save averaged_results/clusterFrequencyArray.txt clusterFrequencyArray -ascii;
    save averaged_results/seedingKnockoutArray seedingKnockoutArray;
    % save averaged_results/seedingKnockoutArray.txt seedingKnockoutArray -ascii;
end;




% -----------<-----------
% | --------->--------- |
% | | Simulation loop | |
% | ---------<--------- |
% ----------->-----------




for simulationSession = 1:statisticalAveraging
    
    currentSessionString = ['session_' num2str(simulationSession, '%.4d')];
    
    if frontendFlag == 0
        if ~clusterMode
            order=['mkdir ' currentSessionString];
            unix(order);

            %order=['cd ' currentSessionString];
            %unix(order);
            cd(currentSessionString);
        end;

        if silentIO == 0
            unix('mkdir results');
        end;
    end;
    
    clusterAnalysisMatrix = zeros(verticalParameters + 1, horizontalParameters + 1);
    clusterAnalysisMatrix(1, 1) = NaN;
    
    if performSurvivalScreen == 2
        clusterAnalysisMatrix(2:(verticalParameters + 1), 1) = seedingKnockoutArray';
        clusterAnalysisMatrix(1, 2:(horizontalParameters + 1)) = clusterFrequencyArray';
    else
        clusterAnalysisMatrix(2:(verticalParameters + 1), 1) = verticalParametersList';
        clusterAnalysisMatrix(1, 2:(horizontalParameters + 1)) = horizontalParametersList';
    end;
    
    clusterAnalysisMatrixUnfitted = clusterAnalysisMatrix;
    averageRecombinationSitesAnalysis = clusterAnalysisMatrix;
    averageFunctionalGenesAnalysis = clusterAnalysisMatrix;
    groupingMatrix = clusterAnalysisMatrix;
    
    if ((performSurvivalScreen == 1) && (screeningArray == 1))
        survivalArray = zeros(verticalParameters, 1);
    elseif ((performSurvivalScreen == 1) && (screeningArray == 2)) || (performSurvivalScreen == 2)
        survivalArray = zeros(horizontalParameters, 1);
    end;
    
    if fittingScheme == 0
        totalClusterAnalysisResults = zeros(verticalParameters * horizontalParameters, 9);
    elseif fittingScheme == 1
        totalClusterAnalysisResults = zeros(verticalParameters * horizontalParameters, 12);
    end;
    
    totalSlidingWindowResults = zeros(2 * verticalParameters * horizontalParameters, 13);
    totalSlidingWindowDeviationResults = zeros(2 * verticalParameters * horizontalParameters, 13);
    
    if displayInfo
        disp(' '); disp(' '); disp(' ');
        disp('--------------------------------');
        disp(['Current simulation session: ' num2str(simulationSession, '%.4d')]);
        disp('--------------------------------');
    end;
    
    for a = 1:verticalParameters
        for b = 1:horizontalParameters
            
            if ((performSurvivalScreen == 1) && (((screeningArray == 1) && (survivalArray(a, 1) < positiveSurvivalRuns)) || ((screeningArray == 2) && (survivalArray(b, 1) < positiveSurvivalRuns)))) || ...
                ((performSurvivalScreen == 2) && (survivalArray(b, 1) < positiveSurvivalRuns)) || (performSurvivalScreen == 0)                
                
                if (performSurvivalScreen == 2)
                    
                    currentDirectoryString = ['F' num2str(clusterFrequencyArray(1, b), '%.3d') '_K' num2str(seedingKnockoutArray(1, a), '%.4f')];
                    currentExperimentString = ['_' currentDirectoryString '_E' num2str(simulationSession, '%.2d')];
                    mitoticMutationRate = verticalParametersList(1, 1);
                    mitoticRearrangementRate = horizontalParametersList(1, 1);
                    clusterFrequency = clusterFrequencyArray(1, b);
                    seedingKnockout = seedingKnockoutArray(1, a);
                    
                    nGenePackSize = floor(nEssentialGenes / clusterFrequency);
                    nRPIEPackSize = floor(nRPIE / clusterFrequency);

                    remainingGenes = nEssentialGenes - clusterFrequency * nGenePackSize;
                    remainingRPIEs = nRPIE - clusterFrequency * nRPIEPackSize;

                    genesToDistribute = nEssentialGenes;
                    RPIEsToDistribute = nRPIE;

                    seedingGenome = zeros(nGenomicElements, 2 * maxGenomes, 'uint16');
                    seedingFitness = ones(nEssentialGenes, 2 * maxGenomes, 'uint8');

                    for k = 1:nGenes
                        seedingGenome(2 * k - 1, :) = 2;
                    end;

                    nEssentialGeneKnockouts = 2 * seedingKnockout * nEssentialGenes;

                    if nEssentialGeneKnockouts > 0
                        for k = 1:maxGenomes
                            availableGenes = (1:nEssentialGenes)';
                            nAvailableGenes = nEssentialGenes;
                            knockoutsToDistribute = nEssentialGeneKnockouts;
                            while knockoutsToDistribute > 0
                                currentKnockout = ceil(nAvailableGenes .* rand([1 1]));
                                seedingFitness(availableGenes(currentKnockout, 1), 2 * (k - 1) + ceil(2 .* rand([1 1]))) = 0;
                                availableGenes(currentKnockout, :) = [];
                                nAvailableGenes = nAvailableGenes - 1;
                                knockoutsToDistribute = knockoutsToDistribute - 1;
                            end;
                        end;
                    end;        

                    geneAbortFlag = 0;
                    RPIEAbortFlag = 0;

                    for k = 1:clusterFrequency

                        % the gene cluster starts at currentClusterStart

                        currentClusterStart = floor(1 + nGenomicElements * (k - 1) / clusterFrequency);
                        if mod(currentClusterStart, 2) == 0
                            currentClusterStart = currentClusterStart + 1;
                        end;

                        for m = 1:nGenePackSize
                            for p = 1:(2 * maxGenomes)
                                if seedingFitness(genesToDistribute, p) == 1
                                    seedingGenome(currentClusterStart + 2 * (m - 1), p) = genesToDistribute + fullFitnessIdentifier;
                                else
                                    seedingGenome(currentClusterStart + 2 * (m - 1), p) = genesToDistribute + noFitnessIdentifier;
                                end;
                            end;
                            genesToDistribute = genesToDistribute - 1;
                            if genesToDistribute == 0
                                geneAbortFlag = 1;
                                break;
                            end;
                        end;

                        if geneAbortFlag == 1
                            break;
                        end;

                        if remainingGenes > 0
                            seedingGenome(currentClusterStart + 2 * nGenePackSize, :) = genesToDistribute + fullFitnessIdentifier;
                            genesToDistribute = genesToDistribute - 1;
                            remainingGenes = remainingGenes - 1;
                            if genesToDistribute == 0
                                geneAbortFlag = 1;
                                break;
                            end;
                        end;

                        if geneAbortFlag == 1
                            break;
                        end;

                    end;

                    for k = 1:clusterFrequency

                        % the RPIE cluster starts at currentClusterCenter

                        currentClusterStart = floor(1 + nGenomicElements * (k - 1) / clusterFrequency);
                        currentClusterEnd = floor(1 + nGenomicElements * k / clusterFrequency) - 1;
                        currentClusterCenter = round((currentClusterEnd + currentClusterStart) / 2);
                        if mod(currentClusterStart, 2) == 0
                            currentClusterStart = currentClusterStart + 1;
                        end;
                        if mod(currentClusterCenter, 2) == 1
                            currentClusterCenter = currentClusterCenter + 1;
                        end;

                        for m = 1:nRPIEPackSize
                            seedingGenome(currentClusterCenter + 2 * (m - 1), :) = 1;
                            RPIEsToDistribute = RPIEsToDistribute - 1;
                            if RPIEsToDistribute == 0
                                RPIEAbortFlag = 1;
                                break;
                            end;
                        end;

                        if RPIEAbortFlag == 1
                            break;
                        end;

                        if remainingRPIEs > 0
                            seedingGenome(currentClusterCenter + 2 * nRPIEPackSize, :) = 1;
                            RPIEsToDistribute = RPIEsToDistribute - 1;
                            remainingRPIEs = remainingRPIEs - 1;
                            if RPIEsToDistribute == 0
                                RPIEAbortFlag = 1;
                                break;
                            end;
                        end;

                        if RPIEAbortFlag == 1
                            break;
                        end;

                    end;
                    
                else
                    
                    currentDirectoryString = ['M' num2str(verticalParametersList(1, a), '%.6f') '_R' num2str(horizontalParametersList(1, b), '%.6f')];
                    currentExperimentString = ['_' currentDirectoryString '_E' num2str(simulationSession, '%.2d')];
                    mitoticMutationRate = verticalParametersList(1, a);
                    mitoticRearrangementRate = horizontalParametersList(1, b);
                    
                end;                    
                
                if displayInfo
                    disp(' '); disp(' ');
                    disp(['Current simulation settings: ' currentDirectoryString]);
                end;

                simulationSteps = totalSimulationSteps;
                
                startWithMeiosis = 0;
                if randomizeCycles
                    currentState = round(rand);
                    if currentState == 1
                    	startWithMeiosis = 1;
                    end;
                else
                    currentState = 0;
                end;
                
                mitosisCounter = 0;
                meiosisCounter = 0;
                
                currentProtocolSection = 0;
                
                defeatException = 0;
                defeatOutcome = NaN;
                deathmatchFlag = (seedingSetup == 14) | (seedingSetup == 15) | (seedingSetup == 16) | ...
                    (seedingSetup == 17) | (seedingSetup == 18) | (seedingSetup == 19) | (seedingSetup == 20) | (seedingSetup == 21);
                if deathmatchFlag
                    populationIdentities = populationIdentitiesMaster;
                end;

                clusterAnalysis = [];
                averageRecombinationSitesArray = [];
                averageFunctionalGenesArray = [];
                simulationProtocol = zeros(simulationSteps, 22);

                % -----------------------------------------------------------------
                % Simulation protocol:
                % 1)  Simulation step
                % 2)  mutation rate during mitosis
                % 2)  # of mutations during mitosis
                % 4)  # of mutations affecting functional essential genes during
                %       mitosis
                % 5)  # of genomes that were deleted because of zero fitness
                % 6)  # of genomes that were deleted because of maximum cell limit
                % 7)  # of genomic rearrangements during mitosis
                % 8)  # of genomic recombinations during meiosis
                % 9)  # of inbreeding attempts during meiosis
                % 10) # of successful inbreeding attempts during meiosis
                % 11) # of outbreeding attempts during meiosis
                % 12) # of successful outbreeding attempts during meiosis
                % 13) # of mating type switching attempts during meiosis
                % 14) # of successful mating type switching attempts during meiosis
                % 15) # of genomes that were deleted because of lacking homology
                % 16) # of genomes that were deleted because of lacking RPIEs
                % 17) mean clustering value
                % 18) standard deviation of clustering value
                % 19) mean proficient recombination sites number
                % 20) standard deviation of proficient recombination sites
                %     number
                % 21) mean functional essential genes number
                % 22) standard deviation of functional essential genes number
                % -----------------------------------------------------------------
               
                if silentIO ~= 1
                    populationSizeMitosisProtocol = zeros(simulationSteps / 2, 1);
                    populationSizeMeiosisProtocol = zeros(simulationSteps / 2, 1);
                    averageTotalMutationsProtocol = zeros(simulationSteps / 2, 1);
                    averageFunctionalMutationsProtocol = zeros(simulationSteps / 2, 1);
                    zeroFitnessKillsMitosisProtocol = zeros(simulationSteps / 2, 1);
                    zeroFitnessKillsMeiosisProtocol = zeros(simulationSteps / 2, 1);
                    starvationKillsMitosisProtocol = zeros(simulationSteps / 2, 1);
                    starvationKillsMeiosisProtocol = zeros(simulationSteps / 2, 1);
                    averageRearrangementsProtocol = zeros(simulationSteps / 2, 1);
                    averageRealRearrangementsProtocol = zeros(simulationSteps / 2, 1);
                    averageRecombinationsProtocol = zeros(simulationSteps / 2, 1);
                    inbreedingAttemptsProtocol = zeros(simulationSteps / 2, 1);
                    successfulInbreedingsProtocol = zeros(simulationSteps / 2, 1);
                    inbreedingSuccessRatio = zeros(simulationSteps / 2, 1);
                    outbreedingAttemptsProtocol = zeros(simulationSteps / 2, 1);
                    successfulOutbreedingsProtocol = zeros(simulationSteps / 2, 1);
                    outbreedingSuccessRatio = zeros(simulationSteps / 2, 1);
                    lackingHomologyKillsProtocol = zeros(simulationSteps / 2, 1);
                    lackingRPIEKillsProtocol = zeros(simulationSteps / 2, 1);
                    falseOutbreedersProtocol = zeros(simulationSteps / 2, 1);
                    populationLoadMitosisProtocol = zeros(simulationSteps / 2, 2);
                    dropoutLoadMitosisProtocol = zeros(simulationSteps / 2, 4);
                    populationLoadMeiosisProtocol = zeros(simulationSteps / 2, 2);
                    dropoutLoadMeiosisProtocol = zeros(simulationSteps / 2, 6);
                end;
                
                if deathmatchFlag
                    population1SizeMitosisProtocol = zeros(simulationSteps / 2, 1);
                    population1SizeMeiosisProtocol = zeros(simulationSteps / 2, 1);
                    population2SizeMitosisProtocol = zeros(simulationSteps / 2, 1);
                    population2SizeMeiosisProtocol = zeros(simulationSteps / 2, 1);
                end;
                
                populationSizeDuringMitosisProtocol = zeros(simulationSteps / 2, 1);
                if adaptiveMutationRate
                	mutationRateProtocol = zeros(totalSimulationSteps / 2, 1);
                    mutationRateProtocol(1, 1) = mitoticMutationRate;
                end;
                
                if frontendFlag == 0
                    
                    if ~clusterMode
                        order=['mkdir ' currentDirectoryString];
                        unix(order);

                        %order=['cd ' currentDirectoryString];
                        %unix(order);
                        cd(currentDirectoryString);
                        unix('mkdir genomes');
                        if (silentIO ~= 1) || ((seedingSetup >= 14) && (seedingSetup <= 21))
                            unix('mkdir protocols');
                        end;
                        unix('mkdir clustering');
                    end;
                    if printoutSeedingGenome || printoutMitoticGenome || printoutMeioticGenome
                        unix('mkdir images');
                    end;
                    
                    if ~clusterMode
                        currentDirectory = 'genomes';
                    else
                        currentDirectory = masterDirectory;
                    end;
                    eval(['save ' currentDirectory '/seedingGenome seedingGenome;']);
                    
                    if saveASCIIFiles == 1
                        seedingGenomeDouble = double(seedingGenome);
                        save genomes/seedingGenome.txt seedingGenomeDouble -ascii;
                        clear seedingGenomeDouble;
                    end;
                end;

                if seedingSetup < 3
                    nGenomes = 1;
                elseif seedingSetup == 3
                    nGenomes = 2;
                elseif seedingSetup == 11
                    nGenomes = nSeedingGenomes;
                else
                    nGenomes = maxGenomes;
                end;
                
                if seedingSetup < 3
                    fitness = ones(nEssentialGenes, 2, 'uint8');
                elseif seedingSetup == 3
                    fitness = ones(nEssentialGenes, 4, 'uint8');
                elseif seedingSetup < 9
                    fitness = ones(nEssentialGenes, 2 * maxGenomes, 'uint8');
                else
                	fitness = seedingFitness;
                end;
                
                genomes = seedingGenome;
                
                if matingLocusFlag
                    if matingLocusPositionFlag == 1
                        matingLoci = matingLocusPosition .* ones(1, 2 * nGenomes);
                    else
                        matingLoci = cat(2, matingLocusPosition1 .* ones(1, nGenomes), matingLocusPosition2 .* ones(1, nGenomes));
                    end;
                    matingTypes = ones(1, 2 * nGenomes);
                    evenNumbers = 2:2:(2 * nGenomes);
                    matingTypes(1, evenNumbers) = -1;
                end;
                    
                

                % ------------------------------
                % | Printout of seeding genome |
                % ------------------------------

                if printoutSeedingGenome == 1
                    currentSimulationStep = 0;
                    currentSimulationStepFormatted = num2str(currentSimulationStep, '%.6d');

                    seedingGenomeDeficientIntergenicElementFingerprint     = uint8(1 .* (seedingGenome == 0));
                    seedingGenomeProficientIntergenicElementFingerprint    = uint8(2 .* (seedingGenome == 1));
                    seedingGenomeNonEssentialGeneFingerprint               = uint8(3 .* (seedingGenome == 2));
                    seedingGenomeEssentialGeneFullFitnessFingerprint       = uint8(4 .* (seedingGenome > fullFitnessIdentifier) .* (seedingGenome < noFitnessIdentifier));
                    seedingGenomeEssentialGeneNoFitnessFingerprint         = uint8(5 .* (seedingGenome > noFitnessIdentifier));

                    seedingGenomeSimplified = seedingGenomeDeficientIntergenicElementFingerprint + seedingGenomeProficientIntergenicElementFingerprint + ...
                        seedingGenomeNonEssentialGeneFingerprint + seedingGenomeEssentialGeneFullFitnessFingerprint + seedingGenomeEssentialGeneNoFitnessFingerprint;
                    figure;
                    imagesc(seedingGenomeSimplified);

                    h = gca;
                    colorbar;
                    set(h, 'FontSize', 14, 'LineWidth', 1.5);
                    xlabel('chromosome number');
                    ylabel('genomic site');
                    currentFile = ['images/simplifiedGenome' currentSimulationStepFormatted '.jpg'];
                    saveas(h, currentFile);
                    currentFrame = imread(currentFile);
                    imwrite(currentFrame, currentFile, 'jpg', 'Quality', 100);
                    close;

                    figure;
                    imagesc(seedingGenomeEssentialGeneFullFitnessFingerprint + seedingGenomeEssentialGeneNoFitnessFingerprint);

                    h = gca;
                    colorbar;
                    set(h, 'FontSize', 14, 'LineWidth', 1.5);
                    xlabel('chromosome number');
                    ylabel('genomic site');
                    currentFile = ['images/essentialGenes' currentSimulationStepFormatted '.jpg'];
                    saveas(h, currentFile);
                    currentFrame = imread(currentFile);
                    imwrite(currentFrame, currentFile, 'jpg', 'Quality', 100);
                    close;

                    figure;
                    imagesc(seedingGenomeProficientIntergenicElementFingerprint + seedingGenomeEssentialGeneFullFitnessFingerprint);

                    h = gca;
                    colorbar;
                    set(h, 'FontSize', 14, 'LineWidth', 1.5);
                    xlabel('chromosome number');
                    ylabel('genomic site');
                    currentFile = ['images/proficientIEs_functionalEGs' currentSimulationStepFormatted '.jpg'];
                    saveas(h, currentFile);
                    currentFrame = imread(currentFile);
                    imwrite(currentFrame, currentFile, 'jpg', 'Quality', 100);
                    close;
                end;

                % -----------------------------------------------------------------
                % The genomes undergo mitosis and meiosis in an alternating fashion
                % Mitosis: Duplication of genome, mutations and genomic
                %          rearrangements (swapping) are possible, determination
                %          of genome survival based on fitness value
                % Meiosis: Discrete number of recombination events at the position
                %          of proficient IE sites, subsequently mating either by
                %          inbreeding or outbreeding, determination of genome
                %          survival based on homologous fraction and presence of
                %          at least one copy of each essential gene
                % -----------------------------------------------------------------
                
                if silentIO == 0
                    if totalSimulationSteps > protocolSplittingSize
                        proficientRecombinationSitesProtocol = zeros(protocolSplittingSize, 2 * maxGenomes, 'uint16');
                        geneClusterSizesProtocol = zeros(1, maxGenomes, protocolSplittingSize, 'uint16');
                        starvationProtocol = zeros(2 * maxGenomes, protocolSplittingSize, 'uint16');
                        recombinationProtocol = zeros(1, 3, protocolSplittingSize / 2, 'uint16');
                        rearrangementProtocol = zeros(1, 13, protocolSplittingSize / 2, 'uint16');
                        mutationProtocol = zeros(1, 5, protocolSplittingSize / 2, 'uint16');
                        matingProtocol = zeros(4 * maxGenomes, 4, protocolSplittingSize / 2, 'uint16');
                    else
                        proficientRecombinationSitesProtocol = zeros(totalSimulationSteps, 2 * maxGenomes, 'uint16');
                        geneClusterSizesProtocol = zeros(1, maxGenomes, totalSimulationSteps, 'uint16');
                        starvationProtocol = zeros(2 * maxGenomes, totalSimulationSteps, 'uint16');
                        recombinationProtocol = zeros(1, 3, floor(totalSimulationSteps / 2), 'uint16');
                        rearrangementProtocol = zeros(1, 13, ceil(totalSimulationSteps / 2), 'uint16');
                        mutationProtocol = zeros(1, 5, ceil(totalSimulationSteps / 2), 'uint16');
                        matingProtocol = zeros(4 * maxGenomes, 4, floor(totalSimulationSteps / 2), 'uint16');
                    end;
                end;
                
                
                
                % Simulation cycle

                while (simulationSteps > 0) && (nGenomes > 0)
                    currentSimulationStep = totalSimulationSteps - simulationSteps + 1;
                    currentSimulationStepFormatted = num2str(currentSimulationStep, '%.6d');
                    if displayInfo
                        disp(' ');
                    end;
                    
                    if mod(currentSimulationStep, 10000) == 0 && strcmp(touchDirectory, '.') ~= 1
                        unix(['find ' touchDirectory ' -exec touch -a {} \;']);
                    end;

                    if mod(currentSimulationStep + 1, protocolSplittingSize) == 0
                        currentProtocolSection = currentProtocolSection + 1;
                        currentProtocolSectionFormatted = num2str(currentProtocolSection, '%.5d');
                    end;

                    switch currentState

                        case 0 % Mitotic division

                            mitosisCounter = mitosisCounter + 1;
                            if displayInfo
                                disp([':: Mitosis ' num2str(mitosisCounter) ' ::']);
                            end;

                            % -------------------------------------------------
                            % Mitotic Duplication
                            % Notes:
                            % Two genomes in data table before duplication:
                            % 1A-1B-2A-2B
                            % Same two genomes in data table after duplication:
                            % 1A-1B-1A-1B-2A-2B-2A-2B 
                            % -------------------------------------------------
                            
                            if subPopulationFlag
                                subPopulations = subPopulations(:, sort(cat(1, (1:nGenomes)', (1:nGenomes)')));
                            end;
                            
                            nGenomes = 2 * nGenomes;
                            genomeDuplicationArray = zeros(2 * nGenomes, 1, 'uint32');
                            for n = 1:(nGenomes / 2)
                                genomeDuplicationArray(4 * n - 3, 1) = 2 * n - 1;
                                genomeDuplicationArray(4 * n - 2, 1) = 2 * n;
                                genomeDuplicationArray(4 * n - 1, 1) = 2 * n - 1;
                                genomeDuplicationArray(4 * n , 1) = 2 * n;
                            end;
                            genomes = genomes(:, genomeDuplicationArray);
                            fitness = fitness(:, genomeDuplicationArray);
                            fitGenomes = ones(2 * nGenomes, 1, 'uint8');
                            if matingLocusFlag
                                matingLoci = matingLoci(:, genomeDuplicationArray);
                                matingTypes = matingTypes(:, genomeDuplicationArray);
                            end;
                            if deathmatchFlag
                                populationIdentities = populationIdentities(:, genomeDuplicationArray);
                            end;
                            
                            
                            % -------------
                            % | Mutations |
                            % -------------

                            % ----------------------------------------
                            % Mutation protocol:
                            % column 1: index of mutated chromosome
                            % column 2: index of mutated site
                            % column 3: gene identity before mutation
                            % column 4: gene identity after mutation
                            % column 5: fitness of gene after mutation
                            % ----------------------------------------
                            
                            if adaptiveMutationRate
                                mitoticMutationRate = mutationRateProtocol(mitosisCounter, 1);
                            end;
                            
                            if mod(mitosisCounter - 1, mutationApplication) == 0
                                if sum(mutationMode ~= 1) > 0
                                    mutationDecisions = rand(nGenes, 2 * nGenomes) <= mitoticMutationRate;
                                    if deathmatchFlag && mutationModeFlag == 2
                                        if mutationMode1 == 1
                                            mutationDecisions(:, populationIdentities == 0) = 0;
                                        end;
                                        if mutationMode2 == 1
                                            mutationDecisions(:, populationIdentities == 1) = 0;
                                        end;
                                    end;
                                    [siteIndices genomeIndices] = find(mutationDecisions == 1);
                                    chromosomeIndices = mod((genomeIndices + 1), 2) + 1;
                                    genomeIndices = uint32(genomeIndices ./ 2);
                                    nMutations = size(genomeIndices, 1);
                                else
                                    nMutations = 0;
                                end;

                                if nMutations > 0

                                    currentMutationProtocol = ones(0, 5);
                                    nRealMutations = 0;

                                    for n = 1:nMutations
                                        currentChromosomeIndex = 2 * (genomeIndices(n, 1) - 1) + chromosomeIndices(n, 1);
                                        currentGeneIndex = 2 * siteIndices(n, 1) - 1;
                                        backupChromosomeIndex = 2 * (genomeIndices(n, 1) - 1) + 3 - chromosomeIndices(n, 1);
                                        currentGene = genomes(currentGeneIndex, currentChromosomeIndex);

                                        if (currentGene > fullFitnessIdentifier) && (currentGene < noFitnessIdentifier)
                                            currentMutationProtocol = cat(1, currentMutationProtocol, ones(0, 5));
                                            nRealMutations = nRealMutations + 1;
                                            currentMutationProtocol(nRealMutations, 1) = currentChromosomeIndex;
                                            currentMutationProtocol(nRealMutations, 2) = currentGeneIndex;
                                            currentMutationProtocol(nRealMutations, 3) = currentGene;

                                            genePositions = find(genomes(:, currentChromosomeIndex) == currentGene);
                                            foundGenes = size(genePositions, 1);
                                            backupPositions = find(genomes(:, backupChromosomeIndex) == currentGene);
                                            foundBackupGenes = size(backupPositions, 1);
                                            if (foundGenes == 1) && (foundBackupGenes == 0)
                                                fitGenomes(currentChromosomeIndex, 1) = 0;
                                                fitGenomes(backupChromosomeIndex, 1) = 0;

                                                currentMutationProtocol(nRealMutations, 5) = 0;

                                            elseif (foundGenes == 1)
                                                fitness(currentGene - fullFitnessIdentifier, currentChromosomeIndex) = 0;
                                            end;
                                            genomes(currentGeneIndex, currentChromosomeIndex) = currentGene - fullFitnessIdentifier + noFitnessIdentifier;

                                            currentMutationProtocol(nRealMutations, 4) = currentGene - fullFitnessIdentifier + noFitnessIdentifier;
                                        end;
                                    end;

                                    if displayInfo
                                        disp([num2str(mitoticMutationRate, '%.6f') ' is the current mitotic mutation rate']);
                                        disp([num2str(nMutations) ' mutations occured during mitosis ' num2str(mitosisCounter) ', affecting ' ...
                                            num2str(nRealMutations) ' functional genes']);
                                    end;
                                    simulationProtocol(currentSimulationStep, 2) = mitoticMutationRate;
                                    simulationProtocol(currentSimulationStep, 3) = nMutations;
                                    simulationProtocol(currentSimulationStep, 4) = nRealMutations;
                                    
                                    if silentIO ~= 1
                                    	if startWithMeiosis
	                                        averageTotalMutationsProtocol(floor(currentSimulationStep / 2), 1) = nMutations / (2 * nGenomes);
	                                        averageFunctionalMutationsProtocol(floor(currentSimulationStep / 2), 1) = nRealMutations / (2 * nGenomes);
	                                    else
	                                        averageTotalMutationsProtocol(ceil(currentSimulationStep / 2), 1) = nMutations / (2 * nGenomes);
	                                        averageFunctionalMutationsProtocol(ceil(currentSimulationStep / 2), 1) = nRealMutations / (2 * nGenomes);
	                                    end;    
                                    end;
                                else
                                    currentMutationProtocol = ones(0, 5);
                                    nRealMutations = 0;

                                    if displayInfo
                                        disp([num2str(mitoticMutationRate, '%.6f') ' is the current mitotic mutation rate']);
                                        disp(['No mutations occurred during mitosis ' num2str(mitosisCounter)]);
                                    end;
                                    simulationProtocol(currentSimulationStep, 2) = mitoticMutationRate;
                                    simulationProtocol(currentSimulationStep, 3) = 0;
                                    simulationProtocol(currentSimulationStep, 4) = 0;
                                end;
                                
                                if silentIO == 0
                                    if totalSimulationSteps > protocolSplittingSize
                                        [currentXSize currentYSize currentZSize] = size(mutationProtocol);
                                        if nRealMutations > currentXSize
                                            mutationProtocolBackup = mutationProtocol;
                                            mutationProtocol = zeros(nRealMutations, 5, protocolSplittingSize / 2, 'uint16');
                                            mutationProtocol(1:currentXSize, :, :) = mutationProtocolBackup(:, :, :);
                                            clear mutationProtocolBackup;
                                        end;

                                        [currentXSize currentYSize] = size(currentMutationProtocol);
                                        if startWithMeiosis
                                        	temporaryStepCounter = floor(mod(currentSimulationStep, protocolSplittingSize) / 2);
                                    	else
                                    		temporaryStepCounter = ceil(mod(currentSimulationStep, protocolSplittingSize) / 2);
                                    	end;
                                        
                                        if mod(temporaryStepCounter, protocolSplittingSize / 2) == 0
                                            if size(currentMutationProtocol, 1) > 0
                                                mutationProtocol(1:currentXSize, 1:currentYSize, protocolSplittingSize / 2) = currentMutationProtocol(:, :);
                                            end;
                                            eval(['mutationProtocol' currentProtocolSectionFormatted ' = mutationProtocol;']);
                                            eval(['save protocols/mutationProtocol' currentProtocolSectionFormatted ' mutationProtocol' currentProtocolSectionFormatted ';']);
                                            eval(['clear mutationProtocol' currentProtocolSectionFormatted ';']);
                                            mutationProtocol = zeros(1, 5, protocolSplittingSize / 2, 'uint16');

                                        else
                                            if size(currentMutationProtocol, 1) > 0
                                                mutationProtocol(1:currentXSize, 1:currentYSize, temporaryStepCounter) = currentMutationProtocol(:, :);
                                            end;
                                        end;
                                    else
                                        [currentXSize currentYSize currentZSize] = size(mutationProtocol);
                                        if nRealMutations > currentXSize
                                            mutationProtocolBackup = mutationProtocol;
                                            mutationProtocol = zeros(nRealMutations, 5, ceil(totalSimulationSteps / 2), 'uint16');
                                            mutationProtocol(1:currentXSize, :, :) = mutationProtocolBackup(:, :, :);
                                            clear mutationProtocolBackup;

                                        end;
                                        [currentXSize currentYSize] = size(currentMutationProtocol);
                                        if size(currentMutationProtocol, 1) > 0
                                        	if startWithMeiosis
                                        		mutationProtocol(1:currentXSize, 1:currentYSize, floor(currentSimulationStep / 2)) = currentMutationProtocol(:, :);
                                        	else
                                            	mutationProtocol(1:currentXSize, 1:currentYSize, ceil(currentSimulationStep / 2)) = currentMutationProtocol(:, :);
                                            end;
                                        end;
                                    end;
                                end;
                                if silentIO ~= 1
                                	if startWithMeiosis
                                    	zeroFitnessKillsMitosisProtocol(floor(currentSimulationStep / 2), 1) = 2 * nGenomes - sum(fitGenomes);
                                	else
                                		zeroFitnessKillsMitosisProtocol(ceil(currentSimulationStep / 2), 1) = 2 * nGenomes - sum(fitGenomes);
                                	end;
                                end;

                                if (sum(fitGenomes) < 2 * nGenomes) && (sum(fitGenomes) > 0)
                                    fitGenomesLocations = find(fitGenomes == 1);

                                    if silentIO ~= 1
                                    	if startWithMeiosis
                                    	    dropoutLoadMitosisProtocol(floor(currentSimulationStep / 2), 1) = ...
	                                            sum(fitGenomes == 0) / 2;
	                                        dropoutLoadMitosisProtocol(floor(currentSimulationStep / 2), 2) = ...
	                                            sum(sum(~fitness(:, find(fitGenomes == 0))));
                                    	else
	                                        dropoutLoadMitosisProtocol(ceil(currentSimulationStep / 2), 1) = ...
	                                            sum(fitGenomes == 0) / 2;
	                                        dropoutLoadMitosisProtocol(ceil(currentSimulationStep / 2), 2) = ...
	                                            sum(sum(~fitness(:, find(fitGenomes == 0))));
                                        end;
                                    end;

                                    genomes = genomes(:, fitGenomesLocations');
                                    fitness = fitness(:, fitGenomesLocations');
                                    if deathmatchFlag
                                        populationIdentities = populationIdentities(:, fitGenomesLocations');
                                    end;
                                    if matingLocusFlag
                                        matingLoci = matingLoci(:, fitGenomesLocations');
                                        matingTypes = matingTypes(:, fitGenomesLocations');
                                    end;
                                    if subPopulationFlag
                                        reducedList = fitGenomesLocations(2:2:sum(fitGenomes), 1) ./ 2;
                                        subPopulations = subPopulations(:, reducedList');
                                    end;
                                    
                                    if displayInfo
                                        disp([num2str(nGenomes - (sum(fitGenomes) / 2)) ' genomes died during mitosis ' num2str(mitosisCounter)]);
                                    end;
                                    simulationProtocol(currentSimulationStep, 5) = nGenomes - (sum(fitGenomes) / 2);
                                    nGenomes = sum(fitGenomes) / 2;
                                elseif sum(fitGenomes) == 0
                                    genomes = genomes(:, []);
                                    fitness = fitness(:, []);
                                    if deathmatchFlag
                                        populationIdentities = populationIdentities(:, []);
                                    end;
                                    if matingLocusFlag
                                        matingLoci = matingLoci(:, []);
                                        matingTypes = matingTypes(:, []);
                                    end;
                                    if subPopulationFlag
                                        subPopulations = subPopulations(:, []);
                                    end;
                                    nGenomes = 0;
                                    if displayInfo
                                        disp(['All genomes died during mitosis ' num2str(mitosisCounter)]);
                                    end;
                                    simulationProtocol(currentSimulationStep, 5) = nGenomes;
                                else
                                    if displayInfo
                                        disp(['No genomes died during mitosis ' num2str(mitosisCounter)]);
                                    end;
                                    simulationProtocol(currentSimulationStep, 5) = 0;
                                end;
                                
                                if silentIO ~= 1
                                	if startWithMeiosis
                                		populationSizeDuringMitosisProtocol(floor(currentSimulationStep / 2), 1) = 2 * nGenomes;
                                	else
                                    	populationSizeDuringMitosisProtocol(ceil(currentSimulationStep / 2), 1) = 2 * nGenomes;
                                    end;
                                end;

                                % if adaptive mutation rate feedback is
                                % enabled, the new mutation rate has to be
                                % calculated on the basis of simulation history

                                if adaptiveMutationRate
                                    if feedbackDerivativeDistance > mitosisCounter
                                        populationHistory = [2 * maxGenomes populationSizeDuringMitosisProtocol(1:mitosisCounter, 1).'].';
                                        populationHistoryLength = length(populationHistory);
                                    else
                                        populationHistory = populationSizeDuringMitosisProtocol((mitosisCounter - feedbackDerivativeDistance + 1):mitosisCounter, 1);
                                        populationHistoryLength = feedbackDerivativeDistance;
                                    end;

                                    currentRegressors = ones(populationHistoryLength, 2);
                                    currentRegressors(:, 1) = (1:populationHistoryLength).';
                                    [regressionOutput, regressionOutputIntervals, regressionResiduals, regressionResidualsIntervals, regressionStatistics] = ...
                                        regress(populationHistory, currentRegressors);
                                    currentDerivative = 0.5 * regressionOutput(1, 1);

                                    currentPopulationSizeFeedbackFactor = (1 + (nGenomes - maxGenomes) / maxGenomes) .^ feedbackPopulationSize;
                                    currentPopulationsSizeDerivativeFeedbackFactor = 1 + (currentDerivative ./ maxGenomes .* feedbackPopulationSizeDerivative);
                                    if currentPopulationsSizeDerivativeFeedbackFactor <= 0
                                        currentPopulationsSizeDerivativeFeedbackFactor = derivativeFeedbackCap;
                                    end;
                                    if (mitosisCounter < totalSimulationSteps / 2)
                                        if relativeFeedback
                                            mutationRateProtocol(mitosisCounter + 1, 1) = mean([mutationRateProtocol(mitosisCounter, 1) ...
                                                mutationRateProtocol(mitosisCounter, 1) * currentPopulationSizeFeedbackFactor * currentPopulationsSizeDerivativeFeedbackFactor]);
                                        else
                                            mutationRateProtocol(mitosisCounter + 1, 1) = mean([mutationRateProtocol(mitosisCounter, 1) ...
                                                mutationRateProtocol(1, 1) * currentPopulationSizeFeedbackFactor * currentPopulationsSizeDerivativeFeedbackFactor]);
                                        end;
                                    end;
                                end;
                            end;
                            
                            % If there are more genomes than the maximum allowed
                            % number (after subtraction of genomes with fitness of
                            % zero), random kills occur, unless populationCapSize
                            % has been set to zero

                            if populationCapSize && (nGenomes > maxGenomes)
                                nGenomesToDelete = nGenomes - maxGenomes;
                                genomesToDelete = ceil(nGenomes .* rand([nGenomesToDelete 1]));
                                for n = 1:nGenomesToDelete
                                    duplicateFound = 1;
                                    while duplicateFound
                                        objectOccurence = find(genomesToDelete == genomesToDelete(n, 1));
                                        numberOfObjects = size(objectOccurence, 1);
                                        if numberOfObjects > 1
                                            genomesToDelete(n, 1) = ceil(nGenomes .* rand([1 1]));
                                        else
                                            duplicateFound = 0;
                                        end;
                                    end;
                                end;
                                genomesToDelete = sort(genomesToDelete);
                                fullDeletionList = zeros(nGenomesToDelete, 2);
                                fullDeletionList(:, 1) = 2 .* genomesToDelete(:) - 1;
                                fullDeletionList(:, 2) = 2 .* genomesToDelete(:);
                                fullDeletionList = sort(cat(1, fullDeletionList(:, 1), fullDeletionList(:, 2)));

                                if silentIO == 0
                                    if totalSimulationSteps > protocolSplittingSize
                                        if mod(currentSimulationStep, protocolSplittingSize) == 0
                                            starvationProtocol(1:length(fullDeletionList), protocolSplittingSize) = fullDeletionList(:, :);
                                            eval(['starvationProtocol' currentProtocolSectionFormatted ' = starvationProtocol;']);
                                            eval(['save protocols/starvationProtocol' currentProtocolSectionFormatted ' starvationProtocol' currentProtocolSectionFormatted ';']);
                                            eval(['clear starvationProtocol' currentProtocolSectionFormatted ';']);
                                            starvationProtocol = zeros(2 * maxGenomes, protocolSplittingSize, 'uint16');

                                        else
                                            starvationProtocol(1:length(fullDeletionList), mod(currentSimulationStep, protocolSplittingSize)) = fullDeletionList(:, :);
                                        end;
                                    else
                                        starvationProtocol(1:length(fullDeletionList), currentSimulationStep) = fullDeletionList(:, :);
                                    end;
                                end;

                                survivingGenomes = (1:(2 * nGenomes))';
                                for n = 1:(2 * nGenomesToDelete)
                                    survivingGenomes(fullDeletionList(2 * nGenomesToDelete - n + 1, 1), :) = [];
                                end;
                                
                                if subPopulationFlag
                                    simpleSurvivalList = (1:nGenomes)';
                                    for n = 1:nGenomesToDelete
                                        simpleSurvivalList(genomesToDelete(nGenomesToDelete - n + 1, 1), :) = [];
                                    end;
                                    subPopulations = subPopulations(:, simpleSurvivalList');
                                end;
                                
                                if silentIO ~= 1
                                	if startWithMeiosis
                                		dropoutLoadMitosisProtocol(floor(currentSimulationStep / 2), 3) = ...
	                                        nGenomesToDelete;
	                                    dropoutLoadMitosisProtocol(floor(currentSimulationStep / 2), 4) = ...
	                                        sum(sum(~fitness(:, fullDeletionList)));
                                	else
	                                    dropoutLoadMitosisProtocol(ceil(currentSimulationStep / 2), 3) = ...
	                                        nGenomesToDelete;
	                                    dropoutLoadMitosisProtocol(ceil(currentSimulationStep / 2), 4) = ...
	                                        sum(sum(~fitness(:, fullDeletionList)));
	                                end;
                                end;

                                genomes = genomes(:, survivingGenomes');
                                fitness = fitness(:, survivingGenomes');
                                if deathmatchFlag
                                    populationIdentities = populationIdentities(:, survivingGenomes');
                                end;
                                if matingLocusFlag
                                	matingLoci = matingLoci(:, survivingGenomes');
                                	matingTypes = matingTypes(:, survivingGenomes');
                                end;
                                
                                nGenomes = maxGenomes;
                                if displayInfo
                                    disp([num2str(nGenomesToDelete) ' genomes died of starvation during mitosis ' num2str(mitosisCounter)]);
                                end;
                                simulationProtocol(currentSimulationStep, 6) = nGenomesToDelete;
                                
                                if silentIO ~= 1
                                	if startWithMeiosis
                                		starvationKillsMitosisProtocol(floor(currentSimulationStep / 2), 1) = 2 * nGenomesToDelete;
                                	else
                                    	starvationKillsMitosisProtocol(ceil(currentSimulationStep / 2), 1) = 2 * nGenomesToDelete;
                                   	end;
                                end;
                                                                
                            else
                                if displayInfo
                                    disp(['No genomes died of starvation during mitosis ' num2str(mitosisCounter)]);
                                end;
                                simulationProtocol(currentSimulationStep, 6) = 0;
                            end;                           
                            
                            if silentIO ~= 1
                            	if startWithMeiosis
                            		populationLoadMitosisProtocol(floor(currentSimulationStep / 2), 1) = ...
	                                    nGenomes;
	                                populationLoadMitosisProtocol(floor(currentSimulationStep / 2), 2) = ...
	                                    sum(sum(~fitness));
                            	else
	                                populationLoadMitosisProtocol(ceil(currentSimulationStep / 2), 1) = ...
	                                    nGenomes;
	                                populationLoadMitosisProtocol(ceil(currentSimulationStep / 2), 2) = ...
	                                    sum(sum(~fitness));
	                            end;
                            end;
                            
                            % --------------------------
                            % | Genomic rearrangements |
                            % --------------------------

                            if (nGenomes > 0)
                                rearrangementDecisions = rand(2 * nGenes - 1, nGenomes) <= mitoticRearrangementRate;
                                [siteIndices genomeIndices] = find(rearrangementDecisions == 1);
                                nRearrangements = size(genomeIndices, 1);

                                if nRearrangements > 0
                                    currentChromosomesList = ceil(2 .* rand([nRearrangements 1]));

                                    if breedingSituation == 1 || subPopulationFlag == 1
                                        currentRearrangementProtocol = zeros(0, 13);
                                        nRealRearrangements = 0;
                                    else
                                        currentRearrangementProtocol = zeros(nRearrangements, 13);
                                        currentMatingLocusRearrangementProtocol = zeros(0, 13);
                                        nMatingLocusRearrangements = 0;
                                    end;

                                    if breedingSituation == 1 || subPopulationFlag == 1

                                        if rearrangementMode == 0 % swapping of sites

                                            for n = 1:nRearrangements
                                                currentChromosome = 2 * (genomeIndices(n, 1) - 1) + currentChromosomesList(n, 1);

                                                geneIdentity = genomes(siteIndices(n, 1), currentChromosome);
                                                
                                                subPopulationMatingSwap = 0;

                                                if (geneIdentity == 0) || (geneIdentity == 1) % Recombination site

                                                    swappingSite = recombinationSites(ceil(nIntergenicElements .* rand([1 1])), 1);
                                                    while swappingSite == siteIndices(n, 1)
                                                        swappingSite = recombinationSites(ceil(nIntergenicElements .* rand([1 1])), 1);
                                                    end;
                                                    
                                                    chromosomePair = (2 * genomeIndices(n, 1) - 1):(2 * genomeIndices(n, 1));
                                                    
                                                    swappingSiteBackup = genomes(swappingSite, chromosomePair);
                                                    genomes(swappingSite, chromosomePair) = genomes(siteIndices(n, 1), chromosomePair);
                                                    genomes(siteIndices(n, 1), chromosomePair) = swappingSiteBackup;
                                                    
                                                    if matingLocusFlag
                                                        if (swappingSite == matingLoci(1, currentChromosome))
                                                            subPopulationMatingSwap = 1;
                                                            nRealRearrangements = nRealRearrangements + 1;
                                                            if mod(currentChromosome, 2) == 0
                                                                matingLoci(1, (currentChromosome - 1):currentChromosome) = siteIndices(n, 1);
                                                                chromosomePair = [currentChromosome - 1 currentChromosome];
                                                            else
                                                                matingLoci(1, currentChromosome:(currentChromosome + 1)) = siteIndices(n, 1);
                                                                chromosomePair = [currentChromosome currentChromosome + 1];
                                                            end;
                                                            currentRearrangementProtocol = cat(1, currentRearrangementProtocol, zeros(1, 13));

                                                            currentRearrangementProtocol(nRealRearrangements, 1) = 2;

                                                            currentRearrangementProtocol(nRealRearrangements, [2 5]) = chromosomePair;
                                                            currentRearrangementProtocol(nRealRearrangements, [3 6]) = swappingSite;
                                                            currentRearrangementProtocol(nRealRearrangements, [4 7]) = matingLocusIdentifier;

                                                            currentRearrangementProtocol(nRealRearrangements, [8 11]) = chromosomePair;
                                                            currentRearrangementProtocol(nRealRearrangements, [9 12]) = siteIndices(n, 1);
                                                            currentRearrangementProtocol(nRealRearrangements, [10 13]) = matingLocusIdentifier;
                                                        elseif (siteIndices(n, 1) == matingLoci(1, currentChromosome))
                                                            subPopulationMatingSwap = 1;
                                                            nRealRearrangements = nRealRearrangements + 1;
                                                            if mod(currentChromosome, 2) == 0
                                                                matingLoci(1, (currentChromosome - 1):currentChromosome) = swappingSite;
                                                                chromosomePair = [currentChromosome - 1 currentChromosome];
                                                            else
                                                                matingLoci(1, currentChromosome:(currentChromosome + 1)) = swappingSite;
                                                                chromosomePair = [currentChromosome currentChromosome + 1];
                                                            end;
                                                            currentRearrangementProtocol = cat(1, currentRearrangementProtocol, zeros(1, 13));

                                                            currentRearrangementProtocol(nRealRearrangements, 1) = 2;

                                                            currentRearrangementProtocol(nRealRearrangements, [2 5]) = chromosomePair;
                                                            currentRearrangementProtocol(nRealRearrangements, [3 6]) = siteIndices(n, 1);
                                                            currentRearrangementProtocol(nRealRearrangements, [4 7]) = matingLocusIdentifier;

                                                            currentRearrangementProtocol(nRealRearrangements, [8 11]) = chromosomePair;
                                                            currentRearrangementProtocol(nRealRearrangements, [9 12]) = swappingSite;
                                                            currentRearrangementProtocol(nRealRearrangements, [10 13]) = matingLocusIdentifier;
                                                        end;
                                                    end;    
                                                    
                                                    if geneIdentity ~= swappingSiteBackup                                                        
                                                        nRealRearrangements = nRealRearrangements + 1;
                                                        currentRearrangementProtocol = cat(1, currentRearrangementProtocol, zeros(1, 13));

                                                        currentRearrangementProtocol(nRealRearrangements, 1) = 0;

                                                        currentRearrangementProtocol(nRealRearrangements, [2 5]) = chromosomePair;
                                                        currentRearrangementProtocol(nRealRearrangements, [3 6]) = siteIndices(n, 1);
                                                        currentRearrangementProtocol(nRealRearrangements, [4 7]) = genomes(swappingSite, chromosomePair);

                                                        currentRearrangementProtocol(nRealRearrangements, [8 11]) = chromosomePair;
                                                        currentRearrangementProtocol(nRealRearrangements, [9 12]) = swappingSite;
                                                        currentRearrangementProtocol(nRealRearrangements, [10 13]) = swappingSiteBackup;
                                                    end;

                                                else % Gene

                                                    swappingSite = geneSites(ceil(nGenes .* rand([1 1])), 1);
                                                    while swappingSite == siteIndices(n, 1)
                                                        swappingSite = geneSites(ceil(nGenes .* rand([1 1])), 1);
                                                    end;

                                                    % -------------------------------
                                                    % | Swapping with random target |
                                                    % -------------------------------

                                                    chromosomePair = (2 * genomeIndices(n, 1) - 1):(2 * genomeIndices(n, 1));

                                                    swappingSiteBackup = genomes(swappingSite, chromosomePair);
                                                    genomes(swappingSite, chromosomePair) = genomes(siteIndices(n, 1), chromosomePair);
                                                    genomes(siteIndices(n, 1), chromosomePair) = swappingSiteBackup;
                                                    
                                                    if matingLocusFlag
                                                        if (swappingSite == matingLoci(1, chromosomePair(1, 1)))
                                                            subPopulationMatingSwap = 1;
                                                            matingLoci(1, chromosomePair) = siteIndices(n, 1);
                                                            nRealRearrangements = nRealRearrangements + 1;
                                                            currentRearrangementProtocol = cat(1, currentRearrangementProtocol, zeros(1, 13));

                                                            currentRearrangementProtocol(nRealRearrangements, 1) = 2;

                                                            currentRearrangementProtocol(nRealRearrangements, [2 5]) = chromosomePair;
                                                            currentRearrangementProtocol(nRealRearrangements, [3 6]) = swappingSite;
                                                            currentRearrangementProtocol(nRealRearrangements, [4 7]) = matingLocusIdentifier;

                                                            currentRearrangementProtocol(nRealRearrangements, [8 11]) = chromosomePair;
                                                            currentRearrangementProtocol(nRealRearrangements, [9 12]) = siteIndices(n, 1);
                                                            currentRearrangementProtocol(nRealRearrangements, [10 13]) = matingLocusIdentifier;
                                                        elseif (siteIndices(n, 1) == matingLoci(1, chromosomePair(1, 1)))
                                                            subPopulationMatingSwap = 1;
                                                            matingLoci(1, chromosomePair) = swappingSite;
                                                            nRealRearrangements = nRealRearrangements + 1;
                                                            currentRearrangementProtocol = cat(1, currentRearrangementProtocol, zeros(1, 13));

                                                            currentRearrangementProtocol(nRealRearrangements, 1) = 2;

                                                            currentRearrangementProtocol(nRealRearrangements, [2 5]) = chromosomePair;
                                                            currentRearrangementProtocol(nRealRearrangements, [3 6]) = siteIndices(n, 1);
                                                            currentRearrangementProtocol(nRealRearrangements, [4 7]) = matingLocusIdentifier;

                                                            currentRearrangementProtocol(nRealRearrangements, [8 11]) = chromosomePair;
                                                            currentRearrangementProtocol(nRealRearrangements, [9 12]) = swappingSite;
                                                            currentRearrangementProtocol(nRealRearrangements, [10 13]) = matingLocusIdentifier;
                                                        end;
                                                    end; 
                                                    
                                                    if genomes(swappingSite, chromosomePair(1, 1)) ~= swappingSiteBackup(1, 1)
                                                        nRealRearrangements = nRealRearrangements + 1;
                                                        currentRearrangementProtocol = cat(1, currentRearrangementProtocol, zeros(1, 13));

                                                        currentRearrangementProtocol(nRealRearrangements, 1) = 1;

                                                        currentRearrangementProtocol(nRealRearrangements, [2 5]) = chromosomePair;
                                                        currentRearrangementProtocol(nRealRearrangements, [3 6]) = siteIndices(n, 1);
                                                        currentRearrangementProtocol(nRealRearrangements, [4 7]) = genomes(swappingSite, chromosomePair);

                                                        currentRearrangementProtocol(nRealRearrangements, [8 11]) = chromosomePair;
                                                        currentRearrangementProtocol(nRealRearrangements, [9 12]) = swappingSite;
                                                        currentRearrangementProtocol(nRealRearrangements, [10 13]) = swappingSiteBackup;
                                                    end;

                                                end;
                                                
                                                if subPopulationFlag
                                                    if (mod(geneIdentity, 1000) ~= mod(swappingSiteBackup(1, 1), 1000)) || subPopulationMatingSwap || ...
                                                            (geneIdentity == 2 && swappingSiteBackup(1, 1) ~= 2) || ...
                                                            (geneIdentity ~= 2 && swappingSiteBackup(1, 1) == 2)
                                                        newID = 1;
                                                        while ~isempty(find(subPopulations == newID, 1, 'first'))
                                                            newID = newID + 1;
                                                        end;
                                                        subPopulations(1, genomeIndices(n, 1)) = newID;
                                                    end;
                                                end;
                                            end;

                                        else % rearrangementMode == 1, i.e. flipping of sites

                                            for n = 1:nRearrangements
                                                currentChromosome = 2 * (genomeIndices(n, 1) - 1) + currentChromosomesList(n, 1);
                                                chromosomePair = (2 * genomeIndices(n, 1) - 1):(2 * genomeIndices(n, 1));
                                                geneIdentity = genomes(siteIndices(n, 1), currentChromosome);

                                                subPopulationMatingSwap = 0;
                                                
                                                if (geneIdentity == 0) || (geneIdentity == 1) % Recombination site

                                                    swappingSite = recombinationSites(ceil(nIntergenicElements .* rand([1 1])), 1);
                                                    while swappingSite == siteIndices(n, 1)
                                                        swappingSite = recombinationSites(ceil(nIntergenicElements .* rand([1 1])), 1);
                                                    end;

                                                    if swappingSite < siteIndices(n, 1)
                                                        backupSite = swappingSite;
                                                        swappingSite = siteIndices(n, 1);
                                                        siteIndices(n, 1) = backupSite;
                                                    end;

                                                    genomes(siteIndices(n, 1):swappingSite, chromosomePair) = flipud(genomes(siteIndices(n, 1):swappingSite, chromosomePair));
                                                    
                                                    if matingLocusFlag
                                                        if (matingLoci(1, chromosomePair(1, 1)) <= swappingSite) && (matingLoci(1, chromosomePair(1, 1)) >= siteIndices(n, 1))
                                                            subPopulationMatingSwap = 1;
                                                            oldLocus = matingLoci(1, chromosomePair(1, 1));
                                                            currentDistance = swappingSite - matingLoci(1, chromosomePair(1, 1));
                                                            matingLoci(1, chromosomePair) = siteIndices(n, 1) + currentDistance;
                                                            nRealRearrangements = nRealRearrangements + 1;
                                                            currentRearrangementProtocol = cat(1, currentRearrangementProtocol, zeros(1, 13));

                                                            currentRearrangementProtocol(nRealRearrangements, 1) = 2;

                                                            currentRearrangementProtocol(nRealRearrangements, [2 5]) = chromosomePair;
                                                            currentRearrangementProtocol(nRealRearrangements, [3 6]) = oldLocus;
                                                            currentRearrangementProtocol(nRealRearrangements, [4 7]) = matingLocusIdentifier;

                                                            currentRearrangementProtocol(nRealRearrangements, [8 11]) = chromosomePair;
                                                            currentRearrangementProtocol(nRealRearrangements, [9 12]) = siteIndices(n, 1) + currentDistance;
                                                            currentRearrangementProtocol(nRealRearrangements, [10 13]) = matingLocusIdentifier;
                                                        end;
                                                    end;
                                                    
                                                    nRealRearrangements = nRealRearrangements + 1;
                                                    currentRearrangementProtocol = cat(1, currentRearrangementProtocol, zeros(1, 13));
                                                    currentRearrangementProtocol(nRealRearrangements, 1) = 0;
                                                    
                                                else % Gene

                                                    swappingSite = geneSites(ceil(nGenes .* rand([1 1])), 1);
                                                    while swappingSite == siteIndices(n, 1)
                                                        swappingSite = geneSites(ceil(nGenes .* rand([1 1])), 1);
                                                    end;

                                                    if swappingSite < siteIndices(n, 1)
                                                        backupSite = swappingSite;
                                                        swappingSite = siteIndices(n, 1);
                                                        siteIndices(n, 1) = backupSite;
                                                    end;

                                                    genomes(siteIndices(n, 1):swappingSite, chromosomePair) = flipud(genomes(siteIndices(n, 1):swappingSite, chromosomePair));
                                                    
                                                    if matingLocusFlag
                                                        if (matingLoci(1, chromosomePair(1, 1)) <= swappingSite) && (matingLoci(1, chromosomePair(1, 1)) >= siteIndices(n, 1))
                                                            subPopulationMatingSwap = 1;
                                                            oldLocus = matingLoci(1, chromosomePair(1, 1));
                                                            currentDistance = swappingSite - matingLoci(1, chromosomePair(1, 1));
                                                            matingLoci(1, chromosomePair) = siteIndices(n, 1) + currentDistance;
                                                            nRealRearrangements = nRealRearrangements + 1;
                                                            currentRearrangementProtocol = cat(1, currentRearrangementProtocol, zeros(1, 13));

                                                            currentRearrangementProtocol(nRealRearrangements, 1) = 2;

                                                            currentRearrangementProtocol(nRealRearrangements, [2 5]) = chromosomePair;
                                                            currentRearrangementProtocol(nRealRearrangements, [3 6]) = oldLocus;
                                                            currentRearrangementProtocol(nRealRearrangements, [4 7]) = matingLocusIdentifier;

                                                            currentRearrangementProtocol(nRealRearrangements, [8 11]) = chromosomePair;
                                                            currentRearrangementProtocol(nRealRearrangements, [9 12]) = siteIndices(n, 1) + currentDistance;
                                                            currentRearrangementProtocol(nRealRearrangements, [10 13]) = matingLocusIdentifier;
                                                        end;
                                                    end;
                                                    
                                                    nRealRearrangements = nRealRearrangements + 1;
                                                    currentRearrangementProtocol = cat(1, currentRearrangementProtocol, zeros(1, 13));
                                                    currentRearrangementProtocol(nRealRearrangements, 1) = 1;

                                                end;

                                                currentRearrangementProtocol(nRealRearrangements, [2 5]) = chromosomePair;
                                                currentRearrangementProtocol(nRealRearrangements, [3 6]) = siteIndices(n, 1);
                                                currentRearrangementProtocol(nRealRearrangements, [4 7]) = genomes(swappingSite, chromosomePair);
                                                currentRearrangementProtocol(nRealRearrangements, [8 11]) = chromosomePair;
                                                currentRearrangementProtocol(nRealRearrangements, [9 12]) = swappingSite;
                                                currentRearrangementProtocol(nRealRearrangements, [10 13]) = genomes(siteIndices(n, 1), chromosomePair);
                                                
                                                if subPopulationFlag
                                                    if (~isempty(find(mod(genomes(siteIndices(n, 1):swappingSite, chromosomePair), 1000) ~= ...
                                                            mod(flipud(genomes(siteIndices(n, 1):swappingSite, chromosomePair)), 1000), 1, 'first'))) ...
                                                            || subPopulationMatingSwap || ...
                                                            (~isempty(find(genomes(siteIndices(n, 1):swappingSite, chromosomePair) == 2 & ...
                                                            flipud(genomes(siteIndices(n, 1):swappingSite, chromosomePair)) ~= 2, 1, 'first'))) || ...
                                                            (~isempty(find(genomes(siteIndices(n, 1):swappingSite, chromosomePair) ~= 2 & ...
                                                            flipud(genomes(siteIndices(n, 1):swappingSite, chromosomePair)) == 2, 1, 'first')))
                                                        newID = 1;
                                                        while ~isempty(find(subPopulations == newID, 1, 'first'))
                                                            newID = newID + 1;
                                                        end;
                                                        subPopulations(1, genomeIndices(n, 1)) = newID;
                                                    end;
                                                end;
                                            end;

                                        end;

                                    else % outbreeding fraction is not zero

                                        for n = 1:nRearrangements
                                            currentChromosome = currentChromosomesList(n, 1);
                                            geneIdentity = genomes(siteIndices(n, 1), 2 * (genomeIndices(n, 1) - 1) + currentChromosome);

                                            currentRearrangementProtocol(n, 2) = 2 * (genomeIndices(n, 1) - 1) + currentChromosome;
                                            currentRearrangementProtocol(n, 3) = siteIndices(n, 1);
                                            currentRearrangementProtocol(n, 4) = geneIdentity;

                                            if (geneIdentity == 0) || (geneIdentity == 1) % Recombination site
                                                currentRearrangementProtocol(n, 1) = 0;

                                                swappingSite = recombinationSites(ceil(nIntergenicElements .* rand([1 1])), 1);
                                                while swappingSite == siteIndices(n, 1)
                                                    swappingSite = recombinationSites(ceil(nIntergenicElements .* rand([1 1])), 1);
                                                end;
                                                swappingSiteBackup = genomes(swappingSite, 2 * (genomeIndices(n, 1) - 1) + currentChromosome);
                                                genomes(swappingSite, 2 * (genomeIndices(n, 1) - 1) + currentChromosome) = geneIdentity;
                                                genomes(siteIndices(n, 1), 2 * (genomeIndices(n, 1) - 1) + currentChromosome) = swappingSiteBackup;
                                                
                                                if matingLocusFlag
                                                	if (swappingSite == matingLoci(1, 2 * (genomeIndices(n, 1) - 1) + currentChromosome))
                                                        chromosomePair = [2 * genomeIndices(n, 1) - 1 2 * (genomeIndices(n, 1))];
                                                        matingLoci(1, (2 * genomeIndices(n, 1) - 1):(2 * (genomeIndices(n, 1)))) = siteIndices(n, 1);
                                                        nMatingLocusRearrangements = nMatingLocusRearrangements + 1;
                                                        currentMatingLocusRearrangementProtocol = cat(1, currentMatingLocusRearrangementProtocol, zeros(1, 13));

                                                        currentMatingLocusRearrangementProtocol(nMatingLocusRearrangements, 1) = 2;
                                                        currentMatingLocusRearrangementProtocol(nMatingLocusRearrangements, [2 5]) = chromosomePair;
                                                        currentMatingLocusRearrangementProtocol(nMatingLocusRearrangements, [3 6]) = swappingSite;
                                                        currentMatingLocusRearrangementProtocol(nMatingLocusRearrangements, [4 7]) = matingLocusIdentifier;

                                                        currentMatingLocusRearrangementProtocol(nMatingLocusRearrangements, [8 11]) = chromosomePair;
                                                        currentMatingLocusRearrangementProtocol(nMatingLocusRearrangements, [9 12]) = siteIndices(n, 1);
                                                        currentMatingLocusRearrangementProtocol(nMatingLocusRearrangements, [10 13]) = matingLocusIdentifier;
                                                    elseif (siteIndices(n, 1) == matingLoci(1, 2 * (genomeIndices(n, 1) - 1) + currentChromosome))
                                                        chromosomePair = [2 * genomeIndices(n, 1) - 1 2 * (genomeIndices(n, 1))];
                                                        matingLoci(1, (2 * genomeIndices(n, 1) - 1):(2 * (genomeIndices(n, 1)))) = swappingSite;
                                                        nMatingLocusRearrangements = nMatingLocusRearrangements + 1;
                                                        currentMatingLocusRearrangementProtocol = cat(1, currentMatingLocusRearrangementProtocol, zeros(1, 13));

                                                        currentMatingLocusRearrangementProtocol(nMatingLocusRearrangements, 1) = 2;

                                                        currentMatingLocusRearrangementProtocol(nMatingLocusRearrangements, [2 5]) = chromosomePair;
                                                        currentMatingLocusRearrangementProtocol(nMatingLocusRearrangements, [3 6]) = siteIndices(n, 1);
                                                        currentMatingLocusRearrangementProtocol(nMatingLocusRearrangements, [4 7]) = matingLocusIdentifier;

                                                        currentMatingLocusRearrangementProtocol(nMatingLocusRearrangements, [8 11]) = chromosomePair;
                                                        currentMatingLocusRearrangementProtocol(nMatingLocusRearrangements, [9 12]) = swappingSite;
                                                        currentMatingLocusRearrangementProtocol(nMatingLocusRearrangements, [10 13]) = matingLocusIdentifier;
                                                    end;
                                                end; 

                                                currentRearrangementProtocol(n, 8) = 2 * (genomeIndices(n, 1) - 1) + currentChromosome;
                                                currentRearrangementProtocol(n, 9) = swappingSite;
                                                currentRearrangementProtocol(n, 10) = swappingSiteBackup;

                                            else % Gene
                                                currentRearrangementProtocol(n, 1) = 1;

                                                if geneIdentity > noFitnessIdentifier
                                                    geneSpecificity = geneIdentity - noFitnessIdentifier;
                                                elseif geneIdentity > fullFitnessIdentifier
                                                    geneSpecificity = geneIdentity - fullFitnessIdentifier;
                                                else
                                                    geneSpecificity = 0;
                                                end;
                                                if geneSpecificity ~= 0
                                                    fitHomologousSites = find(genomes(:, 2 * (genomeIndices(n, 1) - 1) + 3 - currentChromosome) ...
                                                        == geneSpecificity + fullFitnessIdentifier);
                                                    unfitHomologousSites = find(genomes(:, 2 * (genomeIndices(n, 1) - 1) + 3 - currentChromosome) ...
                                                        == geneSpecificity + noFitnessIdentifier);
                                                    homologousSites = cat(1, fitHomologousSites, unfitHomologousSites);
                                                else
                                                    homologousSites = find(genomes(:, 2 * (genomeIndices(n, 1) - 1) + 3 - currentChromosome) == 2);
                                                end;
                                                nHomologousSites = size(homologousSites, 1);
                                                if nHomologousSites > 1
                                                    homologousSitesDistances = abs(homologousSites - siteIndices(n, 1));
                                                    minimalDistancePosition = find(homologousSitesDistances == min(homologousSitesDistances));
                                                    homologousSite = homologousSites(minimalDistancePosition(1, 1), 1);
                                                elseif nHomologousSites == 1
                                                    homologousSite = homologousSites;
                                                else
                                                    homologousSite = 0;
                                                end;

                                                swappingSite = geneSites(ceil(nGenes .* rand([1 1])), 1);
                                                while swappingSite == siteIndices(n, 1)
                                                    swappingSite = geneSites(ceil(nGenes .* rand([1 1])), 1);
                                                end;
                                                swappingSiteIdentity = genomes(swappingSite, 2 * (genomeIndices(n, 1) - 1) + currentChromosome);
                                                if swappingSiteIdentity > noFitnessIdentifier
                                                    swappingSiteSpecificity = swappingSiteIdentity - noFitnessIdentifier;
                                                elseif swappingSiteIdentity > fullFitnessIdentifier
                                                    swappingSiteSpecificity = swappingSiteIdentity - fullFitnessIdentifier;
                                                else
                                                    swappingSiteSpecificity = 0;
                                                end;
                                                if swappingSiteSpecificity ~= 0
                                                    fitSwappingHomologousSites = find(genomes(:, 2 * (genomeIndices(n, 1) - 1) + 3 - currentChromosome) ...
                                                        == swappingSiteSpecificity + fullFitnessIdentifier);
                                                    unfitSwappingHomologousSites = find(genomes(:, 2 * (genomeIndices(n, 1) - 1) + 3 - currentChromosome) ...
                                                        == swappingSiteSpecificity + noFitnessIdentifier);
                                                    swappingHomologousSites = cat(1, fitSwappingHomologousSites, unfitSwappingHomologousSites);
                                                else
                                                    swappingHomologousSites = find(genomes(:, 2 * (genomeIndices(n, 1) - 1) + 3 - currentChromosome) == 2);
                                                end;
                                                nSwappingHomologousSites = size(swappingHomologousSites, 1);
                                                if nSwappingHomologousSites > 1
                                                    swappingHomologousSitesDistances = abs(swappingHomologousSites - swappingSite);
                                                    minimalDistancePosition = find(swappingHomologousSitesDistances == min(swappingHomologousSitesDistances));
                                                    swappingHomologousSite = swappingHomologousSites(minimalDistancePosition(1, 1), 1);
                                                elseif nSwappingHomologousSites == 1
                                                    swappingHomologousSite = swappingHomologousSites;
                                                else
                                                    swappingHomologousSite = 0;
                                                end;

                                                % -------------------------------------------------
                                                % | Swapping of primary region with random target |
                                                % -------------------------------------------------

                                                swappingSiteBackup = genomes(swappingSite, 2 * (genomeIndices(n, 1) - 1) + currentChromosome);
                                                genomes(swappingSite, 2 * (genomeIndices(n, 1) - 1) + currentChromosome) = geneIdentity;
                                                genomes(siteIndices(n, 1), 2 * (genomeIndices(n, 1) - 1) + currentChromosome) = swappingSiteBackup;
                                                
                                                if matingLocusFlag
                                                	if (swappingSite == matingLoci(1, 2 * (genomeIndices(n, 1) - 1) + currentChromosome))
                                                        matingLoci(1, 2 * (genomeIndices(n, 1) - 1) + currentChromosome) = siteIndices(n, 1);
                                                        nMatingLocusRearrangements = nMatingLocusRearrangements + 1;
                                                        currentMatingLocusRearrangementProtocol = cat(1, currentMatingLocusRearrangementProtocol, zeros(1, 13));

                                                        currentMatingLocusRearrangementProtocol(nMatingLocusRearrangements, 1) = 2;

                                                        currentMatingLocusRearrangementProtocol(nMatingLocusRearrangements, 2) = 2 * (genomeIndices(n, 1) - 1) + currentChromosome;
                                                        currentMatingLocusRearrangementProtocol(nMatingLocusRearrangements, 3) = swappingSite;
                                                        currentMatingLocusRearrangementProtocol(nMatingLocusRearrangements, 4) = matingLocusIdentifier;

                                                        currentMatingLocusRearrangementProtocol(nMatingLocusRearrangements, 8) = 2 * (genomeIndices(n, 1) - 1) + currentChromosome;
                                                        currentMatingLocusRearrangementProtocol(nMatingLocusRearrangements, 9) = siteIndices(n, 1);
                                                        currentMatingLocusRearrangementProtocol(nMatingLocusRearrangements, 10) = matingLocusIdentifier;
                                                    elseif (siteIndices(n, 1) == matingLoci(1, 2 * (genomeIndices(n, 1) - 1) + currentChromosome))
                                                        matingLoci(1, 2 * (genomeIndices(n, 1) - 1) + currentChromosome) = swappingSite;
                                                        nMatingLocusRearrangements = nMatingLocusRearrangements + 1;
                                                        currentMatingLocusRearrangementProtocol = cat(1, currentMatingLocusRearrangementProtocol, zeros(1, 13));

                                                        currentMatingLocusRearrangementProtocol(nMatingLocusRearrangements, 1) = 2;

                                                        currentMatingLocusRearrangementProtocol(nMatingLocusRearrangements, 2) = 2 * (genomeIndices(n, 1) - 1) + currentChromosome;
                                                        currentMatingLocusRearrangementProtocol(nMatingLocusRearrangements, 3) = siteIndices(n, 1);
                                                        currentMatingLocusRearrangementProtocol(nMatingLocusRearrangements, 4) = matingLocusIdentifier;

                                                        currentMatingLocusRearrangementProtocol(nMatingLocusRearrangements, 8) = 2 * (genomeIndices(n, 1) - 1) + currentChromosome;
                                                        currentMatingLocusRearrangementProtocol(nMatingLocusRearrangements, 9) = swappingSite;
                                                        currentMatingLocusRearrangementProtocol(nMatingLocusRearrangements, 10) = matingLocusIdentifier;
                                                    end;
                                                end;

                                                currentRearrangementProtocol(n, 8) = 2 * (genomeIndices(n, 1) - 1) + currentChromosome;
                                                currentRearrangementProtocol(n, 9) = swappingSite;
                                                currentRearrangementProtocol(n, 10) = swappingSiteBackup;

                                                % -------------------------------------------------
                                                % Swapping of homologous primary region with region
                                                % that is homologous to said random target
                                                % Notes:
                                                % 1) If homologous primary region or region
                                                % homologous to said random target cannot be
                                                % identified (i.e. they were lost) they are assumed
                                                % to be located geometrically next to the primary
                                                % region or the random target position respectively
                                                % 2) If neither of both regions can be identified,
                                                % the second swapping is skipped
                                                % -------------------------------------------------

                                                if (homologousSite == 0) && (swappingHomologousSite ~= 0)
                                                    swappingHomologousSiteBackup = genomes(swappingHomologousSite, ...
                                                        2 * (genomeIndices(n, 1) - 1) + 3 - currentChromosome);
                                                    genomes(swappingHomologousSite, 2 * (genomeIndices(n, 1) - 1) + 3 - currentChromosome) = ...
                                                        genomes(siteIndices(n, 1), 2 * (genomeIndices(n, 1) - 1) + 3 - currentChromosome);
                                                    genomes(siteIndices(n, 1), 2 * (genomeIndices(n, 1) - 1) + 3 - currentChromosome) = ...
                                                        swappingHomologousSiteBackup;
                                                    
                                                    if matingLocusFlag
                                                        if (swappingHomologousSite == matingLoci(1, 2 * (genomeIndices(n, 1) - 1) + 3 - currentChromosome))
                                                            matingLoci(1, 2 * (genomeIndices(n, 1) - 1) + 3 - currentChromosome) = siteIndices(n, 1);
                                                            nMatingLocusRearrangements = nMatingLocusRearrangements + 1;
                                                            currentMatingLocusRearrangementProtocol = cat(1, currentMatingLocusRearrangementProtocol, zeros(1, 13));

                                                            currentMatingLocusRearrangementProtocol(nMatingLocusRearrangements, 1) = 2;

                                                            currentMatingLocusRearrangementProtocol(nMatingLocusRearrangements, 2) = 2 * (genomeIndices(n, 1) - 1) + 3 - currentChromosome;
                                                            currentMatingLocusRearrangementProtocol(nMatingLocusRearrangements, 3) = swappingHomologousSite;
                                                            currentMatingLocusRearrangementProtocol(nMatingLocusRearrangements, 4) = matingLocusIdentifier;

                                                            currentMatingLocusRearrangementProtocol(nMatingLocusRearrangements, 8) = 2 * (genomeIndices(n, 1) - 1) + 3 - currentChromosome;
                                                            currentMatingLocusRearrangementProtocol(nMatingLocusRearrangements, 9) = siteIndices(n, 1);
                                                            currentMatingLocusRearrangementProtocol(nMatingLocusRearrangements, 10) = matingLocusIdentifier;
                                                        elseif (siteIndices(n, 1) == matingLoci(1, 2 * (genomeIndices(n, 1) - 1) + 3 - currentChromosome))
                                                            matingLoci(1, 2 * (genomeIndices(n, 1) - 1) + 3 - currentChromosome) = swappingHomologousSite;
                                                            nMatingLocusRearrangements = nMatingLocusRearrangements + 1;
                                                            currentMatingLocusRearrangementProtocol = cat(1, currentMatingLocusRearrangementProtocol, zeros(1, 13));

                                                            currentMatingLocusRearrangementProtocol(nMatingLocusRearrangements, 1) = 2;

                                                            currentMatingLocusRearrangementProtocol(nMatingLocusRearrangements, 2) = 2 * (genomeIndices(n, 1) - 1) + 3 - currentChromosome;
                                                            currentMatingLocusRearrangementProtocol(nMatingLocusRearrangements, 3) = siteIndices(n, 1);
                                                            currentMatingLocusRearrangementProtocol(nMatingLocusRearrangements, 4) = matingLocusIdentifier;

                                                            currentMatingLocusRearrangementProtocol(nMatingLocusRearrangements, 8) = 2 * (genomeIndices(n, 1) - 1) + 3 - currentChromosome;
                                                            currentMatingLocusRearrangementProtocol(nMatingLocusRearrangements, 9) = swappingHomologousSite;
                                                            currentMatingLocusRearrangementProtocol(nMatingLocusRearrangements, 10) = matingLocusIdentifier;
                                                        end;
                                                    end;

                                                    currentRearrangementProtocol(n, 5) = 2 * (genomeIndices(n, 1) - 1) + 3 - currentChromosome;
                                                    currentRearrangementProtocol(n, 6) = siteIndices(n, 1);
                                                    currentRearrangementProtocol(n, 7) = genomes(swappingHomologousSite, ...
                                                        2 * (genomeIndices(n, 1) - 1) + 3 - currentChromosome);
                                                    currentRearrangementProtocol(n, 11) = 2 * (genomeIndices(n, 1) - 1) + 3 - currentChromosome;
                                                    currentRearrangementProtocol(n, 12) = swappingHomologousSite;
                                                    currentRearrangementProtocol(n, 13) = swappingHomologousSiteBackup;

                                                elseif (homologousSite ~= 0) && (swappingHomologousSite == 0)
                                                    swappingHomologousSiteBackup = genomes(swappingSite, 2 * (genomeIndices(n, 1) - 1) + 3 - currentChromosome);
                                                    genomes(swappingSite, 2 * (genomeIndices(n, 1) - 1) + 3 - currentChromosome) = ...
                                                        genomes(homologousSite, 2 * (genomeIndices(n, 1) - 1) + 3 - currentChromosome);
                                                    genomes(homologousSite, 2 * (genomeIndices(n, 1) - 1) + 3 - currentChromosome) = ...
                                                        swappingHomologousSiteBackup;
                                                    
                                                    if matingLocusFlag
                                                        if (swappingSite == matingLoci(1, 2 * (genomeIndices(n, 1) - 1) + 3 - currentChromosome))
                                                            matingLoci(1, 2 * (genomeIndices(n, 1) - 1) + 3 - currentChromosome) = homologousSite;
                                                            nMatingLocusRearrangements = nMatingLocusRearrangements + 1;
                                                            currentMatingLocusRearrangementProtocol = cat(1, currentMatingLocusRearrangementProtocol, zeros(1, 13));

                                                            currentMatingLocusRearrangementProtocol(nMatingLocusRearrangements, 1) = 2;

                                                            currentMatingLocusRearrangementProtocol(nMatingLocusRearrangements, 2) = 2 * (genomeIndices(n, 1) - 1) + 3 - currentChromosome;
                                                            currentMatingLocusRearrangementProtocol(nMatingLocusRearrangements, 3) = swappingSite;
                                                            currentMatingLocusRearrangementProtocol(nMatingLocusRearrangements, 4) = matingLocusIdentifier;

                                                            currentMatingLocusRearrangementProtocol(nMatingLocusRearrangements, 8) = 2 * (genomeIndices(n, 1) - 1) + 3 - currentChromosome;
                                                            currentMatingLocusRearrangementProtocol(nMatingLocusRearrangements, 9) = homologousSite;
                                                            currentMatingLocusRearrangementProtocol(nMatingLocusRearrangements, 10) = matingLocusIdentifier;
                                                        elseif (homologousSite == matingLoci(1, 2 * (genomeIndices(n, 1) - 1) + 3 - currentChromosome))
                                                            matingLoci(1, 2 * (genomeIndices(n, 1) - 1) + 3 - currentChromosome) = swappingSite;
                                                            nMatingLocusRearrangements = nMatingLocusRearrangements + 1;
                                                            currentMatingLocusRearrangementProtocol = cat(1, currentMatingLocusRearrangementProtocol, zeros(1, 13));

                                                            currentMatingLocusRearrangementProtocol(nMatingLocusRearrangements, 1) = 2;

                                                            currentMatingLocusRearrangementProtocol(nMatingLocusRearrangements, 2) = 2 * (genomeIndices(n, 1) - 1) + 3 - currentChromosome;
                                                            currentMatingLocusRearrangementProtocol(nMatingLocusRearrangements, 3) = homologousSite;
                                                            currentMatingLocusRearrangementProtocol(nMatingLocusRearrangements, 4) = matingLocusIdentifier;

                                                            currentMatingLocusRearrangementProtocol(nMatingLocusRearrangements, 8) = 2 * (genomeIndices(n, 1) - 1) + 3 - currentChromosome;
                                                            currentMatingLocusRearrangementProtocol(nMatingLocusRearrangements, 9) = swappingSite;
                                                            currentMatingLocusRearrangementProtocol(nMatingLocusRearrangements, 10) = matingLocusIdentifier;
                                                        end;
                                                    end;

                                                    currentRearrangementProtocol(n, 5) = 2 * (genomeIndices(n, 1) - 1) + 3 - currentChromosome;
                                                    currentRearrangementProtocol(n, 6) = homologousSite;
                                                    currentRearrangementProtocol(n, 7) = genomes(swappingSite, 2 * (genomeIndices(n, 1) - 1) + 3 - currentChromosome);
                                                    currentRearrangementProtocol(n, 11) = 2 * (genomeIndices(n, 1) - 1) + 3 - currentChromosome;
                                                    currentRearrangementProtocol(n, 12) = swappingSite;
                                                    currentRearrangementProtocol(n, 13) = swappingHomologousSiteBackup;

                                                elseif (homologousSite ~= 0) && (swappingHomologousSite ~= 0)
                                                    swappingHomologousSiteBackup = genomes(swappingHomologousSite, ...
                                                        2 * (genomeIndices(n, 1) - 1) + 3 - currentChromosome);
                                                    genomes(swappingHomologousSite, 2 * (genomeIndices(n, 1) - 1) + 3 - currentChromosome) = ...
                                                        genomes(homologousSite, 2 * (genomeIndices(n, 1) - 1) + 3 - currentChromosome);
                                                    genomes(homologousSite, 2 * (genomeIndices(n, 1) - 1) + 3 - currentChromosome) = swappingHomologousSiteBackup;
                                                    
                                                    if matingLocusFlag
                                                        if (swappingHomologousSite == matingLoci(1, 2 * (genomeIndices(n, 1) - 1) + 3 - currentChromosome))
                                                            matingLoci(1, 2 * (genomeIndices(n, 1) - 1) + 3 - currentChromosome) = homologousSite;
                                                            nMatingLocusRearrangements = nMatingLocusRearrangements + 1;
                                                            currentMatingLocusRearrangementProtocol = cat(1, currentMatingLocusRearrangementProtocol, zeros(1, 13));

                                                            currentMatingLocusRearrangementProtocol(nMatingLocusRearrangements, 1) = 2;

                                                            currentMatingLocusRearrangementProtocol(nMatingLocusRearrangements, 2) = 2 * (genomeIndices(n, 1) - 1) + 3 - currentChromosome;
                                                            currentMatingLocusRearrangementProtocol(nMatingLocusRearrangements, 3) = swappingHomologousSite;
                                                            currentMatingLocusRearrangementProtocol(nMatingLocusRearrangements, 4) = matingLocusIdentifier;

                                                            currentMatingLocusRearrangementProtocol(nMatingLocusRearrangements, 8) = 2 * (genomeIndices(n, 1) - 1) + 3 - currentChromosome;
                                                            currentMatingLocusRearrangementProtocol(nMatingLocusRearrangements, 9) = homologousSite;
                                                            currentMatingLocusRearrangementProtocol(nMatingLocusRearrangements, 10) = matingLocusIdentifier;
                                                        elseif (homologousSite == matingLoci(1, 2 * (genomeIndices(n, 1) - 1) + 3 - currentChromosome))
                                                            matingLoci(1, 2 * (genomeIndices(n, 1) - 1) + 3 - currentChromosome) = swappingHomologousSite;
                                                            nMatingLocusRearrangements = nMatingLocusRearrangements + 1;
                                                            currentMatingLocusRearrangementProtocol = cat(1, currentMatingLocusRearrangementProtocol, zeros(1, 13));

                                                            currentMatingLocusRearrangementProtocol(nMatingLocusRearrangements, 1) = 2;

                                                            currentMatingLocusRearrangementProtocol(nMatingLocusRearrangements, 2) = 2 * (genomeIndices(n, 1) - 1) + 3 - currentChromosome;
                                                            currentMatingLocusRearrangementProtocol(nMatingLocusRearrangements, 3) = homologousSite;
                                                            currentMatingLocusRearrangementProtocol(nMatingLocusRearrangements, 4) = matingLocusIdentifier;

                                                            currentMatingLocusRearrangementProtocol(nMatingLocusRearrangements, 8) = 2 * (genomeIndices(n, 1) - 1) + 3 - currentChromosome;
                                                            currentMatingLocusRearrangementProtocol(nMatingLocusRearrangements, 9) = swappingHomologousSite;
                                                            currentMatingLocusRearrangementProtocol(nMatingLocusRearrangements, 10) = matingLocusIdentifier;
                                                        end;
                                                    end;

                                                    currentRearrangementProtocol(n, 5) = 2 * (genomeIndices(n, 1) - 1) + 3 - currentChromosome;
                                                    currentRearrangementProtocol(n, 6) = homologousSite;
                                                    currentRearrangementProtocol(n, 7) = genomes(swappingHomologousSite, ...
                                                        2 * (genomeIndices(n, 1) - 1) + 3 - currentChromosome);
                                                    currentRearrangementProtocol(n, 11) = 2 * (genomeIndices(n, 1) - 1) + 3 - currentChromosome;
                                                    currentRearrangementProtocol(n, 12) = swappingHomologousSite;
                                                    currentRearrangementProtocol(n, 13) = swappingHomologousSiteBackup;

                                                end;
                                            end;
                                        end;
                                        
                                        currentRearrangementProtocol = cat(1, currentRearrangementProtocol, currentMatingLocusRearrangementProtocol);
                                    end;
                                    
                                    if silentIO ~= 1
                                        if (breedingSituation == 1) || subPopulationFlag == 1
                                            currentProtocolLength = nRealRearrangements;
                                            if startWithMeiosis
		                                        averageRearrangementsProtocol(floor(currentSimulationStep / 2), 1) = nRearrangements / (2 * nGenomes);
		                                        averageRealRearrangementsProtocol(floor(currentSimulationStep / 2), 1) = nRealRearrangements / (2 * nGenomes);                                            
                                            else
		                                        averageRearrangementsProtocol(ceil(currentSimulationStep / 2), 1) = nRearrangements / (2 * nGenomes);
		                                        averageRealRearrangementsProtocol(ceil(currentSimulationStep / 2), 1) = nRealRearrangements / (2 * nGenomes);
                                            end;
                                        else
                                            currentProtocolLength = nRearrangements + nMatingLocusRearrangements;
                                            if startWithMeiosis
                                            	averageRearrangementsProtocol(floor(currentSimulationStep / 2), 1) = (nRearrangements + nMatingLocusRearrangements) / (2 * nGenomes);
                                        	else
                                        		averageRearrangementsProtocol(ceil(currentSimulationStep / 2), 1) = (nRearrangements + nMatingLocusRearrangements) / (2 * nGenomes);
                                        	end;
                                        end;
                                    end;

                                    if displayInfo
                                        disp([num2str(nRearrangements) ' genomic rearrangements occurred during mitosis ' num2str(mitosisCounter)]);
                                    end;
                                    simulationProtocol(currentSimulationStep, 7) = nRearrangements;
                                else
                                    currentRearrangementProtocol = zeros(0, 13);
                                    currentProtocolLength = 0;
                                    if displayInfo
                                        disp(['No genomic rearrangements occurred during mitosis ' num2str(mitosisCounter)]);
                                    end;
                                    simulationProtocol(currentSimulationStep, 7) = 0;
                                end;
                                
                                if silentIO == 0
                                    if totalSimulationSteps > protocolSplittingSize
                                        [currentXSize currentYSize currentZSize] = size(rearrangementProtocol);
                                        if currentProtocolLength > currentXSize
                                            rearrangementProtocolBackup = rearrangementProtocol;
                                            rearrangementProtocol = zeros(currentProtocolLength, 13, protocolSplittingSize / 2, 'uint16');
                                            rearrangementProtocol(1:currentXSize, :, :) = rearrangementProtocolBackup(:, :, :);
                                            clear rearrangementProtocolBackup;
                                        end;

                                        [currentXSize currentYSize] = size(currentRearrangementProtocol);
                                        if startWithMeiosis
                                        	temporaryStepCounter = floor(mod(currentSimulationStep, protocolSplittingSize) / 2);
                                        else
                                        	temporaryStepCounter = ceil(mod(currentSimulationStep, protocolSplittingSize) / 2);
                                        end;
                                        if mod(temporaryStepCounter, protocolSplittingSize / 2) == 0
                                            if currentProtocolLength > 0
                                                rearrangementProtocol(1:currentXSize, 1:currentYSize, protocolSplittingSize / 2) = currentRearrangementProtocol(:, :);
                                            end;
                                            eval(['rearrangementProtocol' currentProtocolSectionFormatted ' = rearrangementProtocol;']);
                                            eval(['save protocols/rearrangementProtocol' currentProtocolSectionFormatted ' rearrangementProtocol' currentProtocolSectionFormatted ';']);
                                            eval(['clear rearrangementProtocol' currentProtocolSectionFormatted ';']);
                                            rearrangementProtocol = zeros(1, 13, protocolSplittingSize / 2, 'uint16');   
                                        else
                                            if currentProtocolLength > 0
                                                rearrangementProtocol(1:currentXSize, 1:currentYSize, temporaryStepCounter) = currentRearrangementProtocol(:, :);
                                            end;
                                        end;
                                    else
                                        [currentXSize currentYSize currentZSize] = size(rearrangementProtocol);
                                        if currentProtocolLength > currentXSize
                                            rearrangementProtocolBackup = rearrangementProtocol;
                                            if startWithMeiosis
                                            	rearrangementProtocol = zeros(currentProtocolLength, 13, floor(totalSimulationSteps / 2), 'uint16');
                                        	else
                                        		rearrangementProtocol = zeros(currentProtocolLength, 13, ceil(totalSimulationSteps / 2), 'uint16');
                                        	end;
                                            rearrangementProtocol(1:currentXSize, :, :) = rearrangementProtocolBackup(:, :, :);
                                            clear rearrangementProtocolBackup;

                                        end;
                                        [currentXSize currentYSize] = size(currentRearrangementProtocol);
                                        if currentProtocolLength > 0
                                        	if startWithMeiosis
                                        		rearrangementProtocol(1:currentXSize, 1:currentYSize, floor(currentSimulationStep / 2)) = currentRearrangementProtocol(:, :);
                                        	else
                                            	rearrangementProtocol(1:currentXSize, 1:currentYSize, ceil(currentSimulationStep / 2)) = currentRearrangementProtocol(:, :);
                                            end;
                                        end;
                                    end;
                                end;
                                
                                if startWithMeiosis
                                	populationSizeMitosisProtocol(floor(currentSimulationStep / 2), 1) = 2 * nGenomes;
                                else
                                	populationSizeMitosisProtocol(ceil(currentSimulationStep / 2), 1) = 2 * nGenomes;
                                end;
                            end;
                            
                            if deathmatchFlag
                                alphaSize = length(find(populationIdentities == 0));
                                betaSize = length(find(populationIdentities == 1));
                                if startWithMeiosis
                                	population1SizeMitosisProtocol(floor(currentSimulationStep / 2), 1) = alphaSize;
	                                population2SizeMitosisProtocol(floor(currentSimulationStep / 2), 1) = betaSize;
                                else
	                                population1SizeMitosisProtocol(ceil(currentSimulationStep / 2), 1) = alphaSize;
	                                population2SizeMitosisProtocol(ceil(currentSimulationStep / 2), 1) = betaSize;
	                            end;
                                
                                if size(genomes, 2) == 0
                                    defeatException = 1;
                                    defeatOutcome = 3;
                                    break;
                                else
                                    if alphaSize == 0
                                        defeatException = 1;
                                        defeatOutcome = 0;
                                        break;
                                    elseif betaSize == 0
                                        defeatException = 1;
                                        defeatOutcome = 1;
                                        break;
                                    end;
                                end;
                                
                                % optional control analysis:
                                % measuredContent = 0;
                                % for i = 1:(2 * nGenomes)
                                %     measuredContent = measuredContent + ...
                                %         min(mod(genomes(:, i), fullFitnessIdentifier) == primaryTemplate);
                                % end;
                                % [measuredContent sum(~populationIdentities)]
                                
                            end;
                            
                            
                            
                            %%%
                            %%% END OF MITOSIS
                            %%%
                            
                            

                        case 1 % Meiotic division
                            
                            if ~skipMeiosis

                                meiosisCounter = meiosisCounter + 1;
                                if displayInfo
                                    disp([':: Meiosis ' num2str(meiosisCounter) ' ::']);
                                end;

                                % -------------------------------------------------
                                % Meiotic Duplication
                                % Notes:
                                % Two genomes in data table before duplication:
                                % 1A-1B-2A-2B
                                % Same two genomes in data table after duplication:
                                % 1A-1A-1B-1B-2A-2A-2B-2B 
                                % -------------------------------------------------

                                if subPopulationFlag
                                    subPopulations = subPopulations(:, sort(cat(1, (1:nGenomes)', (1:nGenomes)')));
                                end;
                                
                                nGenomes = 2 * nGenomes;
                                genomeDuplicationArray = sort(cat(1, (1:nGenomes)', (1:nGenomes)'));
                                genomes = genomes(:, genomeDuplicationArray');
                                fitness = fitness(:, genomeDuplicationArray');
                                fitGenomes = ones(2 * nGenomes, 1, 'uint8');
                                if deathmatchFlag
                                    populationIdentities = populationIdentities(:, genomeDuplicationArray');
                                end;
                                if matingLocusFlag
                                    matingLoci = matingLoci(:, genomeDuplicationArray');
                                    matingTypes = matingTypes(:, genomeDuplicationArray');
                                end;

                                % ---------------------------------------------------------
                                % Recombination
                                % Notes:
                                % 1) Occurs for pairs (4n-2, 4n-1) throughout the genome
                                % 2) May only occur at proficient recombination sites,
                                % probability for recombination at a single PRS (i.e.
                                % neighbouring region on the other chromosome is not a PRS)
                                % is half as high as for a dual PRS
                                % 3) Reevaluation of fitness matrix after recombination
                                % ---------------------------------------------------------

                                % Calculation of recombination events
                                % based on the Erlang distribution

                                recombiningChromosomesLeftPartners =  4 .* (1:(nGenomes / 2)) - 2;
                                recombiningChromosomesRightPartners = 4 .* (1:(nGenomes / 2)) - 1;
                                recombinationSitesLeftPartners =  genomes(:, recombiningChromosomesLeftPartners')  == 1;
                                recombinationSitesRightPartners = genomes(:, recombiningChromosomesRightPartners') == 1;
                                meioticRecombinationSites = max(recombinationSitesLeftPartners, recombinationSitesRightPartners);
                                nMeioticRecombinationSites = sum(meioticRecombinationSites, 1);
                                nMaxMeioticRecombinationSites = max(nMeioticRecombinationSites);
                                if nMaxMeioticRecombinationSites > 0
                                    meioticRecombinationSitesPositions = zeros(nMaxMeioticRecombinationSites, nGenomes / 2);
                                    for n = 1:(nGenomes / 2)
                                        if nMeioticRecombinationSites(1, n) > 0
                                            meioticRecombinationSitesPositions(1:nMeioticRecombinationSites(1, n), n) = find(meioticRecombinationSites(:, n));
                                        end;
                                    end;
                                end;

                                recombinationSitesPositions = zeros(nMaxMeioticRecombinationSites, nGenomes / 2, 2);
                                nRecombinationEvents = zeros(nGenomes / 2, 1);

                                for n = 1:(nGenomes / 2)
                                    
                                    if deathmatchFlag == 1 && recombinationUnitFactorFlag == 2
                                        if populationIdentities(1, 4 * n) == 0
                                            erlangIntervals = erlangIntervals1;
                                        else
                                            erlangIntervals = erlangIntervals2;
                                        end;
                                    end;
                                    if deathmatchFlag == 1 && recombinationProbabilityFlag == 2
                                        if populationIdentities(1, 4 * n) == 0
                                            recombinationProbability = recombinationProbability1;
                                        else
                                            recombinationProbability = recombinationProbability2;
                                        end;
                                    end;
                                    if deathmatchFlag == 1 && recombinationModeFlag == 2
                                        if populationIdentities(1, 4 * n) == 0
                                            recombinationMode = recombinationMode1;
                                        else
                                            recombinationMode = recombinationMode2;
                                        end;
                                    end;

                                    if (recombinationMode == 0) % Erlang-based crossover interference

                                        if (nMeioticRecombinationSites(1, n) > 0) && (rand <= recombinationProbability)

                                            nUpstreamRecombinationEvents = 1;
                                            nDownstreamRecombinationEvents = 1;

                                            currentGenomicRegion = ceil((2 * nGenes + 1) .* rand([1 1]));
                                            currentBestHit = find(abs(meioticRecombinationSitesPositions(1:nMeioticRecombinationSites(1, n), n) - currentGenomicRegion) == ...
                                                min(abs(meioticRecombinationSitesPositions(1:nMeioticRecombinationSites(1, n), n) - currentGenomicRegion)));

                                            currentUpstreamRecombinationEvents = meioticRecombinationSitesPositions(currentBestHit(1, 1), n);
                                            currentUpstreamBestHits = currentBestHit(1, 1);
                                            currentDownstreamRecombinationEvents = meioticRecombinationSitesPositions(currentBestHit(1, 1), n);
                                            currentDownstreamBestHits = currentBestHit(1, 1);

                                            if erlangApplicationFlag == 0
                                                outOfRangeFlag = 0;
                                                while (~outOfRangeFlag) && ...
                                                        (currentUpstreamRecombinationEvents(nUpstreamRecombinationEvents, 1) ~= meioticRecombinationSitesPositions(nMeioticRecombinationSites(1, n), n))
                                                    erlangHits = find(erlangIntervals > rand);
                                                    currentGenomicRegion = currentUpstreamRecombinationEvents(nUpstreamRecombinationEvents, 1) + 2 * erlangHits(1, 1);
                                                    if currentGenomicRegion <= (2 * nGenes + 1)
                                                        currentBestHit = find(abs(meioticRecombinationSitesPositions((currentUpstreamBestHits(nUpstreamRecombinationEvents, 1) + 1):...
                                                            nMeioticRecombinationSites(1, n), n) - currentGenomicRegion) == ...
                                                            min(abs(meioticRecombinationSitesPositions((currentUpstreamBestHits(nUpstreamRecombinationEvents, 1) + 1):...
                                                            nMeioticRecombinationSites(1, n), n) - currentGenomicRegion)));
                                                        currentUpstreamRecombinationEvents = cat(1, currentUpstreamRecombinationEvents, ...
                                                            meioticRecombinationSitesPositions(currentUpstreamBestHits(nUpstreamRecombinationEvents, 1) + currentBestHit(1, 1), n));
                                                        currentUpstreamBestHits = cat(1, currentUpstreamBestHits, currentUpstreamBestHits(nUpstreamRecombinationEvents, 1) + currentBestHit(1, 1));
                                                        nUpstreamRecombinationEvents = nUpstreamRecombinationEvents + 1;
                                                    else
                                                        outOfRangeFlag = 1;
                                                    end;
                                                end;
                                            else
                                                outOfRangeFlag = 0;
                                                while (~outOfRangeFlag)
                                                    erlangHits = find(erlangIntervals > rand);
                                                    currentRPIEOffset = currentUpstreamBestHits(nUpstreamRecombinationEvents, 1) + erlangHits(1, 1);
                                                    if (currentRPIEOffset <= nMaxMeioticRecombinationSites) && (meioticRecombinationSitesPositions(currentRPIEOffset, n) ~= 0)
                                                        currentUpstreamBestHits = cat(1, currentUpstreamBestHits, currentRPIEOffset);
                                                        currentUpstreamRecombinationEvents = cat(1, currentUpstreamRecombinationEvents, ...
                                                            meioticRecombinationSitesPositions(currentRPIEOffset, n));
                                                        nUpstreamRecombinationEvents = nUpstreamRecombinationEvents + 1;
                                                    else
                                                        outOfRangeFlag = 1;
                                                    end;
                                                end;
                                            end;

                                            if erlangApplicationFlag == 0
                                                outOfRangeFlag = 0;
                                                while (~outOfRangeFlag) && ...
                                                        (currentDownstreamRecombinationEvents(nDownstreamRecombinationEvents, 1) ~= meioticRecombinationSitesPositions(1, n))
                                                    erlangHits = find(erlangIntervals > rand);
                                                    currentGenomicRegion = currentDownstreamRecombinationEvents(nDownstreamRecombinationEvents, 1) - 2 * erlangHits(1, 1);
                                                    if currentGenomicRegion >= 0
                                                        currentBestHit = find(abs(meioticRecombinationSitesPositions(1:(currentDownstreamBestHits(nDownstreamRecombinationEvents, 1) - 1), n) ...
                                                            - currentGenomicRegion) == ...
                                                            min(abs(meioticRecombinationSitesPositions(1:(currentDownstreamBestHits(nDownstreamRecombinationEvents, 1) - 1), n) - currentGenomicRegion)));
                                                        currentDownstreamRecombinationEvents = cat(1, currentDownstreamRecombinationEvents, ...
                                                            meioticRecombinationSitesPositions(currentBestHit(1, 1), n));
                                                        currentDownstreamBestHits = cat(1, currentDownstreamBestHits, currentBestHit(1, 1));
                                                        nDownstreamRecombinationEvents = nDownstreamRecombinationEvents + 1;
                                                    else
                                                        outOfRangeFlag = 1;
                                                    end;
                                                end;
                                            else
                                                outOfRangeFlag = 0;
                                                while (~outOfRangeFlag)
                                                    erlangHits = find(erlangIntervals > rand);
                                                    currentRPIEOffset = currentDownstreamBestHits(nDownstreamRecombinationEvents, 1) - erlangHits(1, 1);
                                                    if currentRPIEOffset > 0
                                                        currentDownstreamBestHits = cat(1, currentDownstreamBestHits, currentRPIEOffset);
                                                        currentDownstreamRecombinationEvents = cat(1, currentDownstreamRecombinationEvents, ...
                                                            meioticRecombinationSitesPositions(currentRPIEOffset, n));
                                                        nDownstreamRecombinationEvents = nDownstreamRecombinationEvents + 1;
                                                    else
                                                        outOfRangeFlag = 1;
                                                    end;
                                                end;
                                            end;

                                            nCurrentEvents = nUpstreamRecombinationEvents + nDownstreamRecombinationEvents - 1;
                                            currentEvents = cat(1, currentUpstreamRecombinationEvents, currentDownstreamRecombinationEvents(2:nDownstreamRecombinationEvents, :));

                                            nRecombinationEvents(n, 1) = nCurrentEvents;
                                            recombinationSitesPositions(1:nCurrentEvents, n, 1) = currentEvents(:, :);

                                        end;

                                    elseif randomApplicationFlag == 0 % No crossover interference, Erlang-based recombination event number

                                        if (nMeioticRecombinationSites(1, n) > 0) && (rand <= recombinationProbability)

                                            nUpstreamRecombinationEvents = 1;
                                            nDownstreamRecombinationEvents = 1;

                                            currentGenomicRegion = ceil((2 * nGenes + 1) .* rand([1 1]));
                                            currentBestHit = find(abs(meioticRecombinationSitesPositions(1:nMeioticRecombinationSites(1, n), n) - currentGenomicRegion) == ...
                                                min(abs(meioticRecombinationSitesPositions(1:nMeioticRecombinationSites(1, n), n) - currentGenomicRegion)));

                                            currentUpstreamRecombinationEvents = meioticRecombinationSitesPositions(currentBestHit(1, 1), n);
                                            currentUpstreamBestHits = currentBestHit(1, 1);
                                            currentDownstreamRecombinationEvents = meioticRecombinationSitesPositions(currentBestHit(1, 1), n);
                                            currentDownstreamBestHits = currentBestHit(1, 1);

                                            currentEvents = currentUpstreamRecombinationEvents;

                                            if erlangApplicationFlag == 0
                                                outOfRangeFlag = 0;
                                                while (~outOfRangeFlag) && ...
                                                        (currentUpstreamRecombinationEvents(nUpstreamRecombinationEvents, 1) ~= meioticRecombinationSitesPositions(nMeioticRecombinationSites(1, n), n))
                                                    erlangHits = find(erlangIntervals > rand);
                                                    currentGenomicRegion = currentUpstreamRecombinationEvents(nUpstreamRecombinationEvents, 1) + 2 * erlangHits(1, 1);
                                                    if currentGenomicRegion <= (2 * nGenes + 1)
                                                        currentBestHit = find(abs(meioticRecombinationSitesPositions((currentUpstreamBestHits(nUpstreamRecombinationEvents, 1) + 1):...
                                                            nMeioticRecombinationSites(1, n), n) - currentGenomicRegion) == ...
                                                            min(abs(meioticRecombinationSitesPositions((currentUpstreamBestHits(nUpstreamRecombinationEvents, 1) + 1):...
                                                            nMeioticRecombinationSites(1, n), n) - currentGenomicRegion)));
                                                        currentUpstreamRecombinationEvents = cat(1, currentUpstreamRecombinationEvents, ...
                                                            meioticRecombinationSitesPositions(currentUpstreamBestHits(nUpstreamRecombinationEvents, 1) + currentBestHit(1, 1), n));
                                                        currentUpstreamBestHits = cat(1, currentUpstreamBestHits, currentUpstreamBestHits(nUpstreamRecombinationEvents, 1) + currentBestHit(1, 1));

                                                        unclonedRandomEventComputed = 0;

                                                        while ~unclonedRandomEventComputed
                                                            currentHit = ceil(nMeioticRecombinationSites(1, n) .* rand([1 1]));
                                                            currentComputedSite = meioticRecombinationSitesPositions(currentHit, n);
                                                            unclonedRandomEventComputed = ~(length(find(currentEvents == currentComputedSite) > 0));
                                                            if unclonedRandomEventComputed
                                                                currentEvents = cat(1, currentEvents, currentComputedSite);
                                                            end;
                                                        end;

                                                        nUpstreamRecombinationEvents = nUpstreamRecombinationEvents + 1;
                                                    else
                                                        outOfRangeFlag = 1;
                                                    end;
                                                end;
                                            else
                                                outOfRangeFlag = 0;
                                                while (~outOfRangeFlag)
                                                    erlangHits = find(erlangIntervals > rand);
                                                    currentRPIEOffset = currentUpstreamBestHits(nUpstreamRecombinationEvents, 1) + erlangHits(1, 1);
                                                    if (currentRPIEOffset <= nMaxMeioticRecombinationSites) && (meioticRecombinationSitesPositions(currentRPIEOffset, n) ~= 0)
                                                        currentUpstreamBestHits = cat(1, currentUpstreamBestHits, currentRPIEOffset);
                                                        currentUpstreamRecombinationEvents = cat(1, currentUpstreamRecombinationEvents, ...
                                                            meioticRecombinationSitesPositions(currentRPIEOffset, n));

                                                        unclonedRandomEventComputed = 0;

                                                        while ~unclonedRandomEventComputed
                                                            currentHit = ceil(nMeioticRecombinationSites(1, n) .* rand([1 1]));
                                                            currentComputedSite = meioticRecombinationSitesPositions(currentHit, n);
                                                            unclonedRandomEventComputed = ~(length(find(currentEvents == currentComputedSite) > 0));
                                                            if unclonedRandomEventComputed
                                                                currentEvents = cat(1, currentEvents, currentComputedSite);
                                                            end;
                                                        end;

                                                        nUpstreamRecombinationEvents = nUpstreamRecombinationEvents + 1;
                                                    else
                                                        outOfRangeFlag = 1;
                                                    end;
                                                end;
                                            end;

                                            if erlangApplicationFlag == 0
                                                outOfRangeFlag = 0;
                                                while (~outOfRangeFlag) && ...
                                                        (currentDownstreamRecombinationEvents(nDownstreamRecombinationEvents, 1) ~= meioticRecombinationSitesPositions(1, n))
                                                    erlangHits = find(erlangIntervals > rand);
                                                    currentGenomicRegion = currentDownstreamRecombinationEvents(nDownstreamRecombinationEvents, 1) - 2 * erlangHits(1, 1);
                                                    if currentGenomicRegion >= 0
                                                        currentBestHit = find(abs(meioticRecombinationSitesPositions(1:(currentDownstreamBestHits(nDownstreamRecombinationEvents, 1) - 1), n) ...
                                                            - currentGenomicRegion) == ...
                                                            min(abs(meioticRecombinationSitesPositions(1:(currentDownstreamBestHits(nDownstreamRecombinationEvents, 1) - 1), n) - currentGenomicRegion)));
                                                        currentDownstreamRecombinationEvents = cat(1, currentDownstreamRecombinationEvents, ...
                                                            meioticRecombinationSitesPositions(currentBestHit(1, 1), n));
                                                        currentDownstreamBestHits = cat(1, currentDownstreamBestHits, currentBestHit(1, 1));

                                                        unclonedRandomEventComputed = 0;

                                                        while ~unclonedRandomEventComputed
                                                            currentHit = ceil(nMeioticRecombinationSites(1, n) .* rand([1 1]));
                                                            currentComputedSite = meioticRecombinationSitesPositions(currentHit, n);
                                                            unclonedRandomEventComputed = ~(length(find(currentEvents == currentComputedSite) > 0));
                                                            if unclonedRandomEventComputed
                                                                currentEvents = cat(1, currentEvents, currentComputedSite);
                                                            end;
                                                        end;

                                                        nDownstreamRecombinationEvents = nDownstreamRecombinationEvents + 1;
                                                    else
                                                        outOfRangeFlag = 1;
                                                    end;
                                                end;
                                            else
                                                outOfRangeFlag = 0;
                                                while (~outOfRangeFlag)
                                                    erlangHits = find(erlangIntervals > rand);
                                                    currentRPIEOffset = currentDownstreamBestHits(nDownstreamRecombinationEvents, 1) - erlangHits(1, 1);
                                                    if currentRPIEOffset > 0
                                                        currentDownstreamBestHits = cat(1, currentDownstreamBestHits, currentRPIEOffset);
                                                        currentDownstreamRecombinationEvents = cat(1, currentDownstreamRecombinationEvents, ...
                                                            meioticRecombinationSitesPositions(currentRPIEOffset, n));

                                                        unclonedRandomEventComputed = 0;

                                                        while ~unclonedRandomEventComputed
                                                            currentHit = ceil(nMeioticRecombinationSites(1, n) .* rand([1 1]));
                                                            currentComputedSite = meioticRecombinationSitesPositions(currentHit, n);
                                                            unclonedRandomEventComputed = ~(length(find(currentEvents == currentComputedSite) > 0));
                                                            if unclonedRandomEventComputed
                                                                currentEvents = cat(1, currentEvents, currentComputedSite);
                                                            end;
                                                        end;

                                                        nDownstreamRecombinationEvents = nDownstreamRecombinationEvents + 1;
                                                    else
                                                        outOfRangeFlag = 1;
                                                    end;
                                                end;
                                            end;

                                            nCurrentEvents = nUpstreamRecombinationEvents + nDownstreamRecombinationEvents - 1;

                                            nRecombinationEvents(n, 1) = nCurrentEvents;
                                            recombinationSitesPositions(1:nCurrentEvents, n, 1) = currentEvents(:, :);
                                            
                                        end;

                                    else % No crossover interference, fixed formula-based recombination event number

                                        if (nMeioticRecombinationSites(1, n) > 0) && (rand <= recombinationProbability)

                                            if erlangApplicationFlag == 0
                                                maxRecombinationEvents = round(mostLikelyEventNumber);
                                            else
                                                maxRecombinationEvents = round(mostLikelyEventNumber * nMeioticRecombinationSites(1, n) / nRPIE);
                                            end;

                                            if maxRecombinationEvents > nMeioticRecombinationSites(1, n)
                                                maxRecombinationEvents = nMeioticRecombinationSites(1, n);
                                            end;

                                            nCurrentEvents = maxRecombinationEvents;

                                            if maxRecombinationEvents > 0

                                                currentEvents = [];

                                                while maxRecombinationEvents > 0

                                                    currentHit = ceil(nMeioticRecombinationSites(1, n) .* rand([1 1]));
                                                    currentComputedSite = meioticRecombinationSitesPositions(currentHit, n);
                                                    unclonedRandomEventComputed = ~(length(find(currentEvents == currentComputedSite) > 0));
                                                    if unclonedRandomEventComputed
                                                        currentEvents = cat(1, currentEvents, currentComputedSite);
                                                        maxRecombinationEvents = maxRecombinationEvents - 1;
                                                    end;

                                                end;

                                            end;

                                            nRecombinationEvents(n, 1) = nCurrentEvents;
                                            recombinationSitesPositions(1:nCurrentEvents, n, 1) = currentEvents(:, :);

                                        end;

                                    end;
                                end;
                                
                                maxEvents = max(nRecombinationEvents);
                                if maxEvents > 1
                                    recombinationSitesPositions = sort(recombinationSitesPositions(1:max(nRecombinationEvents), :, :));
                                end;
                                

                                % -----------------------------------
                                % Recombination assignment code:
                                % Chromosome configuration: A A' B B'
                                % 1 = A  B
                                % 2 = A  B'
                                % 3 = A' B
                                % 4 = A' B'
                                % ------------------------------------                        

                                for n = 1:(nGenomes / 2)
                                    nCurrentEvents = nRecombinationEvents(n, 1);
                                    if nCurrentEvents > 0
                                        recombinationSitesPositions((maxEvents - nCurrentEvents + 1):maxEvents, n, 2) = ceil(4 .* rand([nCurrentEvents 1]));
                                    end;
                                end;

                                nTotalRecombinationEvents = sum(nRecombinationEvents) + sum(nRecombinationEvents > 0); % one additional recombination event per mating pair

                                % The recombination sites are defined as not being part of the
                                % genomic region that is swapped in each recombination step,
                                % i.e. they always remain on the original chromosome.

                                currentRecombinationProtocol = ones(nTotalRecombinationEvents - sum(nRecombinationEvents > 0), 3);
                                currentEventNumber = 1;

                                for n = 1:(nGenomes / 2)
                                    if (nRecombinationEvents(n, 1) ~= 0)
                                        currentRecombinationMatrix = zeros(nRecombinationEvents(n, 1) + 1, 5);
                                        currentRecombinationMatrix(1:nRecombinationEvents(n, 1), 1) = ...
                                            recombinationSitesPositions((maxEvents - nRecombinationEvents(n, 1) + 1):maxEvents, n, 1);
                                        currentRecombinationMatrix(nRecombinationEvents(n, 1) + 1, 1) = nGenomicElements + 1;
                                        currentRecombinationMatrix(1, 2:5) = [1 2 3 4];
                                        for k = 1:nRecombinationEvents(n, 1)

                                            currentEvent = recombinationSitesPositions(maxEvents - nRecombinationEvents(n, 1) + k, n, 2);
                                            currentRecombinationProtocol(currentEventNumber, 1) = n;
                                            currentRecombinationProtocol(currentEventNumber, 2) = currentRecombinationMatrix(k, 1);
                                            currentRecombinationProtocol(currentEventNumber, 3) = currentEvent;
                                            currentEventNumber = currentEventNumber + 1;

                                            % -------------------
                                            % currentEvent rules:
                                            % 1 = 1-3 crossover
                                            % 2 = 1-4 crossover
                                            % 3 = 2-3 crossover
                                            % 4 = 2-4 crossover
                                            % -------------------

                                            for m = 2:5
                                                if currentRecombinationMatrix(k, m) == 1
                                                    if currentEvent == 1
                                                        currentRecombinationMatrix(k + 1, m) = 3;
                                                    elseif currentEvent == 2
                                                        currentRecombinationMatrix(k + 1, m) = 4;
                                                    else
                                                        currentRecombinationMatrix(k + 1, m) = 1;
                                                    end;
                                                elseif currentRecombinationMatrix(k, m) == 2
                                                    if currentEvent == 3
                                                        currentRecombinationMatrix(k + 1, m) = 3;
                                                    elseif currentEvent == 4
                                                        currentRecombinationMatrix(k + 1, m) = 4;
                                                    else
                                                        currentRecombinationMatrix(k + 1, m) = 2;
                                                    end;
                                                elseif currentRecombinationMatrix(k, m) == 3
                                                    if currentEvent == 1
                                                        currentRecombinationMatrix(k + 1, m) = 1;
                                                    elseif currentEvent == 3
                                                        currentRecombinationMatrix(k + 1, m) = 2;
                                                    else
                                                        currentRecombinationMatrix(k + 1, m) = 3;
                                                    end;
                                                else
                                                    if currentEvent == 2
                                                        currentRecombinationMatrix(k + 1, m) = 1;
                                                    elseif currentEvent == 4
                                                        currentRecombinationMatrix(k + 1, m) = 2;
                                                    else
                                                        currentRecombinationMatrix(k + 1, m) = 4;
                                                    end;
                                                end;
                                            end;

                                        end;

                                        regionBackup = genomes(:, (4 * n - 3):(4 * n));
                                        previousRecombinationSite = currentRecombinationMatrix(1, 1);
                                        stateDetermined = [0 0 0 0];
                                        if matingLocusFlag
                                            oldMatingLoci = matingLoci(1, (4 * n - 3):(4 * n));
                                            oldMatingTypes = matingTypes(1, (4 * n - 3):(4 * n));
                                        end;

                                        for k = 1:nRecombinationEvents(n, 1)
                                            nextRecombinationSite = currentRecombinationMatrix(k + 1, 1);
                                            for m = 2:5
                                                currentInsertionID = currentRecombinationMatrix(k + 1, m);
                                                genomes((previousRecombinationSite + 1):(nextRecombinationSite - 1), 4 * n - (5 - m)) = ...
                                                    regionBackup((previousRecombinationSite + 1):(nextRecombinationSite - 1), currentInsertionID);
                                                if (stateDetermined(1, m - 1) == 0) && matingLocusFlag
                                                    if (matingLoci(1, (4 * n - 3) + currentInsertionID - 1) <= (nextRecombinationSite - 1)) && ...
                                                            (matingLoci(1, (4 * n - 3) + currentInsertionID - 1) >= (previousRecombinationSite + 1)) && ...
                                                            (matingLoci(1, 4 * n - (5 - m)) <= (nextRecombinationSite - 1)) && ...
                                                            (matingLoci(1, 4 * n - (5 - m)) >= (previousRecombinationSite + 1))
                                                        matingLoci(1, 4 * n - (5 - m)) = oldMatingLoci(1, currentInsertionID);
                                                        matingTypes(1, 4 * n - (5 - m)) = oldMatingTypes(1, currentInsertionID);                                                    
                                                        stateDetermined(1, m - 1) = 1;
                                                    end;
                                                end;
                                            end;
                                            previousRecombinationSite = nextRecombinationSite;
                                        end;

                                        for p = 1:4
                                            unmatchedSites = find(genomes(:, 4 * n - (4 - p)) ~= regionBackup(:, p));
                                            unmatchedIdentities = sort(cat(1, genomes(unmatchedSites, 4 * n - (4 - p)), regionBackup(unmatchedSites, p)));
                                            unmatchedIdentities = double(unmatchedIdentities) - ...
                                                double((unmatchedIdentities > noFitnessIdentifier) .* (noFitnessIdentifier - fullFitnessIdentifier));
                                            unmatchedIdentities = sort(unmatchedIdentities);
                                            if ~isempty(unmatchedIdentities)
                                                potentialSearchTargetsFlag = zeros(length(unmatchedIdentities), 1);
                                                for k = 1:length(unmatchedIdentities)
                                                    if k < length(unmatchedIdentities)
                                                        if (unmatchedIdentities(k, 1) > fullFitnessIdentifier) && (unmatchedIdentities(k, 1) ~= unmatchedIdentities(k + 1, 1))
                                                            potentialSearchTargetsFlag(k, 1) = 1;
                                                        end;
                                                    else
                                                        if (unmatchedIdentities(k, 1) > fullFitnessIdentifier)
                                                            potentialSearchTargetsFlag(k, 1) = 1;
                                                        end;
                                                    end;
                                                end;
                                                if sum(potentialSearchTargetsFlag) > 0
                                                    potentialSearchTargets = find(potentialSearchTargetsFlag == 1);
                                                    potentialSearchTargetIdentities = unmatchedIdentities(potentialSearchTargets, 1);
                                                    fitnessMatrixPositions = potentialSearchTargetIdentities - fullFitnessIdentifier;
                                                    for k = 1:length(fitnessMatrixPositions)
                                                        fitness(fitnessMatrixPositions(k, 1), 4 * n - (4 - p)) = ...
                                                            (~isempty(find(genomes(:, 4 * n - (4 - p)) == potentialSearchTargetIdentities(k, 1), 1)));
                                                    end;
                                                end;
                                            end;
                                        end;
                                    end;
                                end;
                                
                                if silentIO == 0
                                    if totalSimulationSteps > protocolSplittingSize
                                        [currentXSize currentYSize currentZSize] = size(recombinationProtocol);
                                        if nTotalRecombinationEvents > currentXSize
                                            recombinationProtocolBackup = recombinationProtocol;
                                            recombinationProtocol = zeros(nTotalRecombinationEvents, 3, protocolSplittingSize / 2, 'uint16');
                                            recombinationProtocol(1:currentXSize, :, :) = recombinationProtocolBackup(:, :, :);
                                            clear recombinationProtocolBackup;

                                        end;
                                        [currentXSize currentYSize] = size(currentRecombinationProtocol);
                                        if mod(currentSimulationStep, protocolSplittingSize) == 0
                                            recombinationProtocol(1:currentXSize, 1:currentYSize, protocolSplittingSize / 2) = currentRecombinationProtocol(:, :);
                                            eval(['recombinationProtocol' currentProtocolSectionFormatted ' = recombinationProtocol;']);
                                            eval(['save protocols/recombinationProtocol' currentProtocolSectionFormatted ' recombinationProtocol' currentProtocolSectionFormatted ';']);
                                            eval(['clear recombinationProtocol' currentProtocolSectionFormatted ';']);
                                            recombinationProtocol = zeros(1, 3, protocolSplittingSize / 2, 'uint16');

                                        else
                                        	if startWithMeiosis
                                        		recombinationProtocol(1:currentXSize, 1:currentYSize, ceil(mod(currentSimulationStep, protocolSplittingSize) / 2)) = currentRecombinationProtocol(:, :);
                                        	else
                                            	recombinationProtocol(1:currentXSize, 1:currentYSize, floor(mod(currentSimulationStep, protocolSplittingSize) / 2)) = currentRecombinationProtocol(:, :);
                                            end;
                                        end;
                                    else
                                        [currentXSize currentYSize currentZSize] = size(recombinationProtocol);
                                        if nTotalRecombinationEvents > currentXSize
                                            recombinationProtocolBackup = recombinationProtocol;
                                            recombinationProtocol = zeros(nTotalRecombinationEvents, 3, floor(totalSimulationSteps / 2), 'uint16');
                                            recombinationProtocol(1:currentXSize, :, :) = recombinationProtocolBackup(:, :, :);
                                            clear recombinationProtocolBackup;

                                        end;
                                        [currentXSize currentYSize] = size(currentRecombinationProtocol);
                                        if startWithMeiosis
                                        	recombinationProtocol(1:currentXSize, 1:currentYSize, ceil(currentSimulationStep / 2)) = currentRecombinationProtocol(:, :);
                                        else
                                        	recombinationProtocol(1:currentXSize, 1:currentYSize, floor(currentSimulationStep / 2)) = currentRecombinationProtocol(:, :);
                                        end;
                                    end;
                                end;

                                if displayInfo
                                    disp([num2str(nTotalRecombinationEvents) ' genomic recombinations occurred during meiosis ' num2str(meiosisCounter)]);
                                end;
                                simulationProtocol(currentSimulationStep, 8) = nTotalRecombinationEvents;
                                
                                if silentIO ~= 1
                                	if startWithMeiosis
                                		averageRecombinationsProtocol(ceil(currentSimulationStep / 2), 1) = 2 * sum(nRecombinationEvents) / nGenomes;
                                	else
                                    	averageRecombinationsProtocol(floor(currentSimulationStep / 2), 1) = 2 * sum(nRecombinationEvents) / nGenomes;
                                    end;
                                end;
                                
                                
                                % Meiotic Mutations:
                                
                                if sum(mutationMode ~= 0) > 0
                                    mutationDecisions = rand(nGenes, 2 * nGenomes) <= mitoticMutationRate;
                                    if deathmatchFlag && mutationModeFlag == 2
                                        if mutationMode1 == 0
                                            mutationDecisions(:, populationIdentities == 0) = 0;
                                        end;
                                        if mutationMode2 == 0
                                            mutationDecisions(:, populationIdentities == 1) = 0;
                                        end;
                                    end;
                                    [siteIndices genomeIndices] = find(mutationDecisions == 1);
                                    chromosomeIndices = mod((genomeIndices + 1), 2) + 1;
                                    genomeIndices = uint32(genomeIndices ./ 2);
                                    nMutations = size(genomeIndices, 1);

                                    if nMutations > 0

                                        currentMutationProtocol = ones(0, 5);
                                        nRealMutations = 0;

                                        for n = 1:nMutations
                                            currentChromosomeIndex = 2 * (genomeIndices(n, 1) - 1) + chromosomeIndices(n, 1);
                                            currentGeneIndex = 2 * siteIndices(n, 1) - 1;
                                            backupChromosomeIndex = 2 * (genomeIndices(n, 1) - 1) + 3 - chromosomeIndices(n, 1);
                                            currentGene = genomes(currentGeneIndex, currentChromosomeIndex);

                                            if (currentGene > fullFitnessIdentifier) && (currentGene < noFitnessIdentifier)
                                                currentMutationProtocol = cat(1, currentMutationProtocol, ones(0, 5));
                                                nRealMutations = nRealMutations + 1;
                                                currentMutationProtocol(nRealMutations, 1) = currentChromosomeIndex;
                                                currentMutationProtocol(nRealMutations, 2) = currentGeneIndex;
                                                currentMutationProtocol(nRealMutations, 3) = currentGene;

                                                genePositions = find(genomes(:, currentChromosomeIndex) == currentGene);
                                                foundGenes = size(genePositions, 1);
                                                if (foundGenes == 1)
                                                    fitness(currentGene - fullFitnessIdentifier, currentChromosomeIndex) = 0;
                                                end;
                                                genomes(currentGeneIndex, currentChromosomeIndex) = currentGene - fullFitnessIdentifier + noFitnessIdentifier;

                                                currentMutationProtocol(nRealMutations, 4) = currentGene - fullFitnessIdentifier + noFitnessIdentifier;
                                            end;
                                        end;

                                        if displayInfo
                                            disp([num2str(mitoticMutationRate, '%.6f') ' is the current meiotic mutation rate']);
                                            disp([num2str(nMutations) ' mutations occured during meiosis ' num2str(meiosisCounter) ', affecting ' ...
                                                num2str(nRealMutations) ' functional genes']);
                                        end;
                                        simulationProtocol(currentSimulationStep, 2) = mitoticMutationRate;
                                        simulationProtocol(currentSimulationStep, 3) = nMutations;
                                        simulationProtocol(currentSimulationStep, 4) = nRealMutations;
                                    else
                                        currentMutationProtocol = ones(0, 5);
                                        nRealMutations = 0;

                                        if displayInfo
                                            disp([num2str(mitoticMutationRate, '%.6f') ' is the current meiotic mutation rate']);
                                            disp(['No mutations occurred during meiosis ' num2str(meiosisCounter)]);
                                        end;
                                        simulationProtocol(currentSimulationStep, 2) = mitoticMutationRate;
                                        simulationProtocol(currentSimulationStep, 3) = 0;
                                        simulationProtocol(currentSimulationStep, 4) = 0;
                                    end;
                                end;

                                
                                % --------------------------------------------------------
                                % Mating
                                % Notes:
                                % 1) Determination of inbreeding and outbreeding partners
                                % 2) Determination of viability of resulting genomes, i.e.
                                %    test for homologous fraction and genome fitness
                                % 3) Restructuring of genome matrix
                                % --------------------------------------------------------
                                
                                
                                preMatingGenomes = nGenomes;
                                
                                if matingLocusFlag && externalLocus
                                    for currentTetrad = 1:round(nGenomes / 2)
                                        if rand <= 0.5
                                            matingTypes(1, (4 * (currentTetrad - 1) + 1):(4 * currentTetrad)) = ...
                                                - matingTypes(1, (4 * (currentTetrad - 1) + 1):(4 * currentTetrad));
                                        end;
                                    end;
                                end;
                                
                                if deathmatchFlag
                                    firstPopulation = length(find(populationIdentities == 0));
                                    secondPopulation = length(find(populationIdentities == 1));
                                    if firstPopulation ~= 0 && secondPopulation ~= 0
                                        separateMating = 1;
                                        genomes1 = genomes(:, 1:firstPopulation);
                                        genomes2 = genomes(:, (firstPopulation + 1):(firstPopulation + secondPopulation));
                                        fitness1 = fitness(:, 1:firstPopulation);
                                        fitness2 = fitness(:, (firstPopulation + 1):(firstPopulation + secondPopulation));
                                        populationIdentities1 = populationIdentities(:, 1:firstPopulation);
                                        populationIdentities2 = populationIdentities(:, (firstPopulation + 1):(firstPopulation + secondPopulation));
                                        if matingLocusFlag
                                            matingLoci1 = matingLoci(:, 1:firstPopulation);
                                            matingLoci2 = matingLoci(:, (firstPopulation + 1):(firstPopulation + secondPopulation));
                                            matingTypes1 = matingTypes(:, 1:firstPopulation);
                                            matingTypes2 = matingTypes(:, (firstPopulation + 1):(firstPopulation + secondPopulation));
                                        end;
                                        nGenomes1 = firstPopulation / 2;
                                        nGenomes2 = secondPopulation / 2;
                                        if inbreedingFlag == 1
                                            nInbreedingEvents1 = ceil(inbreedingFraction * nGenomes1);
                                            nInbreedingEvents2 = ceil(inbreedingFraction * nGenomes2);
                                        elseif inbreedingFlag == 2
                                            nInbreedingEvents1 = ceil(inbreedingFraction(1, 1) * nGenomes1);
                                            nInbreedingEvents2 = ceil(inbreedingFraction(1, 2) * nGenomes2);
                                        end;
                                    elseif firstPopulation == 0
                                        separateMating = 0;
                                        if inbreedingFlag == 1
                                            nInbreedingEvents = ceil(inbreedingFraction * nGenomes);
                                        elseif inbreedingFlag == 2
                                            nInbreedingEvents = ceil(inbreedingFraction(1, 2) * nGenomes);
                                        end;
                                    else
                                        separateMating = 0;
                                        if inbreedingFlag == 1
                                            nInbreedingEvents = ceil(inbreedingFraction * nGenomes);
                                        elseif inbreedingFlag == 2
                                            nInbreedingEvents = ceil(inbreedingFraction(1, 1) * nGenomes);
                                        end;
                                    end;
                                else
                                    separateMating = 0;
                                    nInbreedingEvents = ceil(inbreedingFraction * nGenomes);
                                end;
                                
                                if subPopulationFlag
                                    currentSubPopulations = sort(subPopulations);
                                    currentSubPopulationIndices = [];
                                    previousEntry = -1;
                                    for i = 1:length(currentSubPopulations)
                                        if previousEntry ~= currentSubPopulations(1, i)
                                            currentSubPopulationIndices = cat(1, ...
                                                currentSubPopulationIndices, currentSubPopulations(1, i));
                                            previousEntry = currentSubPopulations(1, i);
                                        end;
                                    end;
                                    separateMating = length(currentSubPopulationIndices) - 1;
                                    singlePopulationLookupTable = cell(length(currentSubPopulationIndices), 1);
                                    doublePopulationLookupTable = cell(length(currentSubPopulationIndices), 1);
                                    for i = 1:length(currentSubPopulationIndices)
                                        singlePopulationLookupTable{i, 1} = ...
                                            find(subPopulations == currentSubPopulationIndices(i, 1));
                                        doublePopulationLookupTable{i, 1} = ...
                                            sort(cat(2, (2 .* singlePopulationLookupTable{i, 1} - 1), ...
                                            (2 .* singlePopulationLookupTable{i, 1})));
                                    end;
                                    backupGenomes = genomes;
                                    backupFitness = fitness;
                                    backupSubpopulations = subPopulations;
                                    if matingLocusFlag
                                        backupMatingLoci = matingLoci;
                                        backupMatingTypes = matingTypes;
                                    end;
                                end;
                                
                                matingAnomaly = 0;
                                
                                for i = 1:(separateMating + 1)
                                    if ~subPopulationFlag && separateMating
                                        switch i
                                            case 1
                                                genomes = genomes1;
                                                fitness = fitness1;
                                                populationIdentities = populationIdentities1;
                                                if matingLocusFlag
                                                    matingLoci = matingLoci1;
                                                    matingTypes = matingTypes1;
                                                end;
                                                nGenomes = nGenomes1;
                                                nInbreedingEvents = nInbreedingEvents1;
                                                recombinationProbability = recombinationProbability1;
                                                if recombinationProbability == 0 && inbreedingFraction(1, 1) == 1
                                                    matingAnomaly = 1;
                                                else
                                                    matingAnomaly = 0;
                                                end;
                                                if matingSwitchingFlag == 2
                                                	matingSwitching = matingSwitching1;
                                                end;
                                            case 2
                                                genomes = genomes2;
                                                fitness = fitness2;
                                                populationIdentities = populationIdentities2;
                                                if matingLocusFlag
                                                    matingLoci = matingLoci2;
                                                    matingTypes = matingTypes2;
                                                end;
                                                nGenomes = nGenomes2;
                                                nInbreedingEvents = nInbreedingEvents2;
                                                recombinationProbability = recombinationProbability2; 
                                                if length(inbreedingFraction) == 1
                                                    if recombinationProbability == 0 && inbreedingFraction(1, 1) == 1
                                                        matingAnomaly = 1;
                                                    else
                                                        matingAnomaly = 0;
                                                    end;
                                                elseif length(inbreedingFraction) == 2
                                                    if recombinationProbability == 0 && inbreedingFraction(1, 2) == 1
                                                        matingAnomaly = 1;
                                                    else
                                                        matingAnomaly = 0;
                                                    end;
                                                end;
                                                if matingSwitchingFlag == 2
                                                	matingSwitching = matingSwitching2;
                                                end;
                                        end;
                                    elseif subPopulationFlag
                                        genomes = backupGenomes(:, doublePopulationLookupTable{i, 1});
                                        fitness = backupFitness(:, doublePopulationLookupTable{i, 1});
                                        subPopulations = backupSubpopulations(:, singlePopulationLookupTable{i, 1});
                                        if matingLocusFlag
                                            matingLoci = backupMatingLoci(:, doublePopulationLookupTable{i, 1});
                                            matingTypes = backupMatingTypes(:, doublePopulationLookupTable{i, 1});
                                        end;
                                        nGenomes = length(singlePopulationLookupTable{i, 1});
                                        nInbreedingEvents = ceil(inbreedingFraction * nGenomes);
                                    end;
                                
                                    if matingLocusFlag == 0

                                        if nInbreedingEvents > 1
                                            inbreedingParticipants = (1:nGenomes)';
                                            while length(inbreedingParticipants) > nInbreedingEvents
                                                inbreedingParticipants(ceil(length(inbreedingParticipants) .* rand([1 1])), :) = [];
                                            end;
                                            inbreedingPairing = [];
                                            outbreedingPairing = (1:(2 * nGenomes))';
                                            skipFlag = 0;
                                            for n = 1:nInbreedingEvents
                                                if skipFlag == 0
                                                    currentGenome = inbreedingParticipants(nInbreedingEvents - n + 1, 1);
                                                    currentChromosome = ceil(2 .* rand([1 1])) - 1;
                                                    if mod(currentGenome, 2) == 1 % odd genome number
                                                        if (n > 1) && (inbreedingParticipants(nInbreedingEvents - n + 2, 1) == currentGenome + 1)
                                                            if ~matingAnomaly
                                                                inbreedingPairing = cat(1, inbreedingPairing, (2 * currentGenome - 1 + randperm(4) - 1)');
                                                            else
                                                                if rand > 0.5
                                                                    inbreedingPairing = cat(1, inbreedingPairing, (2 * currentGenome - 1 + [1 3 2 4] - 1)');
                                                                else
                                                                    inbreedingPairing = cat(1, inbreedingPairing, (2 * currentGenome - 1 + [1 4 2 3] - 1)');
                                                                end;
                                                            end;
                                                            outbreedingPairing((2 * currentGenome - 1):(2 * currentGenome + 2), :) = [];
                                                            skipFlag = 1;
                                                        else
                                                            if ~matingAnomaly
                                                                pairedChromosome = (0:3)';
                                                                pairedChromosome(currentChromosome + 1, :) = [];
                                                                pairedChromosome = pairedChromosome(randperm(3), :);
                                                                pairedChromosome = pairedChromosome(1, 1);
                                                                currentChromosomePair = 2 * currentGenome - 1 + sort(cat(1, currentChromosome, pairedChromosome));
                                                                inbreedingPairing = cat(1, inbreedingPairing, currentChromosomePair);
                                                                outbreedingPairing(currentChromosomePair, :) = [];
                                                            else
                                                                pairedChromosome = ceil(2 .* rand([1 1])) + 1;
                                                                currentChromosomePair = 2 * currentGenome - 1 + sort(cat(1, currentChromosome, pairedChromosome));
                                                                inbreedingPairing = cat(1, inbreedingPairing, currentChromosomePair);
                                                                outbreedingPairing(currentChromosomePair, :) = [];
                                                            end;
                                                        end;                            
                                                    else % even genome number
                                                        if (n < nInbreedingEvents) && (inbreedingParticipants(nInbreedingEvents - n, 1) == currentGenome - 1)
                                                            if ~matingAnomaly
                                                                inbreedingPairing = cat(1, inbreedingPairing, (2 * currentGenome - 3 + randperm(4) - 1)');
                                                            else
                                                                if rand > 0.5
                                                                    inbreedingPairing = cat(1, inbreedingPairing, (2 * currentGenome - 3 + [1 3 2 4] - 1)');
                                                                else
                                                                    inbreedingPairing = cat(1, inbreedingPairing, (2 * currentGenome - 3 + [1 4 2 3] - 1)');
                                                                end;
                                                            end;
                                                            outbreedingPairing((2 * currentGenome - 3):(2 * currentGenome), :) = [];
                                                            skipFlag = 1;
                                                        else
                                                            if ~matingAnomaly
                                                                pairedChromosome = (0:3)';
                                                                pairedChromosome(currentChromosome + 3, :) = [];
                                                                pairedChromosome = pairedChromosome(randperm(3), :);
                                                                pairedChromosome = pairedChromosome(1, 1);
                                                                currentChromosomePair = 2 * currentGenome - 3 + sort(cat(1, currentChromosome + 2, pairedChromosome));
                                                                inbreedingPairing = cat(1, inbreedingPairing, currentChromosomePair);
                                                                outbreedingPairing(currentChromosomePair, :) = [];
                                                            else
                                                                pairedChromosome = ceil(2 .* rand([1 1])) + 1;
                                                                currentChromosomePair = 2 * currentGenome - 3 + sort(cat(1, currentChromosome + 2, pairedChromosome));
                                                                inbreedingPairing = cat(1, inbreedingPairing, currentChromosomePair);
                                                                outbreedingPairing(currentChromosomePair, :) = [];
                                                            end;
                                                        end; 
                                                    end;
                                                else
                                                    skipFlag = 0;
                                                end;
                                            end;
                                            outbreedingPairing = outbreedingPairing(randperm(length(outbreedingPairing)), :);
                                        elseif nInbreedingEvents == 1
                                            inbreedingPairing = zeros(2, 1);
                                            outbreedingPairing = (1:(2 * nGenomes))';
                                            randomGenomePair = ceil((nGenomes / 2) .* rand([1 1]));
                                            if ~matingAnomaly
                                                randomChromosomeIndex = (0:3)';
                                                randomChromosomeIndex(ceil(4 .* rand([1 1])), :) = [];
                                                randomChromosomeIndex(ceil(3 .* rand([1 1])), :) = [];
                                            else
                                                randomChromosomeIndex = [ceil(2 .* rand([1 1])) - 1; ceil(2 .* rand([1 1])) + 1];
                                            end;
                                            inbreedingPairing(1, 1) = 4 * randomGenomePair - randomChromosomeIndex(2, 1);
                                            inbreedingPairing(2, 1) = 4 * randomGenomePair - randomChromosomeIndex(1, 1);
                                            if inbreedingPairing(2, 1) > inbreedingPairing(1, 1)
                                                outbreedingPairing(inbreedingPairing(2, 1), :) = [];
                                                outbreedingPairing(inbreedingPairing(1, 1), :) = [];
                                            else
                                                outbreedingPairing(inbreedingPairing(1, 1), :) = [];
                                                outbreedingPairing(inbreedingPairing(2, 1), :) = [];
                                            end;
                                            outbreedingPairing = outbreedingPairing(randperm(2 * nGenomes - 2), :);
                                        else % outbreeding only
                                            inbreedingPairing = [];
                                            outbreedingPairing = randperm(2 * nGenomes)';
                                        end;

                                        outbreedingOrigins = ceil(outbreedingPairing ./ 4);
                                        falseOutbreeders = 0;
                                        falseOutbreedersPositions = [];
                                        if ~isempty(outbreedingOrigins)
                                            for n = 1:(length(outbreedingOrigins) / 2)
                                                if outbreedingOrigins(2 * n - 1, 1) == outbreedingOrigins(2 * n, 1)
                                                    falseOutbreeders = falseOutbreeders + 1;
                                                    falseOutbreedersPositions = cat(1, falseOutbreedersPositions, ((2 * n - 1):(2 * n))');
                                                end;
                                            end;
                                        end;
                                        if (falseOutbreeders > 0) && (length(outbreedingOrigins) > 2)
                                            for n = 1:falseOutbreeders
                                                newDecisionArray = outbreedingPairing;
                                                oldBreedingPartner = newDecisionArray(falseOutbreedersPositions(2 * n, 1), 1);
                                                newDecisionArray(falseOutbreedersPositions((2 * n - 1):(2 * n), 1), :) = [];                    
                                                newBreedingPartner = newDecisionArray(ceil(length(newDecisionArray) .* rand([1 1])), 1);
                                                newBreedingPartnerPosition = find(outbreedingPairing == newBreedingPartner);
                                                outbreedingPairing(falseOutbreedersPositions(2 * n, 1), 1) = newBreedingPartner;
                                                outbreedingPairing(newBreedingPartnerPosition, 1) = oldBreedingPartner;
                                            end;
                                        end;
                                        outbreedingOrigins = ceil(outbreedingPairing ./ 4);
                                        falseOutbreeders = 0;
                                        if ~isempty(outbreedingOrigins)
                                            for n = 1:(length(outbreedingOrigins) / 2)
                                                if outbreedingOrigins(2 * n - 1, 1) == outbreedingOrigins(2 * n, 1)
                                                    falseOutbreeders = falseOutbreeders + 1;
                                                end;
                                            end;
                                        end;

                                    else % active mating locus

                                        if nInbreedingEvents > 1
                                            inbreedingParticipants = (1:nGenomes)';
                                            while length(inbreedingParticipants) > nInbreedingEvents
                                                inbreedingParticipants(ceil(length(inbreedingParticipants) .* rand([1 1])), :) = [];
                                            end;
                                            inbreedingPairing = [];
                                            outbreedingPairing = (1:(2 * nGenomes))';
                                            skipFlag = 0;
                                            for n = 1:nInbreedingEvents
                                                if skipFlag == 0
                                                    currentGenome = inbreedingParticipants(nInbreedingEvents - n + 1, 1);
                                                    currentChromosome = ceil(2 .* rand([1 1])) - 1;
                                                    if mod(currentGenome, 2) == 1 % odd genome number
                                                        if (n > 1) && (inbreedingParticipants(nInbreedingEvents - n + 2, 1) == currentGenome + 1)
                                                            if matingTypes(1, 2 * currentGenome - 1) == matingTypes(1, 2 * currentGenome)
                                                                firstDecision = ceil(2 .* rand([1 1]));
                                                                inbreedingPairing = cat(1, inbreedingPairing, 2 * currentGenome - 1 + [0 1 + firstDecision 1 4 - firstDecision]');
                                                            elseif matingTypes(1, 2 * currentGenome - 1) == matingTypes(1, 2 * currentGenome + 1)
                                                                firstDecision = ceil(2 .* rand([1 1])) - 1;
                                                                inbreedingPairing = cat(1, inbreedingPairing, 2 * currentGenome - 1 + [0 1 + 2 * firstDecision 2 3 - 2 * firstDecision]');
                                                            else
                                                                firstDecision = ceil(2 .* rand([1 1]));
                                                                inbreedingPairing = cat(1, inbreedingPairing, 2 * currentGenome - 1 + [0 firstDecision 3 3 - firstDecision]');
                                                            end;
                                                            outbreedingPairing((2 * currentGenome - 1):(2 * currentGenome + 2), :) = [];
                                                            skipFlag = 1;
                                                        else
                                                            pairedChromosome = (0:3)';
                                                            pairedChromosome(currentChromosome + 1, :) = [];
                                                            pairedChromosome = pairedChromosome(randperm(3), :);
                                                            chosenChromosome = pairedChromosome(1, 1);
                                                            if matingTypes(1, 2 * currentGenome - 1 + currentChromosome) == matingTypes(1, 2 * currentGenome - 1 + chosenChromosome)
                                                                chosenChromosome = pairedChromosome(2, 1);
                                                            end;
                                                            currentChromosomePair = 2 * currentGenome - 1 + sort(cat(1, currentChromosome, chosenChromosome));
                                                            inbreedingPairing = cat(1, inbreedingPairing, currentChromosomePair);
                                                            outbreedingPairing(currentChromosomePair, :) = [];
                                                        end;                            
                                                    else % even genome number
                                                        if (n < nInbreedingEvents) && (inbreedingParticipants(nInbreedingEvents - n, 1) == currentGenome - 1)
                                                            if matingTypes(1, 2 * currentGenome - 3) == matingTypes(1, 2 * currentGenome - 2)
                                                                firstDecision = ceil(2 .* rand([1 1]));
                                                                inbreedingPairing = cat(1, inbreedingPairing, 2 * currentGenome - 3 + [0 1 + firstDecision 1 4 - firstDecision]');
                                                            elseif matingTypes(1, 2 * currentGenome - 3) == matingTypes(1, 2 * currentGenome - 1)
                                                                firstDecision = ceil(2 .* rand([1 1])) - 1;
                                                                inbreedingPairing = cat(1, inbreedingPairing, 2 * currentGenome - 3 + [0 1 + 2 * firstDecision 2 3 - 2 * firstDecision]');
                                                            else
                                                                firstDecision = ceil(2 .* rand([1 1]));
                                                                inbreedingPairing = cat(1, inbreedingPairing, 2 * currentGenome - 3 + [0 firstDecision 3 3 - firstDecision]');
                                                            end;
                                                            outbreedingPairing((2 * currentGenome - 3):(2 * currentGenome), :) = [];
                                                            skipFlag = 1;
                                                        else
                                                            pairedChromosome = (0:3)';
                                                            pairedChromosome(currentChromosome + 3, :) = [];
                                                            pairedChromosome = pairedChromosome(randperm(3), :);
                                                            chosenChromosome = pairedChromosome(1, 1);
                                                            if matingTypes(1, 2 * currentGenome - 3 + currentChromosome + 2) == matingTypes(1, 2 * currentGenome - 3 + chosenChromosome)
                                                                chosenChromosome = pairedChromosome(2, 1);
                                                            end;
                                                            currentChromosomePair = 2 * currentGenome - 3 + sort(cat(1, currentChromosome + 2, chosenChromosome));
                                                            inbreedingPairing = cat(1, inbreedingPairing, currentChromosomePair);
                                                            outbreedingPairing(currentChromosomePair, :) = [];
                                                        end; 
                                                    end;
                                                else
                                                    skipFlag = 0;
                                                end;
                                            end;
                                            if ~isempty(outbreedingPairing)
                                                firstMatingTypeIndividuals = find(matingTypes(1, outbreedingPairing) == 1);
                                                secondMatingTypeIndividuals = find(matingTypes(1, outbreedingPairing) == -1);
                                                firstTypeOutbreeders = outbreedingPairing(firstMatingTypeIndividuals, :);
                                                secondTypeOutbreeders = outbreedingPairing(secondMatingTypeIndividuals, :);
                                                outbreedingPairing = zeros(2 * length(firstTypeOutbreeders), 1);
                                                outbreedingPairing(1:2:(2 * length(firstTypeOutbreeders) - 1), 1) = firstTypeOutbreeders(randperm(length(firstTypeOutbreeders)), :);
                                                outbreedingPairing(2:2:(2 * length(firstTypeOutbreeders)), 1) = secondTypeOutbreeders(randperm(length(secondTypeOutbreeders)), :);
                                            end;
                                        elseif nInbreedingEvents == 1
                                            inbreedingPairing = zeros(2, 1);
                                            outbreedingPairing = (1:(2 * nGenomes))';
                                            randomGenomePair = ceil((nGenomes / 2) .* rand([1 1]));

                                            currentChromosome = ceil(4 .* rand([1 1])) - 1;
                                            pairedChromosome = (0:3)';
                                            pairedChromosome(currentChromosome + 1, :) = [];
                                            pairedChromosome = pairedChromosome(randperm(3), :);
                                            chosenChromosome = pairedChromosome(1, 1);
                                            if matingTypes(1, 4 * randomGenomePair - currentChromosome) == matingTypes(1, 4 * randomGenomePair - chosenChromosome)
                                                chosenChromosome = pairedChromosome(2, 1);
                                            end;

                                            inbreedingPairing(1, 1) = 4 * randomGenomePair - currentChromosome;
                                            inbreedingPairing(2, 1) = 4 * randomGenomePair - chosenChromosome;
                                            if inbreedingPairing(2, 1) > inbreedingPairing(1, 1)
                                                outbreedingPairing(inbreedingPairing(2, 1), :) = [];
                                                outbreedingPairing(inbreedingPairing(1, 1), :) = [];
                                            else
                                                outbreedingPairing(inbreedingPairing(1, 1), :) = [];
                                                outbreedingPairing(inbreedingPairing(2, 1), :) = [];
                                            end;

                                            if ~isempty(outbreedingPairing)
                                                firstMatingTypeIndividuals = find(matingTypes(1, outbreedingPairing) == 1);
                                                secondMatingTypeIndividuals = find(matingTypes(1, outbreedingPairing) == -1);
                                                firstTypeOutbreeders = outbreedingPairing(firstMatingTypeIndividuals, :);
                                                secondTypeOutbreeders = outbreedingPairing(secondMatingTypeIndividuals, :);
                                                outbreedingPairing = zeros(2 * length(firstTypeOutbreeders), 1);
                                                outbreedingPairing(1:2:(2 * length(firstTypeOutbreeders) - 1), 1) = firstTypeOutbreeders(randperm(length(firstTypeOutbreeders)), :);
                                                outbreedingPairing(2:2:(2 * length(firstTypeOutbreeders)), 1) = secondTypeOutbreeders(randperm(length(secondTypeOutbreeders)), :);
                                            end;
                                        else % outbreeding only
                                            inbreedingPairing = [];
                                            outbreedingPairing = (1:(2 * nGenomes))';

                                            if ~isempty(outbreedingPairing)
                                                firstMatingTypeIndividuals = find(matingTypes(1, outbreedingPairing) == 1);
                                                secondMatingTypeIndividuals = find(matingTypes(1, outbreedingPairing) == -1);
                                                firstTypeOutbreeders = outbreedingPairing(firstMatingTypeIndividuals, :);
                                                secondTypeOutbreeders = outbreedingPairing(secondMatingTypeIndividuals, :);
                                                outbreedingPairing = zeros(2 * length(firstTypeOutbreeders), 1);
                                                outbreedingPairing(1:2:(2 * length(firstTypeOutbreeders) - 1), 1) = firstTypeOutbreeders(randperm(length(firstTypeOutbreeders)), :);
                                                outbreedingPairing(2:2:(2 * length(firstTypeOutbreeders)), 1) = secondTypeOutbreeders(randperm(length(secondTypeOutbreeders)), :);
                                            end;
                                        end;

                                        outbreedingOrigins = ceil(outbreedingPairing ./ 4);
                                        falseOutbreeders = 0;
                                        falseOutbreedersPositions = [];
                                        if ~isempty(outbreedingOrigins)
                                            for n = 1:(length(outbreedingOrigins) / 2)
                                                if outbreedingOrigins(2 * n - 1, 1) == outbreedingOrigins(2 * n, 1)
                                                    falseOutbreeders = falseOutbreeders + 1;
                                                    falseOutbreedersPositions = cat(1, falseOutbreedersPositions, ((2 * n - 1):(2 * n))');
                                                end;
                                            end;
                                        end;
                                        if (falseOutbreeders > 0) && (length(outbreedingOrigins) > 2)
                                            for n = 1:falseOutbreeders
                                                firstMatingTypeIndividuals = find(matingTypes(1, outbreedingPairing) == 1);
                                                secondMatingTypeIndividuals = find(matingTypes(1, outbreedingPairing) == -1);
                                                firstTypeOutbreeders = outbreedingPairing(firstMatingTypeIndividuals, :);
                                                secondTypeOutbreeders = outbreedingPairing(secondMatingTypeIndividuals, :);

                                                oldBreedingPartner = outbreedingPairing(falseOutbreedersPositions(2 * n, 1), 1);

                                                if matingTypes(1, oldBreedingPartner) == 1
                                                    localPosition = find(firstTypeOutbreeders == oldBreedingPartner);
                                                    firstTypeOutbreeders(localPosition, :) = [];
                                                    newBreedingPartner = firstTypeOutbreeders(ceil(length(firstTypeOutbreeders) .* rand([1 1])), 1);
                                                    newBreedingPartnerPosition = find(outbreedingPairing == newBreedingPartner);
                                                else
                                                    localPosition = find(secondTypeOutbreeders == oldBreedingPartner);
                                                    secondTypeOutbreeders(localPosition, :) = [];
                                                    newBreedingPartner = secondTypeOutbreeders(ceil(length(secondTypeOutbreeders) .* rand([1 1])), 1);
                                                    newBreedingPartnerPosition = find(outbreedingPairing == newBreedingPartner);
                                                end;

                                                outbreedingPairing(falseOutbreedersPositions(2 * n, 1), 1) = newBreedingPartner;
                                                outbreedingPairing(newBreedingPartnerPosition, 1) = oldBreedingPartner;
                                            end;
                                        end;

                                        outbreedingOrigins = ceil(outbreedingPairing ./ 4);
                                        falseOutbreeders = 0;
                                        if ~isempty(outbreedingOrigins)
                                            for n = 1:(length(outbreedingOrigins) / 2)
                                                if outbreedingOrigins(2 * n - 1, 1) == outbreedingOrigins(2 * n, 1)
                                                    falseOutbreeders = falseOutbreeders + 1;
                                                end;
                                            end;
                                        end;

                                    end;
                                
                                    currentMatingProtocol = zeros(2 * nGenomes, 4);
                                    currentMatingProtocol(:, 1) = cat(1, inbreedingPairing, outbreedingPairing);

                                    % ----------------------------------
                                    % | Tests for fitness and homology |
                                    % ----------------------------------
                                    
                                    switchingDecisions = rand(nGenomes, 1) < matingSwitching;

                                	for n = 1:nGenomes
                                		if switchingDecisions(n, 1) == 0
                                			currentHealthyGeneSites = sum((fitness(:, currentMatingProtocol(2 * n - 1, 1)) + fitness(:, currentMatingProtocol(2 * n, 1))) > 0);
	                                        if currentHealthyGeneSites == nEssentialGenes
	                                            currentMatingProtocol((2 * n - 1):(2 * n), 2) = 1;
	
	                                            if breedingSituation == 0 && subPopulationFlag == 0
	                                                
	                                                % Paired chromosomes fullfil fitness conditions, but
	                                                % the homologous fraction still has to be determined
	
	                                                firstMatingPartnerSimplifiedGenome = genomes(:, currentMatingProtocol(2 * n - 1, 1)) - ...
	                                                    uint16((genomes(:, currentMatingProtocol(2 * n - 1, 1)) > noFitnessIdentifier) .* (noFitnessIdentifier - fullFitnessIdentifier));
	                                                secondMatingPartnerSimplifiedGenome = genomes(:, currentMatingProtocol(2 * n, 1)) - ...
	                                                    uint16((genomes(:, currentMatingProtocol(2 * n, 1)) > noFitnessIdentifier) .* (noFitnessIdentifier - fullFitnessIdentifier));
	
	                                                nHomologousSites = sum((firstMatingPartnerSimplifiedGenome .* uint16(firstMatingPartnerSimplifiedGenome > fullFitnessIdentifier)) == ...
	                                                    (secondMatingPartnerSimplifiedGenome .* uint16(secondMatingPartnerSimplifiedGenome > fullFitnessIdentifier)));
	
	                                                currentMatingProtocol((2 * n - 1):(2 * n), 3) = (nHomologousSites - nGenomicElements + nEssentialGenes) / nEssentialGenes;
	                                                if currentMatingProtocol(2 * n - 1, 3) >= requiredHomologousFraction
	                                                    currentMatingProtocol((2 * n - 1):(2 * n), 4) = 1;
	                                                end;
	                                            else
	                                                currentMatingProtocol((2 * n - 1):(2 * n), 4) = 1;
	                                            end;
	                                        end;
                                		end;
                                	end;
                                	
                                	survivingChromosomes = currentMatingProtocol(:, 1) .* currentMatingProtocol(:, 4);
                                    nSurvivingChromosomes = sum(currentMatingProtocol(:, 4));
                                	
                                	switchingChromosomesDecisions = zeros(2 * nGenomes, 1);
                                	for n = 1:nGenomes
                                		switchingChromosomesDecisions((2 * n - 1):(2 * n), 1) = switchingDecisions(n, 1);
                                	end;
                                	switchingChromosomes = currentMatingProtocol(logical(switchingChromosomesDecisions), 1);
                                	
                                	if ~isempty(switchingChromosomes)
                                		nSwitchingAttempts = length(switchingChromosomes);
                                		switchingFitness = fitness(:, switchingChromosomes');
                                		survivingSwitchers = (sum(switchingFitness) == nEssentialGenes)';
                                		survivingSwitchers = switchingChromosomes(survivingSwitchers);
                                		nSuccessfulSwitchingAttempts = length(survivingSwitchers);
                                		
                                		if nSuccessfulSwitchingAttempts > 0
                                			switchingDuplicationArray = sort(cat(1, survivingSwitchers, survivingSwitchers));
                                            subPopulationSelectionArray = ceil(survivingSwitchers / 2);
                                			
                                    		switchingFitness = fitness(:, switchingDuplicationArray);
                                    		switchingGenomes = genomes(:, switchingDuplicationArray);
                                    		
                                    		if subPopulationFlag
			                                    switchingSubPopulations = subPopulations(:, subPopulationSelectionArray);
			                                end;
			                                if deathmatchFlag
			                                    switchingPopulationIdentities = populationIdentities(:, switchingDuplicationArray);
			                                end;
			                                if matingLocusFlag
			                                    switchingMatingLoci = matingLoci(:, switchingDuplicationArray);
			                                    switchingMatingTypes = matingTypes(:, switchingDuplicationArray);
			                                    switchingMatingTypes(:, 2:2:(2 * nSuccessfulSwitchingAttempts)) = - switchingMatingTypes(:, 2:2:(2 * nSuccessfulSwitchingAttempts));
			                                end;
                                		end;
                                	else
                                		nSwitchingAttempts = 0;
                                		nSuccessfulSwitchingAttempts = 0;
                                	end;
                                    
                                    survivalList = [];
                                    deathList = [];
                                    if nSurvivingChromosomes > 0
                                        for n = 1:nGenomes
                                            if survivingChromosomes(2 * n - 1, 1) ~= 0
                                                survivalList = cat(1, survivalList, survivingChromosomes((2 * n - 1):(2 * n), 1));
                                            else
                                                deathList = cat(1, deathList, currentMatingProtocol((2 * n - 1):(2 * n), 1));
                                            end;
                                        end;
                                    end;
                                    
                                    if subPopulationFlag
                                        subPopulations = subPopulations(:, 1:(nSurvivingChromosomes / 2));
                                    end;

                                    nKilledGenomes = nGenomes - nSurvivingChromosomes / 2;
                                    nGenomes = nSurvivingChromosomes / 2;
                                    nKillFractionLoad = sum(sum(~fitness(:, deathList')));

                                    genomes = genomes(:, survivalList');
                                    fitness = fitness(:, survivalList');
                                    if deathmatchFlag
                                        populationIdentities = populationIdentities(:, survivalList');
                                    end;
                                    if matingLocusFlag
                                        matingLoci = matingLoci(:, survivalList');
                                        matingTypes = matingTypes(:, survivalList');
                                    end;

                                    if nInbreedingEvents > 0
                                        nSuccessfulInbreedingEvents = sum(currentMatingProtocol(1:(2 * nInbreedingEvents), 4)) / 2;
                                        if (nGenomes - nInbreedingEvents) > 0
                                            nSuccessfulOutbreedingEvents = sum(currentMatingProtocol((2 * nInbreedingEvents + 1):(2 * nGenomes), 4)) / 2;
                                        else
                                            nSuccessfulOutbreedingEvents = 0;
                                        end;
                                    else
                                        nSuccessfulInbreedingEvents = 0;
                                        nSuccessfulOutbreedingEvents = sum(currentMatingProtocol(:, 4)) / 2;
                                    end;
                                    
                                    if nSuccessfulSwitchingAttempts > 0
                                		fitness = cat(2, fitness, switchingFitness);
                                		genomes = cat(2, genomes, switchingGenomes);
                                		
                                		if subPopulationFlag
		                                    subPopulations = cat(2, subPopulations, switchingSubPopulations);
		                                end;
		                                if deathmatchFlag
		                                    populationIdentities = cat(2, populationIdentities, switchingPopulationIdentities);
		                                end;
		                                if matingLocusFlag
		                                    matingLoci = cat(2, matingLoci, switchingMatingLoci);
		                                    matingTypes = cat(2, matingTypes, switchingMatingTypes);
		                                end;
		                                nGenomes = nGenomes + nSuccessfulSwitchingAttempts;
                            		end;
                                    
                                    if ~subPopulationFlag && separateMating
                                        switch i
                                            case 1
                                                genomes1 = genomes;
                                                fitness1 = fitness;
                                                populationIdentities1 = populationIdentities;
                                                if matingLocusFlag
                                                    matingLoci1 = matingLoci;
                                                    matingTypes1 = matingTypes;
                                                end;
                                                nGenomes1 = nGenomes;
                                                nInbreedingEvents1 = nInbreedingEvents;
                                                currentMatingProtocol1 = currentMatingProtocol;
                                                nSuccessfulInbreedingEvents1 = nSuccessfulInbreedingEvents;
                                                nSuccessfulOutbreedingEvents1 = nSuccessfulOutbreedingEvents;
                                                nKilledGenomes1 = nKilledGenomes;
                                                nKillFractionLoad1 = nKillFractionLoad;
                                                falseOutbreeders1 = falseOutbreeders;
                                                nSwitchingAttempts1 = nSwitchingAttempts;
                                				nSuccessfulSwitchingAttempts1 = nSuccessfulSwitchingAttempts;
                                            case 2
                                                genomes2 = genomes;
                                                fitness2 = fitness;
                                                populationIdentities2 = populationIdentities;
                                                if matingLocusFlag
                                                    matingLoci2 = matingLoci;
                                                    matingTypes2 = matingTypes;
                                                end;
                                                nGenomes2 = nGenomes;
                                                nInbreedingEvents2 = nInbreedingEvents;
                                                currentMatingProtocol2 = currentMatingProtocol;
                                                currentMatingProtocol2(:, 1) = currentMatingProtocol2(:, 1) + 2 * nGenomes1;
                                                nSuccessfulInbreedingEvents2 = nSuccessfulInbreedingEvents;
                                                nSuccessfulOutbreedingEvents2 = nSuccessfulOutbreedingEvents;
                                                nKilledGenomes2 = nKilledGenomes;
                                                nKillFractionLoad2 = nKillFractionLoad;
                                                falseOutbreeders2 = falseOutbreeders;
                                                nSwitchingAttempts2 = nSwitchingAttempts;
                                				nSuccessfulSwitchingAttempts2 = nSuccessfulSwitchingAttempts;
                                        end;
                                    elseif subPopulationFlag
                                        if i == 1
                                            newGenomes = [];
                                            newFitness = [];
                                            newSubpopulations = [];
                                            if matingLocusFlag
                                                newMatingLoci = [];
                                                newMatingTypes = [];
                                            end;
                                            newNGenomes = 0;
                                            newNInbreedingEvents = 0;
                                            newCurrentMatingProtocol = [];
                                            newNSuccessfulInbreedingEvents = 0;
                                            newNSuccessfulOutbreedingEvents = 0;
                                            newNKilledGenomes = 0;
                                            newNKillFractionLoad = 0;
                                            newFalseOutbreeders = 0;
                                            newNSwitchingAttempts = 0;
                                			newNSuccessfulSwitchingAttempts = 0;
                                        end;
                                        newGenomes = cat(2, newGenomes, genomes);
                                        newFitness = cat(2, newFitness, fitness);
                                        newSubpopulations = cat(2, newSubpopulations, subPopulations);
                                        if matingLocusFlag
                                            newMatingLoci = cat(2, newMatingLoci, matingLoci);
                                            newMatingTypes = cat(2, newMatingTypes, matingTypes);
                                        end;
                                        newNGenomes = newNGenomes + nGenomes;
                                        newNInbreedingEvents = newNInbreedingEvents + nInbreedingEvents;
                                        newCurrentMatingProtocol = cat(1, newCurrentMatingProtocol, currentMatingProtocol);
                                        newNSuccessfulInbreedingEvents = newNSuccessfulInbreedingEvents + nSuccessfulInbreedingEvents;
                                        newNSuccessfulOutbreedingEvents = newNSuccessfulOutbreedingEvents + nSuccessfulOutbreedingEvents;
                                        newNKilledGenomes = newNKilledGenomes + nKilledGenomes;
                                        newNKillFractionLoad = newNKillFractionLoad + nKillFractionLoad;
                                        newFalseOutbreeders = newFalseOutbreeders + falseOutbreeders;
                                        newNSwitchingAttempts = newNSwitchingAttempts + nSwitchingAttempts;
                                		newNSuccessfulSwitchingAttempts = newNSuccessfulSwitchingAttempts + nSuccessfulSwitchingAttempts;
                                    end;
                                    
                                end;
                                
                                
                                if ~subPopulationFlag && separateMating
                                    genomes = cat(2, genomes1, genomes2);
                                    fitness = cat(2, fitness1, fitness2);
                                    populationIdentities = cat(2, populationIdentities1, populationIdentities2);
                                    if matingLocusFlag
                                        matingLoci = cat(2, matingLoci1, matingLoci2);
                                        matingTypes = cat(2, matingTypes1, matingTypes2);
                                    end;
                                    nGenomes = nGenomes1 + nGenomes2;
                                    nInbreedingEvents = nInbreedingEvents1 + nInbreedingEvents2;
                                    currentMatingProtocol = cat(1, currentMatingProtocol1, currentMatingProtocol2);
                                    nSuccessfulInbreedingEvents = nSuccessfulInbreedingEvents1 + nSuccessfulInbreedingEvents2;
                                    nSuccessfulOutbreedingEvents = nSuccessfulOutbreedingEvents1 + nSuccessfulOutbreedingEvents2;
                                    nKilledGenomes = nKilledGenomes1 + nKilledGenomes2;
                                    nKillFractionLoad = nKillFractionLoad1 + nKillFractionLoad2;
                                    falseOutbreeders = falseOutbreeders1 + falseOutbreeders2;
                                    nSwitchingAttempts = nSwitchingAttempts1 + nSwitchingAttempts2;
                                	nSuccessfulSwitchingAttempts = nSuccessfulSwitchingAttempts1 + nSuccessfulSwitchingAttempts2;
                                elseif subPopulationFlag
                                    genomes = newGenomes;
                                    fitness = newFitness;
                                    subPopulations = newSubpopulations;
                                    if matingLocusFlag
                                        matingLoci = newMatingLoci;
                                        matingTypes = newMatingTypes;
                                    end;
                                    nGenomes = newNGenomes;
                                    nInbreedingEvents = newNInbreedingEvents;
                                    currentMatingProtocol = newCurrentMatingProtocol;
                                    nSuccessfulInbreedingEvents = newNSuccessfulInbreedingEvents;
                                    nSuccessfulOutbreedingEvents = newNSuccessfulOutbreedingEvents;
                                    nKilledGenomes = newNKilledGenomes;
                                    nKillFractionLoad = newNKillFractionLoad;
                                    falseOutbreeders = newFalseOutbreeders;
                                    nSwitchingAttempts = newNSwitchingAttempts;
                                	nSuccessfulSwitchingAttempts = newNSuccessfulSwitchingAttempts;
                                    clear newGenomes newFitness newSubpopulations newMatingLoci newMatingTypes;
                                    clear newNGenomes newNInbreedingEvents newCurrentMatingProtocol newNSuccessfulInbreedingEvents;
                                    clear newNSuccessfulOutbreedingEvents newNKilledGenomes newNKillFractionLoad newFalseOutbreeders;
                                    clear newNSwitchingAttempts newNSuccessfulSwitchingAttempts;
                                end;
                                
                                if silentIO ~= 1
                                	if startWithMeiosis
                                		dropoutLoadMeiosisProtocol(ceil(currentSimulationStep / 2), 1) = ...
	                                        nKilledGenomes;
	                                    dropoutLoadMeiosisProtocol(ceil(currentSimulationStep / 2), 2) = ...
	                                        nKillFractionLoad;
                                	else
	                                    dropoutLoadMeiosisProtocol(floor(currentSimulationStep / 2), 1) = ...
	                                        nKilledGenomes;
	                                    dropoutLoadMeiosisProtocol(floor(currentSimulationStep / 2), 2) = ...
	                                        nKillFractionLoad;
	                                end;
                                end;
                                
                                if silentIO == 0
                                    if totalSimulationSteps > protocolSplittingSize
                                        if mod(currentSimulationStep, protocolSplittingSize) == 0
                                            matingProtocol(1:(2 * preMatingGenomes), :, protocolSplittingSize / 2) = currentMatingProtocol(:, :);
                                            eval(['matingProtocol' currentProtocolSectionFormatted ' = matingProtocol;']);
                                            eval(['save protocols/matingProtocol' currentProtocolSectionFormatted ' matingProtocol' currentProtocolSectionFormatted ';']);
                                            eval(['clear matingProtocol' currentProtocolSectionFormatted ';']);
                                            matingProtocol = zeros(4 * maxGenomes, 4, floor(protocolSplittingSize / 2), 'uint16');

                                        else
                                        	if startWithMeiosis
                                        		matingProtocol(1:(2 * preMatingGenomes), :, ceil(mod(currentSimulationStep, protocolSplittingSize) / 2)) = currentMatingProtocol(:, :);
                                        	else
                                            	matingProtocol(1:(2 * preMatingGenomes), :, floor(mod(currentSimulationStep, protocolSplittingSize) / 2)) = currentMatingProtocol(:, :);
                                            end;
                                        end;
                                    else
                                    	if startWithMeiosis
                                        	matingProtocol(1:(2 * preMatingGenomes), :, ceil(currentSimulationStep / 2)) = currentMatingProtocol(:, :);
                                        else
                                        	matingProtocol(1:(2 * preMatingGenomes), :, floor(currentSimulationStep / 2)) = currentMatingProtocol(:, :);
                                        end;
                                    end;
                                end;

                                if displayInfo
                                    disp([num2str(nInbreedingEvents) ' inbreeding attempts occurred during meiosis ' num2str(meiosisCounter)]);
                                end;
                                simulationProtocol(currentSimulationStep, 9) = nInbreedingEvents;
                                
                                if displayInfo
                                    disp([num2str(preMatingGenomes - nInbreedingEvents) ' outbreeding attempts occurred during meiosis ' num2str(meiosisCounter)]);
                                end;
                                simulationProtocol(currentSimulationStep, 11) = preMatingGenomes - nInbreedingEvents;
                                
                                if silentIO ~= 1
                                	if startWithMeiosis
                                		inbreedingAttemptsProtocol(ceil(currentSimulationStep / 2), 1) = nInbreedingEvents;
	                                    outbreedingAttemptsProtocol(ceil(currentSimulationStep / 2), 1) = preMatingGenomes - nInbreedingEvents;
	                                    falseOutbreedersProtocol(ceil(currentSimulationStep / 2), 1) = falseOutbreeders;
                                	else
	                                    inbreedingAttemptsProtocol(floor(currentSimulationStep / 2), 1) = nInbreedingEvents;
	                                    outbreedingAttemptsProtocol(floor(currentSimulationStep / 2), 1) = preMatingGenomes - nInbreedingEvents;
	                                    falseOutbreedersProtocol(floor(currentSimulationStep / 2), 1) = falseOutbreeders;
	                                end;
                                end;
                                
                                if displayInfo
                                    disp([num2str(nSuccessfulInbreedingEvents) ' successful inbreeding attempts occurred during meiosis ' num2str(meiosisCounter)]);
                                end;
                                simulationProtocol(currentSimulationStep, 10) = nSuccessfulInbreedingEvents;
                                if displayInfo
                                    disp([num2str(nSuccessfulOutbreedingEvents) ' successful outbreeding attempts occurred during meiosis ' num2str(meiosisCounter)]);
                                end;
                                simulationProtocol(currentSimulationStep, 12) = nSuccessfulOutbreedingEvents;
                                
                                if displayInfo
                                    disp([num2str(nSwitchingAttempts) ' mating type switching attempts occurred during meiosis ' num2str(meiosisCounter)]);
                                end;
                                if displayInfo
                                    disp([num2str(nSuccessfulSwitchingAttempts) ' successful mating type switching attempts occurred during meiosis ' num2str(meiosisCounter)]);
                                end;
                                simulationProtocol(currentSimulationStep, 13) = nSwitchingAttempts;
                                simulationProtocol(currentSimulationStep, 14) = nSuccessfulSwitchingAttempts;
                                
                                if silentIO ~= 1
                                	if startWithMeiosis
	                                    successfulInbreedingsProtocol(ceil(currentSimulationStep / 2), 1) = nSuccessfulInbreedingEvents;
	                                    successfulOutbreedingsProtocol(ceil(currentSimulationStep / 2), 1) = nSuccessfulOutbreedingEvents;
	                                
	                                    if inbreedingAttemptsProtocol(ceil(currentSimulationStep / 2), 1) > 0
	                                        inbreedingSuccessRatio(ceil(currentSimulationStep / 2), 1) = successfulInbreedingsProtocol(ceil(currentSimulationStep / 2), 1) / inbreedingAttemptsProtocol(ceil(currentSimulationStep / 2), 1);
	                                    end;
	                                    if outbreedingAttemptsProtocol(ceil(currentSimulationStep / 2), 1) > 0
	                                        outbreedingSuccessRatio(ceil(currentSimulationStep / 2), 1) = successfulOutbreedingsProtocol(ceil(currentSimulationStep / 2), 1) / outbreedingAttemptsProtocol(ceil(currentSimulationStep / 2), 1);
	                                    end;
	                                else
	                                	successfulInbreedingsProtocol(floor(currentSimulationStep / 2), 1) = nSuccessfulInbreedingEvents;
	                                    successfulOutbreedingsProtocol(floor(currentSimulationStep / 2), 1) = nSuccessfulOutbreedingEvents;
	                                
	                                    if inbreedingAttemptsProtocol(floor(currentSimulationStep / 2), 1) > 0
	                                        inbreedingSuccessRatio(floor(currentSimulationStep / 2), 1) = successfulInbreedingsProtocol(floor(currentSimulationStep / 2), 1) / inbreedingAttemptsProtocol(floor(currentSimulationStep / 2), 1);
	                                    end;
	                                    if outbreedingAttemptsProtocol(floor(currentSimulationStep / 2), 1) > 0
	                                        outbreedingSuccessRatio(floor(currentSimulationStep / 2), 1) = successfulOutbreedingsProtocol(floor(currentSimulationStep / 2), 1) / outbreedingAttemptsProtocol(floor(currentSimulationStep / 2), 1);
	                                    end;
	                                end;
                                end;
                                
                                lackOfHomologyDeletions = (sum(currentMatingProtocol(:, 2)) - sum(currentMatingProtocol(:, 4))) / 2;
                                simulationProtocol(currentSimulationStep, 15) = lackOfHomologyDeletions;
                                
                                if displayInfo
                                    disp([num2str(lackOfHomologyDeletions) ' genomes died of lacking homology during meiosis ' num2str(meiosisCounter)]);
                                end;
                                
                                if silentIO ~= 1
                                	if startWithMeiosis
	                                	lackingHomologyKillsProtocol(ceil(currentSimulationStep / 2), 1) = 2 * lackOfHomologyDeletions;
	                                    zeroFitnessKillsMeiosisProtocol(ceil(currentSimulationStep / 2), 1) = 2 * (nKilledGenomes - lackOfHomologyDeletions);
                                	else
                                		lackingHomologyKillsProtocol(floor(currentSimulationStep / 2), 1) = 2 * lackOfHomologyDeletions;
                                    	zeroFitnessKillsMeiosisProtocol(floor(currentSimulationStep / 2), 1) = 2 * (nKilledGenomes - lackOfHomologyDeletions);
                                	end;
                                end;

                                % If there are more genomes than the maximum allowed number
                                % (after subtraction of non-viable genomes or genomes
                                % with a too low homologous fraction), random kills occur,
                                % unless populationCapSize is set to zero

                                if populationCapSize && (nGenomes > maxGenomes)
                                    nGenomesToDelete = nGenomes - maxGenomes;
                                    genomesToDelete = ceil(nGenomes .* rand([nGenomesToDelete 1]));
                                    for n = 1:nGenomesToDelete
                                        duplicateFound = 1;
                                        while duplicateFound
                                            objectOccurence = find(genomesToDelete == genomesToDelete(n, 1));
                                            numberOfObjects = size(objectOccurence, 1);
                                            if numberOfObjects > 1
                                                genomesToDelete(n, 1) = ceil(nGenomes .* rand([1 1]));
                                            else
                                                duplicateFound = 0;
                                            end;
                                        end;
                                    end;
                                    genomesToDelete = sort(genomesToDelete);
                                    fullDeletionList = zeros(nGenomesToDelete, 2);
                                    fullDeletionList(:, 1) = 2 .* genomesToDelete(:) - 1;
                                    fullDeletionList(:, 2) = 2 .* genomesToDelete(:);
                                    fullDeletionList = sort(cat(1, fullDeletionList(:, 1), fullDeletionList(:, 2)));
                                    
                                    if silentIO == 0
                                        if totalSimulationSteps > protocolSplittingSize
                                            if mod(currentSimulationStep, protocolSplittingSize) == 0
                                                starvationProtocol(1:length(fullDeletionList), protocolSplittingSize) = fullDeletionList(:, :);
                                                eval(['starvationProtocol' currentProtocolSectionFormatted ' = starvationProtocol;']);
                                                eval(['save protocols/starvationProtocol' currentProtocolSectionFormatted ' starvationProtocol' currentProtocolSectionFormatted ';']);
                                                eval(['clear starvationProtocol' currentProtocolSectionFormatted ';']);
                                                starvationProtocol = zeros(2 * maxGenomes, protocolSplittingSize, 'uint16');

                                            else
                                                starvationProtocol(1:length(fullDeletionList), mod(currentSimulationStep, protocolSplittingSize)) = fullDeletionList(:, :);
                                            end;
                                        else
                                            starvationProtocol(1:length(fullDeletionList), currentSimulationStep) = fullDeletionList(:, :);
                                        end;
                                    end;

                                    survivingGenomes = (1:(2 * nGenomes))';
                                    for n = 1:(2 * nGenomesToDelete)
                                        survivingGenomes(fullDeletionList(2 * nGenomesToDelete - n + 1, 1), :) = [];
                                    end;
                                    
                                    if subPopulationFlag
                                        simpleSurvivalList = (1:nGenomes)';
                                        for n = 1:nGenomesToDelete
                                            simpleSurvivalList(genomesToDelete(nGenomesToDelete - n + 1, 1), :) = [];
                                        end;
                                        subPopulations = subPopulations(:, simpleSurvivalList');
                                    end;
                                    
                                    if silentIO ~= 1
                                    	if startWithMeiosis
                                    		dropoutLoadMeiosisProtocol(ceil(currentSimulationStep / 2), 3) = ...
	                                            nGenomesToDelete;
	                                        dropoutLoadMeiosisProtocol(ceil(currentSimulationStep / 2), 4) = ...
	                                            sum(sum(~fitness(:, fullDeletionList)));
                                    	else
	                                        dropoutLoadMeiosisProtocol(floor(currentSimulationStep / 2), 3) = ...
	                                            nGenomesToDelete;
	                                        dropoutLoadMeiosisProtocol(floor(currentSimulationStep / 2), 4) = ...
	                                            sum(sum(~fitness(:, fullDeletionList)));
	                                    end;        
                                    end;

                                    genomes = genomes(:, survivingGenomes');
                                    fitness = fitness(:, survivingGenomes');
                                    if deathmatchFlag
                                        populationIdentities = populationIdentities(:, survivingGenomes');
                                    end;
                                    if matingLocusFlag
                                        matingLoci = matingLoci(:, survivingGenomes');
                                        matingTypes = matingTypes(:, survivingGenomes');
                                    end;

                                    nGenomes = maxGenomes;
                                    if displayInfo
                                        disp([num2str(nGenomesToDelete) ' genomes died of starvation during meiosis ' num2str(meiosisCounter)]);
                                    end;
                                    simulationProtocol(currentSimulationStep, 6) = nGenomesToDelete;
                                    
                                    if silentIO ~= 1
                                    	if startWithMeiosis
                                    		starvationKillsMeiosisProtocol(ceil(currentSimulationStep / 2), 1) = 2 * nGenomesToDelete;
                                    	else	
                                        	starvationKillsMeiosisProtocol(floor(currentSimulationStep / 2), 1) = 2 * nGenomesToDelete;
                                        end;	
                                    end;

                                else
                                    if displayInfo
                                        disp(['No genomes died of starvation during meiosis ' num2str(meiosisCounter)]);
                                    end;
                                    simulationProtocol(currentSimulationStep, 6) = 0;
                                end;
                                
                                
                                if breedingSituation == 0 && subPopulationFlag == 0

                                    % -----------------------------------------
                                    % | Test for non-zero recombination sites |
                                    % -----------------------------------------

                                    if nGenomes > 0
                                        nonZeroProficientRecombinationSitesFlag = (sum(genomes(:, 2 .* (1:nGenomes) - 1) == 1) + sum(genomes(:, 2 .* (1:nGenomes)) == 1)) > 0;
                                        nBadGenomes = nGenomes - sum(nonZeroProficientRecombinationSitesFlag);

                                        zeroPRChromosomesFlag = zeros(1, 2 * nGenomes);
                                        zeroPRChromosomesFlag(:, 2 .* (1:nGenomes) - 1) = 1 - nonZeroProficientRecombinationSitesFlag;
                                        zeroPRChromosomesFlag(:, 2 .* (1:nGenomes)) = 1 - nonZeroProficientRecombinationSitesFlag;
                                        zeroPRChromosomesPositions = find(zeroPRChromosomesFlag);

                                        nonZeroPRChromosomesFlag = zeros(1, 2 * nGenomes);
                                        nonZeroPRChromosomesFlag(:, 2 .* (1:nGenomes) - 1) = nonZeroProficientRecombinationSitesFlag;
                                        nonZeroPRChromosomesFlag(:, 2 .* (1:nGenomes)) = nonZeroProficientRecombinationSitesFlag;
                                        nonZeroPRChromosomesPositions = find(nonZeroPRChromosomesFlag);

                                        if nBadGenomes > 0

                                            if silentIO ~= 1
                                            	if startWithMeiosis
                                            		dropoutLoadMeiosisProtocol(ceil(currentSimulationStep / 2), 5) = ...
	                                                    nBadGenomes;
	                                                dropoutLoadMeiosisProtocol(ceil(currentSimulationStep / 2), 6) = ...
	                                                    sum(sum(~fitness(:, zeroPRChromosomesPositions)));
                                            	else
	                                                dropoutLoadMeiosisProtocol(floor(currentSimulationStep / 2), 5) = ...
	                                                    nBadGenomes;
	                                                dropoutLoadMeiosisProtocol(floor(currentSimulationStep / 2), 6) = ...
	                                                    sum(sum(~fitness(:, zeroPRChromosomesPositions)));
	                                            end;
                                            end;

                                            genomes = genomes(:, nonZeroPRChromosomesPositions);
                                            fitness = fitness(:, nonZeroPRChromosomesPositions);
                                            if deathmatchFlag
                                                populationIdentities = populationIdentities(:, nonZeroPRChromosomesPositions);
                                            end;
                                            if matingLocusFlag
                                                matingLoci = matingLoci(:, nonZeroPRChromosomesPositions);
                                                matingTypes = matingTypes(:, nonZeroPRChromosomesPositions);
                                            end;
                                            nGenomes = nGenomes - nBadGenomes;
                                            if displayInfo
                                                disp([num2str(nBadGenomes) ' genomes died of lacking proficient recombination sites during meiosis ' num2str(meiosisCounter)]);
                                            end;
                                            simulationProtocol(currentSimulationStep, 16) = nBadGenomes;

                                            if silentIO ~= 1
                                            	if startWithMeiosis
                                            		lackingRPIEKillsProtocol(ceil(currentSimulationStep / 2), 1) = 2 * nBadGenomes;
                                            	else
                                                	lackingRPIEKillsProtocol(floor(currentSimulationStep / 2), 1) = 2 * nBadGenomes;
                                                end;	
                                            end;

                                            if silentIO == 0
                                                eval(['nonZeroRecombinationSitesProtocol' currentSimulationStepFormatted ' = nonZeroProficientRecombinationSitesFlag;']);
                                                eval(['save protocols/nonZeroRecombinationSitesProtocol' currentSimulationStepFormatted ' nonZeroRecombinationSitesProtocol' ...
                                                    currentSimulationStepFormatted ';']);
                                                eval(['clear nonZeroRecombinationSitesProtocol' currentSimulationStepFormatted ';']);
                                            end;
                                        else
                                            if displayInfo
                                                disp(['No genomes died of lacking proficient recombination sites during meiosis ' num2str(meiosisCounter)]);
                                            end;
                                            simulationProtocol(currentSimulationStep, 16) = 0;
                                        end;
                                    end;
                                end;
                                
                                if silentIO ~= 1
                                	if startWithMeiosis
                                		populationLoadMeiosisProtocol(ceil(currentSimulationStep / 2), 1) = ...
	                                        nGenomes;
	                                    populationLoadMeiosisProtocol(ceil(currentSimulationStep / 2), 2) = ...
	                                        sum(sum(~fitness));
	                                    populationSizeMeiosisProtocol(ceil(currentSimulationStep / 2), 1) = 2 * nGenomes;
                                	else
	                                    populationLoadMeiosisProtocol(floor(currentSimulationStep / 2), 1) = ...
	                                        nGenomes;
	                                    populationLoadMeiosisProtocol(floor(currentSimulationStep / 2), 2) = ...
	                                        sum(sum(~fitness));
	                                    populationSizeMeiosisProtocol(floor(currentSimulationStep / 2), 1) = 2 * nGenomes;
	                                end;    
                                end;
                                
                                if deathmatchFlag
                                    alphaSize = length(find(populationIdentities == 0));
                                    betaSize = length(find(populationIdentities == 1));
                                    if startWithMeiosis
	                                    population1SizeMeiosisProtocol(ceil(currentSimulationStep / 2), 1) = alphaSize;
	                                    population2SizeMeiosisProtocol(ceil(currentSimulationStep / 2), 1) = betaSize;
	                                else
	                                	population1SizeMeiosisProtocol(floor(currentSimulationStep / 2), 1) = alphaSize;
	                                    population2SizeMeiosisProtocol(floor(currentSimulationStep / 2), 1) = betaSize;
	                                end;
                                
                                    if size(genomes, 2) == 0
                                        defeatException = 1;
                                        defeatOutcome = 3;
                                        break;
                                    else
                                        if alphaSize == 0
                                            defeatException = 1;
                                            defeatOutcome = 0;
                                            break;
                                        elseif betaSize == 0
                                            defeatException = 1;
                                            defeatOutcome = 1;
                                            break;
                                        end;
                                    end;

                                    % optional control analysis:
                                    % measuredContent = 0;
                                    % for i = 1:(2 * nGenomes)
                                    %     measuredContent = measuredContent + ...
                                    %         min(mod(genomes(:, i), fullFitnessIdentifier) == primaryTemplate);
                                    % end;
                                    % [measuredContent sum(~populationIdentities)]

                                end;
                            end;
                    end;
                    
                    
                    
                    %%%
                    %%% END OF MEIOSIS
                    %%%
                    
                    

                    simulationProtocol(currentSimulationStep, 1) = nGenomes;
                    simulationSteps = simulationSteps - 1;
                    currentState = 1 - currentState;
                    
                    % ------------------------------
                    % | Printout of current genome |
                    % ------------------------------

                    if (currentState == 1) && (printoutMitoticGenome == 1)
                        fileExtension = 'PD';
                    elseif (currentState == 0) && (printoutMeioticGenome == 1)
                        fileExtension = 'PR';
                    end;

                    if ((currentState == 1) && (printoutMitoticGenome == 1)) || ((currentState == 0) && (printoutMeioticGenome == 1)) && ~silentIO
                        genomeDeficientIntergenicElementFingerprint     = uint8(1 .* (genomes == 0));
                        genomeProficientIntergenicElementFingerprint    = uint8(2 .* (genomes == 1));
                        genomeNonEssentialGeneFingerprint               = uint8(3 .* (genomes == 2));
                        genomeEssentialGeneFullFitnessFingerprint       = uint8(4 .* (genomes > fullFitnessIdentifier) .* (genomes < noFitnessIdentifier));
                        genomeEssentialGeneNoFitnessFingerprint         = uint8(5 .* (genomes > noFitnessIdentifier));

                        genomeSimplified = genomeDeficientIntergenicElementFingerprint + genomeProficientIntergenicElementFingerprint + genomeNonEssentialGeneFingerprint + ...
                            genomeEssentialGeneFullFitnessFingerprint + genomeEssentialGeneNoFitnessFingerprint;

                        figure;
                        imagesc(genomeSimplified);

                        h = gca;
                        colorbar;
                        set(h, 'FontSize', 14, 'LineWidth', 1.5);
                        xlabel('chromosome number');
                        ylabel('genomic site');
                        currentFile = ['images/simplifiedGenome' fileExtension currentSimulationStepFormatted '.jpg'];
                        saveas(h, currentFile);
                        currentFrame = imread(currentFile);
                        imwrite(currentFrame, currentFile, 'jpg', 'Quality', 100);
                        close;

                        figure;
                        imagesc(genomeEssentialGeneFullFitnessFingerprint + genomeEssentialGeneNoFitnessFingerprint);

                        h = gca;
                        colorbar;
                        set(h, 'FontSize', 14, 'LineWidth', 1.5);
                        xlabel('chromosome number');
                        ylabel('genomic site');
                        currentFile = ['images/essentialGenes' fileExtension currentSimulationStepFormatted '.jpg'];
                        saveas(h, currentFile);
                        currentFrame = imread(currentFile);
                        imwrite(currentFrame, currentFile, 'jpg', 'Quality', 100);
                        close;

                        figure;
                        imagesc(genomeProficientIntergenicElementFingerprint + genomeEssentialGeneFullFitnessFingerprint);

                        h = gca;
                        colorbar;
                        set(h, 'FontSize', 14, 'LineWidth', 1.5);
                        xlabel('chromosome number');
                        ylabel('genomic site');
                        currentFile = ['images/proficientIEs_functionalEGs' fileExtension currentSimulationStepFormatted '.jpg'];
                        saveas(h, currentFile);
                        currentFrame = imread(currentFile);
                        imwrite(currentFrame, currentFile, 'jpg', 'Quality', 100);
                        close;
                    end;

                    % -------------------------------
                    % | Writing of genomic database |
                    % -------------------------------

                    if createGenomicDatabase == 1
                        if ((minimalDiskSpaceMode == 0) || ((minimalDiskSpaceMode == 1) && (mod(currentSimulationStep, databaseResolution) == 0)) || ...
                                (currentSimulationStep == totalSimulationSteps)) && silentIO ~= 1
                            eval(['genomes' currentSimulationStepFormatted ' = genomes;']);
                            eval(['save genomes/genomes' currentSimulationStepFormatted ' genomes' currentSimulationStepFormatted ';']);
                            eval(['clear genomes' currentSimulationStepFormatted ';']);
                            if saveASCIIFiles == 1
                                genomesDouble = double(genomes);
                                eval(['save genomes/genomes' currentSimulationStepFormatted '.txt genomesDouble -ascii;']);
                                eval(['clear genomesDouble genomes' currentSimulationStepFormatted ';']);
                            end;
                            eval(['fitness' currentSimulationStepFormatted ' = fitness;']);
                            eval(['save genomes/fitness' currentSimulationStepFormatted ' fitness' currentSimulationStepFormatted ';']);
                            eval(['clear fitness' currentSimulationStepFormatted ';']);
                            if matingLocusFlag
                                eval(['matingLoci' currentSimulationStepFormatted ' = matingLoci;']);
                                eval(['save genomes/matingLoci' currentSimulationStepFormatted ' matingLoci' currentSimulationStepFormatted ';']);
                                eval(['clear matingLoci' currentSimulationStepFormatted ';']);
                                
                                eval(['matingTypes' currentSimulationStepFormatted ' = matingTypes;']);
                                eval(['save genomes/matingTypes' currentSimulationStepFormatted ' matingTypes' currentSimulationStepFormatted ';']);
                                eval(['clear matingTypes' currentSimulationStepFormatted ';']);
                            end;
                            if deathmatchFlag
                                eval(['populationIdentities' currentSimulationStepFormatted ' = populationIdentities;']);
                                eval(['save genomes/populationIdentities' currentSimulationStepFormatted ' populationIdentities' currentSimulationStepFormatted ';']);
                                eval(['clear populationIdentities' currentSimulationStepFormatted ';']);
                            end;
                            if subPopulationFlag
                                eval(['subPopulations' currentSimulationStepFormatted ' = subPopulations;']);
                                eval(['save genomes/subPopulations' currentSimulationStepFormatted ' subPopulations' currentSimulationStepFormatted ';']);
                                eval(['clear subPopulations' currentSimulationStepFormatted ';']);
                            end;
                        end;
                    end;

                    % ------------------------------------------------------
                    % | Cluster and genome analysis for current generation |
                    % ------------------------------------------------------

                    if (analyzeClustering == 1) && (nGenomes > 0)
                        %proficientRecombinationSites = genomes == 1;
                        %essentialGenesSites = genomes > fullFitnessIdentifier;                    
                        %[sitePositions genomePositions] = find(proficientRecombinationSites);
                        %nProficientRecombinationSites = sum(proficientRecombinationSites, 1);

                        %averageRecombinationSitesArray = cat(1, averageRecombinationSitesArray, [mean(nProficientRecombinationSites) std(nProficientRecombinationSites)]);
                        %averageFunctionalGenesArray = cat(1, averageFunctionalGenesArray, [mean(sum(fitness, 1)) std(sum(fitness, 1))]);

                        %nMaximalRecombinationSites = max(nProficientRecombinationSites);
                        %proficientRecombinationSitesPositions = zeros(nMaximalRecombinationSites, 2 * nGenomes);
                        %geneClusterSizes = zeros(nMaximalRecombinationSites + 1, 2 * nGenomes);
                        %currentPosition = 1;

                        %for n = 1:(2 * nGenomes)
                        %    if nProficientRecombinationSites(1, n) > 0
                        %        proficientRecombinationSitesPositions(1:nProficientRecombinationSites(1, n), n) = ...
                        %            sitePositions(currentPosition:(currentPosition + nProficientRecombinationSites(1, n) - 1), 1);
                        %        currentPosition = currentPosition + nProficientRecombinationSites(1, n);
                        %    end;
                        %    for k = 1:(nProficientRecombinationSites(1, n) + 1)
                        %        if (k == 1) & (k < (nProficientRecombinationSites(1, n) + 1))
                        %            geneClusterSizes(k, n) = sum(essentialGenesSites(1:(proficientRecombinationSitesPositions(1, n) - 1), n));
                        %        elseif (k > 1) & (k < (nProficientRecombinationSites(1, n) + 1))
                        %            geneClusterSizes(k, n) = sum(essentialGenesSites((proficientRecombinationSitesPositions(k - 1, n) + 1):(proficientRecombinationSitesPositions(k, n) - 1), n));
                        %        elseif (k > 1) & (k == (nProficientRecombinationSites(1, n) + 1))
                        %            geneClusterSizes(k, n) = sum(essentialGenesSites((proficientRecombinationSitesPositions(k - 1, n) + 1):nGenomicElements, n));
                        %        else % (k == 1) & (k == (nProficientRecombinationSites(1, n) + 1))
                        %            geneClusterSizes(k, n) = sum(essentialGenesSites(1:nGenomicElements, n));
                        %        end;
                        %    end;
                        %end;

                        oddNumbers = 2 .* (1:nGenomes) - 1;
                        evenNumbers = 2 .* (1:nGenomes);
                        proficientRecombinationSites = (genomes(:, oddNumbers) == 1) | (genomes(:, evenNumbers) == 1);
                        essentialGenesSites = (genomes(:, oddNumbers) > fullFitnessIdentifier) | (genomes(:, evenNumbers) > fullFitnessIdentifier);
                        [sitePositions genomePositions] = find(proficientRecombinationSites);
                        nProficientRecombinationSites = sum(proficientRecombinationSites, 1);

                        averageRecombinationSitesArray = cat(1, averageRecombinationSitesArray, [mean(nProficientRecombinationSites) std(nProficientRecombinationSites)]);
                        averageFunctionalGenesArray = cat(1, averageFunctionalGenesArray, [mean(sum(fitness, 1)) std(sum(fitness, 1))]);

                        nMaximalRecombinationSites = max(nProficientRecombinationSites);
                        proficientRecombinationSitesPositions = zeros(nMaximalRecombinationSites, nGenomes);
                        geneClusterSizes = zeros(nMaximalRecombinationSites + 1, nGenomes);
                        currentPosition = 1;

                        for n = 1:nGenomes
                            if nProficientRecombinationSites(1, n) > 0
                                proficientRecombinationSitesPositions(1:nProficientRecombinationSites(1, n), n) = ...
                                    sitePositions(currentPosition:(currentPosition + nProficientRecombinationSites(1, n) - 1), 1);
                                currentPosition = currentPosition + nProficientRecombinationSites(1, n);
                            end;
                            for k = 1:(nProficientRecombinationSites(1, n) + 1)
                                if (k == 1) && (k < (nProficientRecombinationSites(1, n) + 1))
                                    geneClusterSizes(k, n) = sum(essentialGenesSites(1:(proficientRecombinationSitesPositions(1, n) - 1), n));
                                elseif (k > 1) && (k < (nProficientRecombinationSites(1, n) + 1))
                                    geneClusterSizes(k, n) = sum(essentialGenesSites((proficientRecombinationSitesPositions(k - 1, n) + 1):(proficientRecombinationSitesPositions(k, n) - 1), n));
                                elseif (k > 1) && (k == (nProficientRecombinationSites(1, n) + 1))
                                    geneClusterSizes(k, n) = sum(essentialGenesSites((proficientRecombinationSitesPositions(k - 1, n) + 1):nGenomicElements, n));
                                else % (k == 1) & (k == (nProficientRecombinationSites(1, n) + 1))
                                    geneClusterSizes(k, n) = sum(essentialGenesSites(1:nGenomicElements, n));
                                end;
                            end;
                        end;
                        
                        if silentIO == 0
                            if totalSimulationSteps > protocolSplittingSize
                                if mod(currentSimulationStep, protocolSplittingSize) == 0
                                    proficientRecombinationSitesProtocol(protocolSplittingSize, 1:length(nProficientRecombinationSites)) = nProficientRecombinationSites(:, :);
                                    eval(['proficientRecombinationSitesProtocol' currentProtocolSectionFormatted ' = proficientRecombinationSitesProtocol;']);
                                    eval(['save clustering/proficientRecombinationSitesProtocol' currentProtocolSectionFormatted ' proficientRecombinationSitesProtocol' currentProtocolSectionFormatted ';']);
                                    eval(['clear proficientRecombinationSitesProtocol' currentProtocolSectionFormatted ';']);
                                    proficientRecombinationSitesProtocol = zeros(protocolSplittingSize, 2 * maxGenomes, 'uint16');

                                else
                                    proficientRecombinationSitesProtocol(mod(currentSimulationStep, protocolSplittingSize), 1:length(nProficientRecombinationSites)) = nProficientRecombinationSites(:, :);
                                end;
                            else
                                proficientRecombinationSitesProtocol(currentSimulationStep, 1:length(nProficientRecombinationSites)) = nProficientRecombinationSites(:, :);
                            end;
                            
                            if totalSimulationSteps > protocolSplittingSize
                                [currentXSize currentYSize currentZSize] = size(geneClusterSizesProtocol);
                                if ((nMaximalRecombinationSites + 1) > currentXSize) || (nGenomes > currentYSize)
                                    geneClusterSizesProtocolBackup = geneClusterSizesProtocol;
                                    geneClusterSizesProtocol = zeros(max(nMaximalRecombinationSites + 1, currentXSize), ...
                                        max(nGenomes, currentYSize), protocolSplittingSize, 'uint16');
                                    geneClusterSizesProtocol(1:currentXSize, 1:currentYSize, :) = geneClusterSizesProtocolBackup(:, :, :);
                                    clear geneClusterSizesProtocolBackup;

                                end;
                                [currentXSize currentYSize] = size(geneClusterSizes);
                                if mod(currentSimulationStep, protocolSplittingSize) == 0
                                    geneClusterSizesProtocol(1:currentXSize, 1:currentYSize, protocolSplittingSize) = geneClusterSizes(:, :);
                                    eval(['geneClusterSizesProtocol' currentProtocolSectionFormatted ' = geneClusterSizesProtocol;']);
                                    eval(['save clustering/geneClusterSizesProtocol' currentProtocolSectionFormatted ' geneClusterSizesProtocol' currentProtocolSectionFormatted ';']);
                                    eval(['clear geneClusterSizesProtocol' currentProtocolSectionFormatted ';']);
                                    geneClusterSizesProtocol = zeros(1, maxGenomes, protocolSplittingSize, 'uint16');

                                else
                                    geneClusterSizesProtocol(1:currentXSize, 1:currentYSize, mod(currentSimulationStep, protocolSplittingSize)) = geneClusterSizes(:, :);
                                end;
                            else
                                [currentXSize currentYSize currentZSize] = size(geneClusterSizesProtocol);
                                if ((nMaximalRecombinationSites + 1) > currentXSize) || (nGenomes > currentYSize)
                                    geneClusterSizesProtocolBackup = geneClusterSizesProtocol;
                                    geneClusterSizesProtocol = zeros(max(nMaximalRecombinationSites + 1, currentXSize), ...
                                        max(nGenomes, currentYSize), totalSimulationSteps, 'uint16');
                                    geneClusterSizesProtocol(1:currentXSize, 1:currentYSize, :) = geneClusterSizesProtocolBackup(:, :, :);
                                    clear geneClusterSizesProtocolBackup;

                                end;
                                [currentXSize currentYSize] = size(geneClusterSizes);
                                geneClusterSizesProtocol(1:currentXSize, 1:currentYSize, currentSimulationStep) = geneClusterSizes(:, :);
                            end;
                        end;

                        %nEssentialGenesPerChromosome = sum(geneClusterSizes, 1);
                        %geneClusterSizes = geneClusterSizes .^ clusterAnalysisPower;
                        %currentClusteringValues = sum(geneClusterSizes, 1) ./ nEssentialGenesPerChromosome .* (nProficientRecombinationSites + 1);
                        %clusterAnalysis = cat(1, clusterAnalysis, [mean(currentClusteringValues) std(currentClusteringValues)]);

                        nEssentialGenesPerGenome = sum(geneClusterSizes, 1);
                        geneClusterSizes = geneClusterSizes .^ clusterAnalysisPower;
                        currentClusteringValues = sum(geneClusterSizes, 1) ./ nEssentialGenesPerGenome .* (nProficientRecombinationSites + 1);
                        clusterAnalysis = cat(1, clusterAnalysis, [mean(currentClusteringValues) std(currentClusteringValues)]);

                        if displayInfo
                            if (currentState == 1) && ~skipMeiosis
                                disp(['A clustering value of ' num2str(clusterAnalysis(currentSimulationStep, 1)) ' +- ' num2str(clusterAnalysis(currentSimulationStep, 2)) ...
                                    ' results for mitosis ' num2str(mitosisCounter)]);
                            elseif (currentState == 0)
                                disp(['A clustering value of ' num2str(clusterAnalysis(currentSimulationStep, 1)) ' +- ' num2str(clusterAnalysis(currentSimulationStep, 2)) ...
                                    ' results for meiosis ' num2str(meiosisCounter)]);
                            end;

                            if (currentState == 1) && ~skipMeiosis
                                disp(['The mean number of proficient recombination sites is ' num2str(averageRecombinationSitesArray(currentSimulationStep, 1)) ' +- ' ...
                                    num2str(averageRecombinationSitesArray(currentSimulationStep, 2)) ...
                                    ' for mitosis ' num2str(mitosisCounter)]);
                            elseif (currentState == 0)
                                disp(['The mean number of proficient recombination sites is ' num2str(averageRecombinationSitesArray(currentSimulationStep, 1)) ' +- ' ...
                                    num2str(averageRecombinationSitesArray(currentSimulationStep, 2)) ...
                                    ' for meiosis ' num2str(meiosisCounter)]);
                            end;

                            if (currentState == 1) && ~skipMeiosis
                                disp(['The mean number of functional essential genes is ' num2str(averageFunctionalGenesArray(currentSimulationStep, 1)) ' +- ' ...
                                    num2str(averageFunctionalGenesArray(currentSimulationStep, 2)) ...
                                    ' for mitosis ' num2str(mitosisCounter)]);
                            elseif (currentState == 0)
                                disp(['The mean number of functional essential genes is ' num2str(averageFunctionalGenesArray(currentSimulationStep, 1)) ' +- ' ...
                                    num2str(averageFunctionalGenesArray(currentSimulationStep, 2)) ...
                                    ' for meiosis ' num2str(meiosisCounter)]);
                            end;
                        end;

                        simulationProtocol(currentSimulationStep, 17) = clusterAnalysis(currentSimulationStep, 1);
                        simulationProtocol(currentSimulationStep, 18) = clusterAnalysis(currentSimulationStep, 2);
                        simulationProtocol(currentSimulationStep, 19) = averageRecombinationSitesArray(currentSimulationStep, 1);
                        simulationProtocol(currentSimulationStep, 20) = averageRecombinationSitesArray(currentSimulationStep, 2);
                        simulationProtocol(currentSimulationStep, 21) = averageFunctionalGenesArray(currentSimulationStep, 1);
                        simulationProtocol(currentSimulationStep, 22) = averageFunctionalGenesArray(currentSimulationStep, 2);

                    end;

                end;

                if simulationSteps > 0
                    simulationProtocol((currentSimulationStep + 1):totalSimulationSteps, :) = [];
                elseif performSurvivalScreen ~= 0 % Simulation was successful with respect to the survival of the population
                    if (screeningArray == 1) && (performSurvivalScreen ~= 2)
                        survivalArray(a, 1) = survivalArray(a, 1) + 1;
                    else
                        survivalArray(b, 1) = survivalArray(b, 1) + 1;
                        if ~clusterMode
                            currentDirectory = 'protocols';
                        else
                            currentDirectory = masterDirectory;
                        end;
                        robustness = verticalParametersList(1, a);
                        eval(['save ' currentDirectory '/robustness robustness;']);
                    end;
                end;
                
                if deathmatchFlag
                    if defeatException == 1
                        if defeatOutcome == 3
                            deathmatchWinner = 3;
                        else
                            deathmatchWinner = 1 - defeatOutcome;
                        end;
                    else % simulation was not interrupted by deathmatch criteria
                        if size(genomes, 2) == 0
                            deathmatchWinner = 3;
                        elseif isempty(find(populationIdentities == 0, 1))
                            deathmatchWinner = 1;
                        elseif isempty(find(populationIdentities == 1, 1))
                            deathmatchWinner = 0;
                        else
                            deathmatchWinner = 2;
                        end;
                    end;
                    deathmatchWinner = [deathmatchWinner currentSimulationStep size(genomes, 2) ...
                        length(find(populationIdentities == 0)) length(find(populationIdentities == 1))];
                    if frontendFlag == 0
                        if ~clusterMode
                            currentDirectory = 'protocols';
                        else
                            currentDirectory = masterDirectory;
                        end;
                        eval(['save ' currentDirectory '/deathmatchWinner.txt deathmatchWinner -ascii;']);
                    end;
                    results = deathmatchWinner;
                    
                    defeatException = 0;
                    defeatOutcome = NaN;
                    
                    if frontendFlag == 0
                        if ~clusterMode
                            currentDirectory = '../../protocol_plots';
                        else
                            currentDirectory = masterDirectory;
                        end;
                        eval(['save ' currentDirectory '/population1SizeMitosisProtocol' currentExperimentString '.mat population1SizeMitosisProtocol;']);
                        eval(['save ' currentDirectory '/population1SizeMeiosisProtocol' currentExperimentString '.mat population1SizeMeiosisProtocol;']);
                        eval(['save ' currentDirectory '/population2SizeMitosisProtocol' currentExperimentString '.mat population2SizeMitosisProtocol;']);
                        eval(['save ' currentDirectory '/population2SizeMeiosisProtocol' currentExperimentString '.mat population2SizeMeiosisProtocol;']);
                    end;
                else
                    results = 0;
                end;
                
                
                
                % --------------------------
                % | Final cluster analysis |
                % --------------------------

                if analyzeClustering == 1 && silentIO ~= 1
                    
                    if simulationSteps > 0
                        
                        if fittingScheme == 0
                            clusterAnalysisResults = zeros(1, 7);
                        elseif fittingScheme == 1
                            clusterAnalysisResults = zeros(1, 10);
                        end;
                        clusterAnalysisResults(1, :) = 0;
                        slidingWindowEssentialGenesHistogram = zeros(1, slidingWindowSize + 1);
                        slidingWindowRecombinationSitesHistogram = zeros(1, slidingWindowSize + 1);
                        slidingWindowEssentialGenesHistogramDeviations = zeros(1, slidingWindowSize + 1);
                        slidingWindowRecombinationSitesHistogramDeviations = zeros(1, slidingWindowSize + 1);
                        
                    else

                        clusterAnalysisValues = clusterAnalysis(:, 1);
                        regressors = ones(length(clusterAnalysisValues), 2);
                        regressors(:, 1) = (1:length(clusterAnalysisValues))';

                        if fittingScheme == 0

                            [regressionParameters, regressionParametersBoundaries] = regress(clusterAnalysisValues, regressors);
                            relativeClustering = (length(clusterAnalysisValues) * regressionParameters(1, 1) + regressionParameters(2, 1)) / ...
                                (regressionParameters(1, 1) + regressionParameters(2, 1));
                            clusterAnalysisResults = [1 length(clusterAnalysisValues) regressionParameters(1, 1) regressionParameters(2, 1) ...
                                regressionParametersBoundaries(1, 2) - regressionParameters(1, 1) regressionParametersBoundaries(2, 2) - regressionParameters(2, 1) relativeClustering];

                            figure;
                            plot(1:length(clusterAnalysisValues), clusterAnalysisValues', ...
                                1:length(clusterAnalysisValues), (1:length(clusterAnalysisValues)) .* regressionParameters(1, 1) + regressionParameters(2, 1), 'LineWidth', 2);

                        elseif fittingScheme == 1

                            if frontendFlag == 0
                                %unix('cd ../..');
                                cd('../..');
                            end;
                            
                            currentSteepnessEstimate = 10 / length(clusterAnalysisValues);
                            currentEquilibriumEstimate = clusterAnalysisValues(length(clusterAnalysisValues), 1);
                            currentAmplitudeEstimate = (clusterAnalysisValues(1, 1) - currentEquilibriumEstimate) / exp(- currentSteepnessEstimate);
                            if currentAmplitudeEstimate == 0
                                currentAmplitudeEstimate = 0.1;
                            end;
                            [currentFittingParameters, currentFittingParametersErrors, currentFittingChiSquared] = ...
                                fitting('exponential', regressors(:, 1), clusterAnalysisValues, regressors(:, 2), ...
                                [currentAmplitudeEstimate currentSteepnessEstimate currentEquilibriumEstimate], [1 1 1]);
                            fittedExponential = exponential(regressors(:, 1), currentFittingParameters);
                            relativeClustering = exponential(length(clusterAnalysisValues), currentFittingParameters) / exponential(1, currentFittingParameters);
                            clusterAnalysisResults = [1 length(clusterAnalysisValues) currentFittingParameters currentFittingParametersErrors currentFittingChiSquared relativeClustering];

                            if frontendFlag == 0
                                %order=['cd ' currentSessionString];
                                %unix(order);
                                cd(currentSessionString);

                                %order=['cd ' currentDirectoryString];
                                %unix(order);
                                cd(currentDirectoryString);
                            end;

                            figure;
                            plot(1:length(clusterAnalysisValues), clusterAnalysisValues', 1:length(clusterAnalysisValues), fittedExponential, 'LineWidth', 2);

                        end;

                        h = gca;
                        set(h, 'FontSize', 14, 'LineWidth', 1.5);
                        xlabel('generation');
                        ylabel('clustering value');
                        axis([1 length(clusterAnalysisValues) minClusteringValue maxClusteringValue]);
                        %currentFile = 'clustering/clusterAnalysisFit.jpg';
                        %saveas(h, currentFile);
                        %currentFrame = imread(currentFile);
                        %imwrite(currentFrame, currentFile, 'jpg', 'Quality', 100);
                        currentFile = ['../../clustering_plots/clusterAnalysisFit' currentExperimentString '.jpg'];
                        saveas(h, currentFile);
                        currentFrame = imread(currentFile);
                        imwrite(currentFrame, currentFile, 'jpg', 'Quality', 100);
                        if (max(clusterAnalysisValues) - min(clusterAnalysisValues)) ~= 0
                            axis([1 length(clusterAnalysisValues) min(clusterAnalysisValues) - (max(clusterAnalysisValues) - min(clusterAnalysisValues)) / 10 ...
                                max(clusterAnalysisValues) + (max(clusterAnalysisValues) - min(clusterAnalysisValues)) / 10]);
                        else
                            axis([1 length(clusterAnalysisValues) min(clusterAnalysisValues) - 100 max(clusterAnalysisValues) + 100]);
                        end;
                        %currentFile = 'clustering/clusterAnalysisFitIndividualScale.jpg';
                        %saveas(h, currentFile);
                        %currentFrame = imread(currentFile);
                        %imwrite(currentFrame, currentFile, 'jpg', 'Quality', 100);
                        currentFile = ['../../clustering_plots/clusterAnalysisFitIndividualScale' currentExperimentString '.jpg'];
                        saveas(h, currentFile);
                        currentFrame = imread(currentFile);
                        imwrite(currentFrame, currentFile, 'jpg', 'Quality', 100);
                        close;

                        save clustering/clusterAnalysisResults clusterAnalysisResults;
                        % save clustering/clusterAnalysisResults.txt clusterAnalysisResults -ascii;

                        finalRecombinationSitesMatrix = (genomes == 1);
                        finalEssentialGenesMatrix = (genomes > fullFitnessIdentifier);
                        evenPositions = 2 .* (1:(nGenes - 1));
                        oddPositions = 2 .* (1:nGenes) - 1;
                        finalRecombinationSitesMatrix = finalRecombinationSitesMatrix(evenPositions, :);
                        finalEssentialGenesMatrix = finalEssentialGenesMatrix(oddPositions, :);

                        slidingWindowEssentialGenesHistogram = zeros(1, slidingWindowSize + 1);
                        slidingWindowRecombinationSitesHistogram = zeros(1, slidingWindowSize + 1);
                        slidingWindowPlotEssentialGenes = zeros(nGenes - slidingWindowPlotSize + 1, 2 * nGenomes);
                        slidingWindowPlotRecombinationSites = zeros(nGenes - slidingWindowPlotSize, 2 * nGenomes);
                        slidingWindowPlotDifference = zeros(nGenes - slidingWindowPlotSize, 2 * nGenomes);

                        for m = 1:(2 * nGenomes)
                            for n = 1:(nGenes - slidingWindowSize)
                                currentEssentialSum = sum(finalEssentialGenesMatrix(n:(n + slidingWindowSize - 1), m));
                                slidingWindowEssentialGenesHistogram(1, currentEssentialSum + 1) = slidingWindowEssentialGenesHistogram(1, currentEssentialSum + 1) + 1;
                                currentRecombinationSum = sum(finalRecombinationSitesMatrix(n:(n + slidingWindowSize - 1), m));
                                slidingWindowRecombinationSitesHistogram(1, currentRecombinationSum + 1) = slidingWindowRecombinationSitesHistogram(1, currentRecombinationSum + 1) + 1;
                            end;
                        end;
                        for m = 1:(2 * nGenomes)
                            currentEssentialSum = sum(finalEssentialGenesMatrix((nGenes - slidingWindowSize + 1):nGenes, m));
                            slidingWindowEssentialGenesHistogram(1, currentEssentialSum + 1) = slidingWindowEssentialGenesHistogram(1, currentEssentialSum + 1) + 1;
                        end;

                        for m = 1:(nGenes - slidingWindowPlotSize + 1)
                            slidingWindowPlotEssentialGenes(m, :) = sum(finalEssentialGenesMatrix(m:(m + slidingWindowPlotSize - 1), :), 1);
                        end;
                        for m = 1:(nGenes - slidingWindowPlotSize)
                            slidingWindowPlotRecombinationSites(m, :) = sum(finalRecombinationSitesMatrix(m:(m + slidingWindowPlotSize - 1), :), 1);
                        end;

                        slidingWindowPlotDifference(:, :) = slidingWindowPlotEssentialGenes(1:(nGenes - slidingWindowPlotSize), :) - ...
                            slidingWindowPlotRecombinationSites(1:(nGenes - slidingWindowPlotSize), :);
                        groupingMatrix(a + 1, b + 1) = mean(std(slidingWindowPlotDifference, 0, 1));

                        slidingWindowEssentialGenesHistogram = slidingWindowEssentialGenesHistogram ./ ((2 .* nGenomes) .* (nGenes - slidingWindowSize + 1));
                        slidingWindowRecombinationSitesHistogram = slidingWindowRecombinationSitesHistogram ./ ((2 .* nGenomes) .* (nGenes - slidingWindowSize));

                        slidingWindowEssentialGenesHistogramDeviations = slidingWindowEssentialGenesHistogram - slidingWindowEssentialGenesHistogramRandomCase;
                        slidingWindowRecombinationSitesHistogramDeviations = slidingWindowRecombinationSitesHistogram - slidingWindowRecombinationSitesHistogramRandomCase;

                        save clustering/slidingWindowEssentialGenesHistogram slidingWindowEssentialGenesHistogram;
                        % save clustering/slidingWindowEssentialGenesHistogram.txt slidingWindowEssentialGenesHistogram -ascii;
                        save clustering/slidingWindowRecombinationSitesHistogram slidingWindowRecombinationSitesHistogram;
                        % save clustering/slidingWindowRecombinationSitesHistogram.txt slidingWindowRecombinationSitesHistogram -ascii;
                        save clustering/slidingWindowEssentialGenesHistogramDeviations slidingWindowEssentialGenesHistogramDeviations;
                        save clustering/slidingWindowRecombinationSitesHistogramDeviations slidingWindowRecombinationSitesHistogramDeviations;

                        save clustering/slidingWindowPlotEssentialGenes slidingWindowPlotEssentialGenes;
                        save clustering/slidingWindowPlotRecombinationSites slidingWindowPlotRecombinationSites;
                        save clustering/slidingWindowPlotDifference slidingWindowPlotDifference;

                        figure;
                        imagesc(slidingWindowPlotEssentialGenes);
                        h = gca;
                        colorbar;
                        set(h, 'FontSize', 14, 'LineWidth', 1.5);
                        xlabel('chromosome number');
                        ylabel('genomic site pair');
                        axis off;
                        axis equal;
                        %currentFile = ['clustering/slidingWindowPlotEssentialGenes.jpg'];
                        %saveas(h, currentFile);
                        %currentFrame = imread(currentFile);
                        %imwrite(currentFrame, currentFile, 'jpg', 'Quality', 100);
                        currentFile = ['../../clustering_plots/slidingWindowPlotEssentialGenes' currentExperimentString '.jpg'];
                        saveas(h, currentFile);
                        currentFrame = imread(currentFile);
                        imwrite(currentFrame, currentFile, 'jpg', 'Quality', 100);
                        close;

                        figure;
                        imagesc(slidingWindowPlotRecombinationSites);
                        h = gca;
                        colorbar;
                        set(h, 'FontSize', 14, 'LineWidth', 1.5);
                        xlabel('chromosome number');
                        ylabel('genomic site pair');
                        axis off;
                        axis equal;
                        %currentFile = ['clustering/slidingWindowPlotRecombinationSites.jpg'];
                        %saveas(h, currentFile);
                        %currentFrame = imread(currentFile);
                        %imwrite(currentFrame, currentFile, 'jpg', 'Quality', 100);
                        currentFile = ['../../clustering_plots/slidingWindowPlotRecombinationSites' currentExperimentString '.jpg'];
                        saveas(h, currentFile);
                        currentFrame = imread(currentFile);
                        imwrite(currentFrame, currentFile, 'jpg', 'Quality', 100);
                        close;

                        figure;
                        imagesc(slidingWindowPlotDifference);
                        h = gca;
                        colorbar;
                        set(h, 'FontSize', 14, 'LineWidth', 1.5);
                        xlabel('chromosome number');
                        ylabel('genomic site pair');
                        axis off;
                        axis equal;
                        %currentFile = ['clustering/slidingWindowPlotDifference.jpg'];
                        %saveas(h, currentFile);
                        %currentFrame = imread(currentFile);
                        %imwrite(currentFrame, currentFile, 'jpg', 'Quality', 100);
                        currentFile = ['../../clustering_plots/slidingWindowPlotDifference' currentExperimentString '.jpg'];
                        saveas(h, currentFile);
                        currentFrame = imread(currentFile);
                        imwrite(currentFrame, currentFile, 'jpg', 'Quality', 100);
                        close;

                        figure;
                        bar(0:slidingWindowSize, slidingWindowEssentialGenesHistogram, 'grouped');
                        h = gca;
                        set(h, 'FontSize', 14, 'LineWidth', 1.5);
                        xlabel('clustered essential genes');
                        ylabel('frequency');
                        axis([-1 slidingWindowSize + 1 0 1]);
                        %currentFile = 'clustering/slidingWindowEssentialGenes.jpg';
                        %saveas(h, currentFile);
                        %currentFrame = imread(currentFile);
                        %imwrite(currentFrame, currentFile, 'jpg', 'Quality', 100);
                        currentFile = ['../../clustering_plots/slidingWindowEssentialGenes' currentExperimentString '.jpg'];
                        saveas(h, currentFile);
                        currentFrame = imread(currentFile);
                        imwrite(currentFrame, currentFile, 'jpg', 'Quality', 100);
                        close;

                        figure;
                        bar(0:slidingWindowSize, slidingWindowRecombinationSitesHistogram, 'grouped');
                        h = gca;
                        set(h, 'FontSize', 14, 'LineWidth', 1.5);
                        xlabel('clustered recombination sites');
                        ylabel('frequency');
                        axis([-1 slidingWindowSize + 1 0 1]);
                        %currentFile = 'clustering/slidingWindowRecombinationSites.jpg';
                        %saveas(h, currentFile);
                        %currentFrame = imread(currentFile);
                        %imwrite(currentFrame, currentFile, 'jpg', 'Quality', 100);
                        currentFile = ['../../clustering_plots/slidingWindowRecombinationSites' currentExperimentString '.jpg'];
                        saveas(h, currentFile);
                        currentFrame = imread(currentFile);
                        imwrite(currentFrame, currentFile, 'jpg', 'Quality', 100);
                        close;

                        figure;
                        bar(0:slidingWindowSize, [slidingWindowEssentialGenesHistogram' slidingWindowRecombinationSitesHistogram'], 'grouped');
                        h = gca;
                        set(h, 'FontSize', 14, 'LineWidth', 1.5);
                        xlabel('clustered units');
                        ylabel('frequency');
                        axis([-1 slidingWindowSize + 1 0 1]);
                        %currentFile = 'clustering/slidingWindowGrouped.jpg';
                        %saveas(h, currentFile);
                        %currentFrame = imread(currentFile);
                        %imwrite(currentFrame, currentFile, 'jpg', 'Quality', 100);
                        currentFile = ['../../clustering_plots/slidingWindowGrouped' currentExperimentString '.jpg'];
                        saveas(h, currentFile);
                        currentFrame = imread(currentFile);
                        imwrite(currentFrame, currentFile, 'jpg', 'Quality', 100);
                        close;

                        figure;
                        bar(0:slidingWindowSize, slidingWindowEssentialGenesHistogramDeviations, 'grouped');
                        h = gca;
                        set(h, 'FontSize', 14, 'LineWidth', 1.5);
                        xlabel('clustered essential genes');
                        ylabel('frequency');
                        axis([-1 slidingWindowSize + 1 -1 1]);
                        %currentFile = 'clustering/slidingWindowDeviationsEssentialGenes.jpg';
                        %saveas(h, currentFile);
                        %currentFrame = imread(currentFile);
                        %imwrite(currentFrame, currentFile, 'jpg', 'Quality', 100);
                        currentFile = ['../../clustering_plots/slidingWindowDeviationsEssentialGenes' currentExperimentString '.jpg'];
                        saveas(h, currentFile);
                        currentFrame = imread(currentFile);
                        imwrite(currentFrame, currentFile, 'jpg', 'Quality', 100);
                        close;

                        figure;
                        bar(0:slidingWindowSize, slidingWindowRecombinationSitesHistogramDeviations, 'grouped');
                        h = gca;
                        set(h, 'FontSize', 14, 'LineWidth', 1.5);
                        xlabel('clustered recombination sites');
                        ylabel('frequency');
                        axis([-1 slidingWindowSize + 1 -1 1]);
                        %currentFile = 'clustering/slidingWindowDeviationsRecombinationSites.jpg';
                        %saveas(h, currentFile);
                        %currentFrame = imread(currentFile);
                        %imwrite(currentFrame, currentFile, 'jpg', 'Quality', 100);
                        currentFile = ['../../clustering_plots/slidingWindowDeviationsRecombinationSites' currentExperimentString '.jpg'];
                        saveas(h, currentFile);
                        currentFrame = imread(currentFile);
                        imwrite(currentFrame, currentFile, 'jpg', 'Quality', 100);
                        close;

                        figure;
                        bar(0:slidingWindowSize, [slidingWindowEssentialGenesHistogramDeviations' slidingWindowRecombinationSitesHistogramDeviations'], 'grouped');
                        h = gca;
                        set(h, 'FontSize', 14, 'LineWidth', 1.5);
                        xlabel('clustered units');
                        ylabel('frequency');
                        axis([-1 slidingWindowSize + 1 -1 1]);
                        %currentFile = 'clustering/slidingWindowDeviationsGrouped.jpg';
                        %saveas(h, currentFile);
                        %currentFrame = imread(currentFile);
                        %imwrite(currentFrame, currentFile, 'jpg', 'Quality', 100);
                        currentFile = ['../../clustering_plots/slidingWindowDeviationsGrouped' currentExperimentString '.jpg'];
                        saveas(h, currentFile);
                        currentFrame = imread(currentFile);
                        imwrite(currentFrame, currentFile, 'jpg', 'Quality', 100);
                        close;
                    end;
                end;
                
                if frontendFlag == 0
                    if ~clusterMode
                        currentDirectory = 'clustering';
                    else
                        currentDirectory = masterDirectory;
                    end;
                    eval(['save ' currentDirectory '/clusterAnalysis clusterAnalysis;']);
                    % save clustering/clusterAnalysis.txt clusterAnalysis -ascii;
                    % save clustering/averageRecombinationSitesArray averageRecombinationSitesArray;
                    % save clustering/averageRecombinationSitesArray.txt averageRecombinationSitesArray -ascii;
                    % save clustering/averageFunctionalGenesArray averageFunctionalGenesArray;
                    % save clustering/averageFunctionalGenesArray.txt averageFunctionalGenesArray -ascii;
                end;
                
                if (visualizeProtocols == 1) && (simulationSteps == 0) && silentIO ~= 1
                    
                    averageRecombinationSitesProtocol = averageRecombinationSitesArray(:, 1);
                    averageFunctionalGenesProtocol = averageFunctionalGenesArray(:, 1);
                    
                    eval(['save ../../protocol_plots/averageRecombinationSitesProtocol' currentExperimentString '.mat averageRecombinationSitesProtocol;']);
                    eval(['save ../../protocol_plots/averageFunctionalGenesProtocol' currentExperimentString '.mat averageFunctionalGenesProtocol;']);
                    eval(['save ../../protocol_plots/populationSizeMitosisProtocol' currentExperimentString '.mat populationSizeMitosisProtocol;']);
                    eval(['save ../../protocol_plots/populationSizeMeiosisProtocol' currentExperimentString '.mat populationSizeMeiosisProtocol;']);
                    eval(['save ../../protocol_plots/averageTotalMutationsProtocol' currentExperimentString '.mat averageTotalMutationsProtocol;']);
                    eval(['save ../../protocol_plots/averageFunctionalMutationsProtocol' currentExperimentString '.mat averageFunctionalMutationsProtocol;']);
                    eval(['save ../../protocol_plots/zeroFitnessKillsMitosisProtocol' currentExperimentString '.mat zeroFitnessKillsMitosisProtocol;']);
                    eval(['save ../../protocol_plots/zeroFitnessKillsMeiosisProtocol' currentExperimentString '.mat zeroFitnessKillsMeiosisProtocol;']);
                    eval(['save ../../protocol_plots/starvationKillsMitosisProtocol' currentExperimentString '.mat starvationKillsMitosisProtocol;']);
                    eval(['save ../../protocol_plots/starvationKillsMeiosisProtocol' currentExperimentString '.mat starvationKillsMeiosisProtocol;']);
                    eval(['save ../../protocol_plots/averageRearrangementsProtocol' currentExperimentString '.mat averageRearrangementsProtocol;']);
                    eval(['save ../../protocol_plots/averageRealRearrangementsProtocol' currentExperimentString '.mat averageRealRearrangementsProtocol;']);
                    eval(['save ../../protocol_plots/averageRecombinationsProtocol' currentExperimentString '.mat averageRecombinationsProtocol;']);
                    eval(['save ../../protocol_plots/inbreedingAttemptsProtocol' currentExperimentString '.mat inbreedingAttemptsProtocol;']);
                    eval(['save ../../protocol_plots/successfulInbreedingsProtocol' currentExperimentString '.mat successfulInbreedingsProtocol;']);
                    eval(['save ../../protocol_plots/inbreedingSuccessRatio' currentExperimentString '.mat inbreedingSuccessRatio;']);
                    eval(['save ../../protocol_plots/outbreedingAttemptsProtocol' currentExperimentString '.mat outbreedingAttemptsProtocol;']);
                    eval(['save ../../protocol_plots/successfulOutbreedingsProtocol' currentExperimentString '.mat successfulOutbreedingsProtocol;']);
                    eval(['save ../../protocol_plots/outbreedingSuccessRatio' currentExperimentString '.mat outbreedingSuccessRatio;']);
                    eval(['save ../../protocol_plots/lackingHomologyKillsProtocol' currentExperimentString '.mat lackingHomologyKillsProtocol;']);
                    eval(['save ../../protocol_plots/lackingRPIEKillsProtocol' currentExperimentString '.mat lackingRPIEKillsProtocol;']);
                    eval(['save ../../protocol_plots/falseOutbreedersProtocol' currentExperimentString '.mat falseOutbreedersProtocol;']);
                    eval(['save ../../protocol_plots/populationLoadMitosisProtocol' currentExperimentString '.mat populationLoadMitosisProtocol;']);
                    eval(['save ../../protocol_plots/dropoutLoadMitosisProtocol' currentExperimentString '.mat dropoutLoadMitosisProtocol;']);
                    eval(['save ../../protocol_plots/populationLoadMeiosisProtocol' currentExperimentString '.mat populationLoadMeiosisProtocol;']);
                    eval(['save ../../protocol_plots/dropoutLoadMeiosisProtocol' currentExperimentString '.mat dropoutLoadMeiosisProtocol;']);
                    
                    % mitosisLoadProtocol:
                    % column 1 surviving population
                    % column 2 homozygous deleterious mutation dropouts
                    % column 3 starvation kills
                    
                    % meiosisLoadProtocol:
                    % column 1 surviving population
                    % column 2 homozygous deleterious mutation dropouts
                    % column 3 starvation kills
                    % column 4 zero recombination sites dropouts
                    
                    mitosisLoadProtocol = zeros(totalSimulationSteps / 2, 3);
                    meiosisLoadProtocol = zeros(totalSimulationSteps / 2, 4);
                    mitosisLoadProtocol(:, :) = NaN;
                    meiosisLoadProtocol(:, :) = NaN;
                    
                    for i = 1:(totalSimulationSteps / 2)
                        if populationLoadMitosisProtocol(i, 1) ~= 0
                            mitosisLoadProtocol(i, 1) = populationLoadMitosisProtocol(i, 2) / populationLoadMitosisProtocol(i, 1);
                        end;
                        if dropoutLoadMitosisProtocol(i, 1) ~= 0
                            mitosisLoadProtocol(i, 2) = dropoutLoadMitosisProtocol(i, 2) / dropoutLoadMitosisProtocol(i, 1);
                        end;
                        if dropoutLoadMitosisProtocol(i, 3) ~= 0
                            mitosisLoadProtocol(i, 3) = dropoutLoadMitosisProtocol(i, 4) / dropoutLoadMitosisProtocol(i, 3);
                        end;
                        if populationLoadMeiosisProtocol(i, 1) ~= 0
                            meiosisLoadProtocol(i, 1) = populationLoadMeiosisProtocol(i, 2) / populationLoadMeiosisProtocol(i, 1);
                        end;
                        if dropoutLoadMeiosisProtocol(i, 1) ~= 0
                            meiosisLoadProtocol(i, 2) = dropoutLoadMeiosisProtocol(i, 2) / dropoutLoadMeiosisProtocol(i, 1);
                        end;
                        if dropoutLoadMeiosisProtocol(i, 3) ~= 0
                            meiosisLoadProtocol(i, 3) = dropoutLoadMeiosisProtocol(i, 4) / dropoutLoadMeiosisProtocol(i, 3);
                        end;
                        if dropoutLoadMeiosisProtocol(i, 5) ~= 0
                            meiosisLoadProtocol(i, 4) = dropoutLoadMeiosisProtocol(i, 6) / dropoutLoadMeiosisProtocol(i, 5);
                        end;
                    end;
                    
                    eval(['save ../../protocol_plots/mitosisLoadProtocol' currentExperimentString '.mat mitosisLoadProtocol;']);
                    eval(['save ../../protocol_plots/meiosisLoadProtocol' currentExperimentString '.mat meiosisLoadProtocol;']);
                    
                    figure;
                    plot(1:length(mitosisLoadProtocol(:, 1)), mitosisLoadProtocol(:, 1)', 'g-', 'LineWidth', 2);
                    hold on;
                    plot(1:length(mitosisLoadProtocol(:, 1)), mitosisLoadProtocol(:, 2)', 'r-', 'LineWidth', 2);
                    hold on;
                    plot(1:length(mitosisLoadProtocol(:, 1)), mitosisLoadProtocol(:, 3)', 'b-', 'LineWidth', 2);

                    h = gca;
                    set(h, 'FontSize', 14, 'LineWidth', 1.5);
                    xlabel('generation');
                    ylabel('average mutagenic load per genome in mitosis');
                    if length(mitosisLoadProtocol(:, 1)) > 1
                        axis([1 length(mitosisLoadProtocol(:, 1)) 0 nEssentialGenes]);
                    else
                        axis([1 2 0 nEssentialGenes]);
                    end;
                    currentFile = ['../../protocol_plots/mitosisLoad' currentExperimentString '.jpg'];
                    saveas(h, currentFile);
                    currentFrame = imread(currentFile);
                    imwrite(currentFrame, currentFile, 'jpg', 'Quality', 100);
                    close;
                    
                    figure;
                    plot(1:length(meiosisLoadProtocol(:, 1)), meiosisLoadProtocol(:, 1)', 'g-', 'LineWidth', 2);
                    hold on;
                    plot(1:length(meiosisLoadProtocol(:, 1)), meiosisLoadProtocol(:, 2)', 'r-', 'LineWidth', 2);
                    hold on;
                    plot(1:length(meiosisLoadProtocol(:, 1)), meiosisLoadProtocol(:, 3)', 'b-', 'LineWidth', 2);
                    hold on;
                    plot(1:length(meiosisLoadProtocol(:, 1)), meiosisLoadProtocol(:, 4)', 'k-', 'LineWidth', 2);
                    
                    h = gca;
                    set(h, 'FontSize', 14, 'LineWidth', 1.5);
                    xlabel('generation');
                    ylabel('average mutagenic load per genome in meiosis');
                    if length(meiosisLoadProtocol(:, 1)) > 1
                        axis([1 length(meiosisLoadProtocol(:, 1)) 0 nEssentialGenes]);
                    else
                        axis([1 2 0 nEssentialGenes]);
                    end;
                    currentFile = ['../../protocol_plots/meiosisLoad' currentExperimentString '.jpg'];
                    saveas(h, currentFile);
                    currentFrame = imread(currentFile);
                    imwrite(currentFrame, currentFile, 'jpg', 'Quality', 100);
                    close;                    
                    
                    if adaptiveMutationRate
                        eval(['save ../../protocol_plots/mutationRateProtocol' currentExperimentString '.mat mutationRateProtocol;']);
                        
                        figure;
                        plot(1:length(mutationRateProtocol), mutationRateProtocol', 'LineWidth', 2);
    
                        h = gca;
                        set(h, 'FontSize', 14, 'LineWidth', 1.5);
                        xlabel('mitosis');
                        ylabel('mutation rate');
                        if length(mutationRateProtocol) > 1
                            axis([1 length(mutationRateProtocol) 0.9 * min(mutationRateProtocol) 1.1 * max(mutationRateProtocol)]);
                        else
                            axis([1 2 min(mutationRateProtocol) max(mutationRateProtocol)]);
                        end;
                        currentFile = ['../../protocol_plots/mutationRateProtocol' currentExperimentString '.jpg'];
                        saveas(h, currentFile);
                        currentFrame = imread(currentFile);
                        imwrite(currentFrame, currentFile, 'jpg', 'Quality', 100);
                        close;
                    end;
                    
                    figure;
                    plot(1:length(averageRecombinationSitesProtocol), averageRecombinationSitesProtocol', 'LineWidth', 2);

                    h = gca;
                    set(h, 'FontSize', 14, 'LineWidth', 1.5);
                    xlabel('generation');
                    ylabel('average number of proficient IEs');
                    if length(averageRecombinationSitesProtocol) > 1
                        axis([1 length(averageRecombinationSitesProtocol) 0 nGenes]);
                    else
                        axis([1 2 0 nGenes]);
                    end;
                    currentFile = ['../../protocol_plots/averageProficientRecombinationSites' currentExperimentString '.jpg'];
                    saveas(h, currentFile);
                    currentFrame = imread(currentFile);
                    imwrite(currentFrame, currentFile, 'jpg', 'Quality', 100);
                    close;

                    figure;
                    plot(1:length(averageFunctionalGenesProtocol), averageFunctionalGenesProtocol', 'LineWidth', 2);

                    h = gca;
                    set(h, 'FontSize', 14, 'LineWidth', 1.5);
                    xlabel('generation');
                    ylabel('av. number of functional essential genes');
                    if length(averageFunctionalGenesProtocol) > 1
                        axis([1 length(averageFunctionalGenesProtocol) nEssentialGenes / 2 nEssentialGenes]);
                    else
                        axis([1 2 nEssentialGenes / 2 nEssentialGenes]);
                    end;
                    currentFile = ['../../protocol_plots/averageFunctionalEssentialGenes' currentExperimentString '.jpg'];
                    saveas(h, currentFile);
                    currentFrame = imread(currentFile);
                    imwrite(currentFrame, currentFile, 'jpg', 'Quality', 100);
                    close;
                    
                    figure;
                    plot(1:length(populationSizeMitosisProtocol), populationSizeMitosisProtocol', 'LineWidth', 2);

                    h = gca;
                    set(h, 'FontSize', 14, 'LineWidth', 1.5);
                    xlabel('mitosis');
                    ylabel('population size after mitosis [chromosomes]');
                    if length(populationSizeMitosisProtocol) > 1
                        axis([1 length(populationSizeMitosisProtocol) 0 2.5 * maxGenomes]);
                    else
                        axis([1 2 0 2.5 * maxGenomes]);
                    end;
                    currentFile = ['../../protocol_plots/populationSizeAfterMitosis' currentExperimentString '.jpg'];
                    saveas(h, currentFile);
                    currentFrame = imread(currentFile);
                    imwrite(currentFrame, currentFile, 'jpg', 'Quality', 100);
                    close;
                    
                    figure;
                    plot(1:length(populationSizeMeiosisProtocol), populationSizeMeiosisProtocol', 'LineWidth', 2);

                    h = gca;
                    set(h, 'FontSize', 14, 'LineWidth', 1.5);
                    xlabel('meiosis');
                    ylabel('population size after meiosis [chromosomes]');
                    if length(populationSizeMeiosisProtocol) > 1
                        axis([1 length(populationSizeMeiosisProtocol) 0 2.5 * maxGenomes]);
                    else
                        axis([1 2 0 2.5 * maxGenomes]);
                    end;
                    currentFile = ['../../protocol_plots/populationSizeAfterMeiosis' currentExperimentString '.jpg'];
                    saveas(h, currentFile);
                    currentFrame = imread(currentFile);
                    imwrite(currentFrame, currentFile, 'jpg', 'Quality', 100);
                    close;
                    
                    figure;
                    plot(1:length(zeroFitnessKillsMitosisProtocol), zeroFitnessKillsMitosisProtocol', 'LineWidth', 2);

                    h = gca;
                    set(h, 'FontSize', 14, 'LineWidth', 1.5);
                    xlabel('mitosis');
                    ylabel('zero fitness kills during mitosis [chromosomes]');
                    if length(zeroFitnessKillsMitosisProtocol) > 1
                        axis([1 length(zeroFitnessKillsMitosisProtocol) 0 4 * maxGenomes]);
                    else
                        axis([1 2 0 4 * maxGenomes]);
                    end;
                    currentFile = ['../../protocol_plots/zeroFitnessKillsDuringMitosis' currentExperimentString '.jpg'];
                    saveas(h, currentFile);
                    currentFrame = imread(currentFile);
                    imwrite(currentFrame, currentFile, 'jpg', 'Quality', 100);
                    close;
                    
                    figure;
                    plot(1:length(zeroFitnessKillsMeiosisProtocol), zeroFitnessKillsMeiosisProtocol', 'LineWidth', 2);

                    h = gca;
                    set(h, 'FontSize', 14, 'LineWidth', 1.5);
                    xlabel('meiosis');
                    ylabel('zero fitness kills during meiosis [chromosomes]');
                    if length(zeroFitnessKillsMeiosisProtocol) > 1
                        axis([1 length(zeroFitnessKillsMeiosisProtocol) 0 4 * maxGenomes]);
                    else
                        axis([1 2 0 4 * maxGenomes]);
                    end;
                    currentFile = ['../../protocol_plots/zeroFitnessKillsDuringMeiosis' currentExperimentString '.jpg'];
                    saveas(h, currentFile);
                    currentFrame = imread(currentFile);
                    imwrite(currentFrame, currentFile, 'jpg', 'Quality', 100);
                    close;
                    
                    figure;
                    plot(1:length(starvationKillsMitosisProtocol), starvationKillsMitosisProtocol', 'LineWidth', 2);

                    h = gca;
                    set(h, 'FontSize', 14, 'LineWidth', 1.5);
                    xlabel('mitosis');
                    ylabel('starvation kills during mitosis [chromosomes]');
                    if length(starvationKillsMitosisProtocol) > 1
                        axis([1 length(starvationKillsMitosisProtocol) 0 2.5 * maxGenomes]);
                    else
                        axis([1 2 0 2.5 * maxGenomes]);
                    end;
                    currentFile = ['../../protocol_plots/starvationKillsDuringMitosis' currentExperimentString '.jpg'];
                    saveas(h, currentFile);
                    currentFrame = imread(currentFile);
                    imwrite(currentFrame, currentFile, 'jpg', 'Quality', 100);
                    close;
                    
                    figure;
                    plot(1:length(starvationKillsMeiosisProtocol), starvationKillsMeiosisProtocol', 'LineWidth', 2);

                    h = gca;
                    set(h, 'FontSize', 14, 'LineWidth', 1.5);
                    xlabel('meiosis');
                    ylabel('starvation kills during meiosis [chromosomes]');
                    if length(starvationKillsMeiosisProtocol) > 1
                        axis([1 length(starvationKillsMeiosisProtocol) 0 2.5 * maxGenomes]);
                    else
                        axis([1 2 0 2.5 * maxGenomes]);
                    end;
                    currentFile = ['../../protocol_plots/starvationKillsDuringMeiosis' currentExperimentString '.jpg'];
                    saveas(h, currentFile);
                    currentFrame = imread(currentFile);
                    imwrite(currentFrame, currentFile, 'jpg', 'Quality', 100);
                    close;
                    
                    figure;
                    plot(1:length(lackingHomologyKillsProtocol), lackingHomologyKillsProtocol', 'LineWidth', 2);

                    h = gca;
                    set(h, 'FontSize', 14, 'LineWidth', 1.5);
                    xlabel('meiosis');
                    ylabel('lacking homology kills during meiosis [chromosomes]');
                    if length(lackingHomologyKillsProtocol) > 1
                        axis([1 length(lackingHomologyKillsProtocol) 0 2 * maxGenomes]);
                    else
                        axis([1 2 0 2 * maxGenomes]);
                    end;
                    currentFile = ['../../protocol_plots/lackingHomologyKills' currentExperimentString '.jpg'];
                    saveas(h, currentFile);
                    currentFrame = imread(currentFile);
                    imwrite(currentFrame, currentFile, 'jpg', 'Quality', 100);
                    close;
                    
                    figure;
                    plot(1:length(lackingRPIEKillsProtocol), lackingRPIEKillsProtocol', 'LineWidth', 2);

                    h = gca;
                    set(h, 'FontSize', 14, 'LineWidth', 1.5);
                    xlabel('meiosis');
                    ylabel('lacking RPIE kills during meiosis [chromosomes]');
                    if length(lackingRPIEKillsProtocol) > 1
                        axis([1 length(lackingRPIEKillsProtocol) 0 2 * maxGenomes]);
                    else
                        axis([1 2 0 2 * maxGenomes]);
                    end;
                    currentFile = ['../../protocol_plots/lackingRPIEKills' currentExperimentString '.jpg'];
                    saveas(h, currentFile);
                    currentFrame = imread(currentFile);
                    imwrite(currentFrame, currentFile, 'jpg', 'Quality', 100);
                    close;
                    
                    figure;
                    plot(1:length(inbreedingAttemptsProtocol), inbreedingAttemptsProtocol', 'LineWidth', 2);

                    h = gca;
                    set(h, 'FontSize', 14, 'LineWidth', 1.5);
                    xlabel('meiosis');
                    ylabel('inbreeding attempts during meiosis [chromosome pairs]');
                    if length(inbreedingAttemptsProtocol) > 1
                        axis([1 length(inbreedingAttemptsProtocol) 0 2.5 * maxGenomes]);
                    else
                        axis([1 2 0 2.5 * maxGenomes]);
                    end;
                    currentFile = ['../../protocol_plots/inbreedingAttempts' currentExperimentString '.jpg'];
                    saveas(h, currentFile);
                    currentFrame = imread(currentFile);
                    imwrite(currentFrame, currentFile, 'jpg', 'Quality', 100);
                    close;
                    
                    figure;
                    plot(1:length(successfulInbreedingsProtocol), successfulInbreedingsProtocol', 'LineWidth', 2);

                    h = gca;
                    set(h, 'FontSize', 14, 'LineWidth', 1.5);
                    xlabel('meiosis');
                    ylabel('successful inbreedings during meiosis [chromosome pairs]');
                    if length(successfulInbreedingsProtocol) > 1
                        axis([1 length(successfulInbreedingsProtocol) 0 2.5 * maxGenomes]);
                    else
                        axis([1 2 0 2.5 * maxGenomes]);
                    end;
                    currentFile = ['../../protocol_plots/successfulInbreedings' currentExperimentString '.jpg'];
                    saveas(h, currentFile);
                    currentFrame = imread(currentFile);
                    imwrite(currentFrame, currentFile, 'jpg', 'Quality', 100);
                    close;
                    
                    figure;
                    plot(1:length(inbreedingSuccessRatio), inbreedingSuccessRatio', 'LineWidth', 2);

                    h = gca;
                    set(h, 'FontSize', 14, 'LineWidth', 1.5);
                    xlabel('meiosis');
                    ylabel('inbreeding success ratio during meiosis');
                    if length(inbreedingSuccessRatio) > 1
                        axis([1 length(inbreedingSuccessRatio) 0 1]);
                    else
                        axis([1 2 0 1]);
                    end;
                    currentFile = ['../../protocol_plots/inbreedingSuccessRatio' currentExperimentString '.jpg'];
                    saveas(h, currentFile);
                    currentFrame = imread(currentFile);
                    imwrite(currentFrame, currentFile, 'jpg', 'Quality', 100);
                    close;                    
                    
                    figure;
                    plot(1:length(outbreedingAttemptsProtocol), outbreedingAttemptsProtocol', 'LineWidth', 2);

                    h = gca;
                    set(h, 'FontSize', 14, 'LineWidth', 1.5);
                    xlabel('meiosis');
                    ylabel('outbreeding attempts during meiosis [chromosome pairs]');
                    if length(outbreedingAttemptsProtocol) > 1
                        axis([1 length(outbreedingAttemptsProtocol) 0 2.5 * maxGenomes]);
                    else
                        axis([1 2 0 2.5 * maxGenomes]);
                    end;
                    currentFile = ['../../protocol_plots/outbreedingAttempts' currentExperimentString '.jpg'];
                    saveas(h, currentFile);
                    currentFrame = imread(currentFile);
                    imwrite(currentFrame, currentFile, 'jpg', 'Quality', 100);
                    close;
                    
                    figure;
                    plot(1:length(successfulOutbreedingsProtocol), successfulOutbreedingsProtocol', 'LineWidth', 2);

                    h = gca;
                    set(h, 'FontSize', 14, 'LineWidth', 1.5);
                    xlabel('meiosis');
                    ylabel('successful outbreedings during meiosis [chromosome pairs]');
                    if length(successfulOutbreedingsProtocol) > 1
                        axis([1 length(successfulOutbreedingsProtocol) 0 2.5 * maxGenomes]);
                    else
                        axis([1 2 0 2.5 * maxGenomes]);
                    end;
                    currentFile = ['../../protocol_plots/successfulOutbreedings' currentExperimentString '.jpg'];
                    saveas(h, currentFile);
                    currentFrame = imread(currentFile);
                    imwrite(currentFrame, currentFile, 'jpg', 'Quality', 100);
                    close;
                    
                    figure;
                    plot(1:length(outbreedingSuccessRatio), outbreedingSuccessRatio', 'LineWidth', 2);

                    h = gca;
                    set(h, 'FontSize', 14, 'LineWidth', 1.5);
                    xlabel('meiosis');
                    ylabel('outbreeding success ratio during meiosis');
                    if length(outbreedingSuccessRatio) > 1
                        axis([1 length(outbreedingSuccessRatio) 0 1]);
                    else
                        axis([1 2 0 1]);
                    end;
                    currentFile = ['../../protocol_plots/outbreedingSuccessRatio' currentExperimentString '.jpg'];
                    saveas(h, currentFile);
                    currentFrame = imread(currentFile);
                    imwrite(currentFrame, currentFile, 'jpg', 'Quality', 100);
                    close;  
                    
                    figure;
                    plot(1:length(falseOutbreedersProtocol), falseOutbreedersProtocol', 'LineWidth', 2);

                    h = gca;
                    set(h, 'FontSize', 14, 'LineWidth', 1.5);
                    xlabel('meiosis');
                    ylabel('false outbreedings during meiosis');
                    if length(falseOutbreedersProtocol) > 1
                        axis([1 length(falseOutbreedersProtocol) 0 max(falseOutbreedersProtocol) + 1]);
                    else
                        axis([1 2 0 max(falseOutbreedersProtocol) + 1]);
                    end;
                    currentFile = ['../../protocol_plots/falseOutbreedersProtocol' currentExperimentString '.jpg'];
                    saveas(h, currentFile);
                    currentFrame = imread(currentFile);
                    imwrite(currentFrame, currentFile, 'jpg', 'Quality', 100);
                    close;  
                    
                    figure;
                    plot(1:length(averageTotalMutationsProtocol), averageTotalMutationsProtocol', 'LineWidth', 2);
                    
                    h = gca;
                    set(h, 'FontSize', 14, 'LineWidth', 1.5);
                    xlabel('mitosis');
                    ylabel('average total mutations during mitosis');
                    axis([1 length(averageTotalMutationsProtocol) 0 0.1 * nGenes]);
                    currentFile = ['../../protocol_plots/averageTotalMutations' currentExperimentString '.jpg'];
                    saveas(h, currentFile);
                    currentFrame = imread(currentFile);
                    imwrite(currentFrame, currentFile, 'jpg', 'Quality', 100);
                    if averageTotalMutationsProtocol == 0
                        if length(averageTotalMutationsProtocol) > 1
                            axis([1 length(averageTotalMutationsProtocol) 0 1]);
                        else
                            axis([1 2 0 1]);
                        end;
                    else
                        if length(averageTotalMutationsProtocol) > 1
                            axis([1 length(averageTotalMutationsProtocol) 0 1.1 * max(averageTotalMutationsProtocol)]);
                        else
                            axis([1 2 0 1.1 * max(averageTotalMutationsProtocol)]);
                        end;
                    end;
                    currentFile = ['../../protocol_plots/averageTotalMutationsIndividualScale' currentExperimentString '.jpg'];
                    saveas(h, currentFile);
                    currentFrame = imread(currentFile);
                    imwrite(currentFrame, currentFile, 'jpg', 'Quality', 100);
                    close;
                    
                    figure;
                    plot(1:length(averageFunctionalMutationsProtocol), averageFunctionalMutationsProtocol', 'LineWidth', 2);

                    h = gca;
                    set(h, 'FontSize', 14, 'LineWidth', 1.5);
                    xlabel('mitosis');
                    ylabel('average essential mutations during mitosis');
                    axis([1 length(averageFunctionalMutationsProtocol) 0 0.1 * nEssentialGenes]);
                    currentFile = ['../../protocol_plots/averageEssentialMutations' currentExperimentString '.jpg'];
                    saveas(h, currentFile);
                    currentFrame = imread(currentFile);
                    imwrite(currentFrame, currentFile, 'jpg', 'Quality', 100);
                    if averageFunctionalMutationsProtocol == 0
                        if length(averageFunctionalMutationsProtocol) > 1
                            axis([1 length(averageFunctionalMutationsProtocol) 0 1]);
                        else
                            axis([1 2 0 1]);
                        end;
                    else
                        if length(averageFunctionalMutationsProtocol) > 1
                            axis([1 length(averageFunctionalMutationsProtocol) 0 1.1 * max(averageFunctionalMutationsProtocol)]);
                        else
                            axis([1 2 0 1.1 * max(averageFunctionalMutationsProtocol)]);
                        end;
                    end;
                    currentFile = ['../../protocol_plots/averageEssentialMutationsIndividualScale' currentExperimentString '.jpg'];
                    saveas(h, currentFile);
                    currentFrame = imread(currentFile);
                    imwrite(currentFrame, currentFile, 'jpg', 'Quality', 100);
                    close;
                    
                    figure;
                    plot(1:length(averageRearrangementsProtocol), averageRearrangementsProtocol', 'LineWidth', 2);

                    h = gca;
                    set(h, 'FontSize', 14, 'LineWidth', 1.5);
                    xlabel('mitosis');
                    ylabel('average total rearrangements during mitosis');
                    axis([1 length(averageRearrangementsProtocol) 0 0.1 * (nGenomicElements + 1)]);
                    currentFile = ['../../protocol_plots/averageTotalRearrangements' currentExperimentString '.jpg'];
                    saveas(h, currentFile);
                    currentFrame = imread(currentFile);
                    imwrite(currentFrame, currentFile, 'jpg', 'Quality', 100);
                    if averageRearrangementsProtocol == 0
                        if length(averageRearrangementsProtocol) > 1
                            axis([1 length(averageRearrangementsProtocol) 0 1]);
                        else
                            axis([1 2 0 1]);
                        end;
                    else
                        if length(averageRearrangementsProtocol) > 1
                            axis([1 length(averageRearrangementsProtocol) 0 1.1 * max(averageRearrangementsProtocol)]);
                        else
                            axis([1 2 0 1.1 * max(averageRearrangementsProtocol)]);
                        end;
                    end;
                    currentFile = ['../../protocol_plots/averageTotalRearrangementsIndividualScale' currentExperimentString '.jpg'];
                    saveas(h, currentFile);
                    currentFrame = imread(currentFile);
                    imwrite(currentFrame, currentFile, 'jpg', 'Quality', 100);
                    close;
                    
                    figure;
                    plot(1:length(averageRealRearrangementsProtocol), averageRealRearrangementsProtocol', 'LineWidth', 2);

                    h = gca;
                    set(h, 'FontSize', 14, 'LineWidth', 1.5);
                    xlabel('mitosis');
                    ylabel('average effective rearrangements during mitosis');
                    axis([1 length(averageRealRearrangementsProtocol) 0 0.01 * (nGenomicElements + 1)]);
                    currentFile = ['../../protocol_plots/averageEffectiveRearrangements' currentExperimentString '.jpg'];
                    saveas(h, currentFile);
                    currentFrame = imread(currentFile);
                    imwrite(currentFrame, currentFile, 'jpg', 'Quality', 100);
                    if averageRealRearrangementsProtocol == 0
                        if length(averageRealRearrangementsProtocol) > 1
                            axis([1 length(averageRealRearrangementsProtocol) 0 1]);
                        else
                            axis([1 2 0 1]);
                        end;
                    else
                        if length(averageRealRearrangementsProtocol) > 1
                            axis([1 length(averageRealRearrangementsProtocol) 0 1.1 * max(averageRealRearrangementsProtocol)]);
                        else
                            axis([1 2 0 1.1 * max(averageRealRearrangementsProtocol)]);
                        end;
                    end;
                    currentFile = ['../../protocol_plots/averageEffectiveRearrangementsIndividualScale' currentExperimentString '.jpg'];
                    saveas(h, currentFile);
                    currentFrame = imread(currentFile);
                    imwrite(currentFrame, currentFile, 'jpg', 'Quality', 100);
                    close;
                    
                    figure;
                    plot(1:length(averageRecombinationsProtocol), averageRecombinationsProtocol', 'LineWidth', 2);

                    h = gca;
                    set(h, 'FontSize', 14, 'LineWidth', 1.5);
                    xlabel('meiosis');
                    ylabel('average recombination events during mitosis [per quadruplet]');
                    axis([1 length(averageRecombinationsProtocol) 0 0.1 * nGenomicElements]);
                    currentFile = ['../../protocol_plots/averageRecombinations' currentExperimentString '.jpg'];
                    saveas(h, currentFile);
                    currentFrame = imread(currentFile);
                    imwrite(currentFrame, currentFile, 'jpg', 'Quality', 100);
                    if averageRecombinationsProtocol == 0
                        if length(averageRecombinationsProtocol) > 1
                            axis([1 length(averageRecombinationsProtocol) 0 1]);
                        else
                            axis([1 2 0 1]);
                        end;
                    else
                        if length(averageRecombinationsProtocol) > 1
                            axis([1 length(averageRecombinationsProtocol) 0 1.1 * max(averageRecombinationsProtocol)]);
                        else
                            axis([1 2 0 1.1 * max(averageRecombinationsProtocol)]);
                        end;
                    end;
                    currentFile = ['../../protocol_plots/averageRecombinationsIndividualScale' currentExperimentString '.jpg'];
                    saveas(h, currentFile);
                    currentFrame = imread(currentFile);
                    imwrite(currentFrame, currentFile, 'jpg', 'Quality', 100);
                    close;
                    
                    if deathmatchFlag
                        figure;
                        plot(1:length(population1SizeMitosisProtocol), population1SizeMitosisProtocol', 'LineWidth', 2);

                        h = gca;
                        set(h, 'FontSize', 14, 'LineWidth', 1.5);
                        xlabel('mitosis');
                        ylabel('population 1 size after mitosis [chromosomes]');
                        if length(population1SizeMitosisProtocol) > 1
                            axis([1 length(population1SizeMitosisProtocol) 0 2.5 * maxGenomes]);
                        else
                            axis([1 2 0 2.5 * maxGenomes]);
                        end;
                        currentFile = ['../../protocol_plots/population1SizeAfterMitosis' currentExperimentString '.jpg'];
                        saveas(h, currentFile);
                        currentFrame = imread(currentFile);
                        imwrite(currentFrame, currentFile, 'jpg', 'Quality', 100);
                        close;
                        
                        figure;
                        plot(1:length(population2SizeMitosisProtocol), population2SizeMitosisProtocol', 'LineWidth', 2);

                        h = gca;
                        set(h, 'FontSize', 14, 'LineWidth', 1.5);
                        xlabel('mitosis');
                        ylabel('population 2 size after mitosis [chromosomes]');
                        if length(population2SizeMitosisProtocol) > 1
                            axis([1 length(population2SizeMitosisProtocol) 0 2.5 * maxGenomes]);
                        else
                            axis([1 2 0 2.5 * maxGenomes]);
                        end;
                        currentFile = ['../../protocol_plots/population2SizeAfterMitosis' currentExperimentString '.jpg'];
                        saveas(h, currentFile);
                        currentFrame = imread(currentFile);
                        imwrite(currentFrame, currentFile, 'jpg', 'Quality', 100);
                        close;
                        
                        figure;
                        plot(1:length(population1SizeMeiosisProtocol), population1SizeMeiosisProtocol', 'LineWidth', 2);

                        h = gca;
                        set(h, 'FontSize', 14, 'LineWidth', 1.5);
                        xlabel('meiosis');
                        ylabel('population 1 size after meiosis [chromosomes]');
                        if length(population1SizeMeiosisProtocol) > 1
                            axis([1 length(population1SizeMeiosisProtocol) 0 2.5 * maxGenomes]);
                        else
                            axis([1 2 0 2.5 * maxGenomes]);
                        end;
                        currentFile = ['../../protocol_plots/population1SizeAfterMeiosis' currentExperimentString '.jpg'];
                        saveas(h, currentFile);
                        currentFrame = imread(currentFile);
                        imwrite(currentFrame, currentFile, 'jpg', 'Quality', 100);
                        close;

                        figure;
                        plot(1:length(population2SizeMeiosisProtocol), population2SizeMeiosisProtocol', 'LineWidth', 2);

                        h = gca;
                        set(h, 'FontSize', 14, 'LineWidth', 1.5);
                        xlabel('meiosis');
                        ylabel('population 2 size after meiosis [chromosomes]');
                        if length(population2SizeMeiosisProtocol) > 1
                            axis([1 length(population2SizeMeiosisProtocol) 0 2.5 * maxGenomes]);
                        else
                            axis([1 2 0 2.5 * maxGenomes]);
                        end;
                        currentFile = ['../../protocol_plots/population2SizeAfterMeiosis' currentExperimentString '.jpg'];
                        saveas(h, currentFile);
                        currentFrame = imread(currentFile);
                        imwrite(currentFrame, currentFile, 'jpg', 'Quality', 100);
                        close;
                    end;
                    
                end;
                
                if mod(currentSimulationStep, protocolSplittingSize) ~= 0 && silentIO == 0
                    
                    currentProtocolSectionFormatted = num2str(currentProtocolSection + 1, '%.5d');
                    
                    eval(['proficientRecombinationSitesProtocol' currentProtocolSectionFormatted ' = proficientRecombinationSitesProtocol;']);
                    eval(['save clustering/proficientRecombinationSitesProtocol' currentProtocolSectionFormatted ' proficientRecombinationSitesProtocol' currentProtocolSectionFormatted ';']);
                    eval(['clear proficientRecombinationSitesProtocol' currentProtocolSectionFormatted ';']);
                    
                    eval(['geneClusterSizesProtocol' currentProtocolSectionFormatted ' = geneClusterSizesProtocol;']);
                    eval(['save clustering/geneClusterSizesProtocol' currentProtocolSectionFormatted ' geneClusterSizesProtocol' currentProtocolSectionFormatted ';']);
                    eval(['clear geneClusterSizesProtocol' currentProtocolSectionFormatted ';']);

                    eval(['starvationProtocol' currentProtocolSectionFormatted ' = starvationProtocol;']);
                    eval(['save protocols/starvationProtocol' currentProtocolSectionFormatted ' starvationProtocol' currentProtocolSectionFormatted ';']);
                    eval(['clear starvationProtocol' currentProtocolSectionFormatted ';']);

                    eval(['recombinationProtocol' currentProtocolSectionFormatted ' = recombinationProtocol;']);
                    eval(['save protocols/recombinationProtocol' currentProtocolSectionFormatted ' recombinationProtocol' currentProtocolSectionFormatted ';']);
                    eval(['clear recombinationProtocol' currentProtocolSectionFormatted ';']);

                    eval(['rearrangementProtocol' currentProtocolSectionFormatted ' = rearrangementProtocol;']);
                    eval(['save protocols/rearrangementProtocol' currentProtocolSectionFormatted ' rearrangementProtocol' currentProtocolSectionFormatted ';']);
                    eval(['clear rearrangementProtocol' currentProtocolSectionFormatted ';']);

                    eval(['mutationProtocol' currentProtocolSectionFormatted ' = mutationProtocol;']);
                    eval(['save protocols/mutationProtocol' currentProtocolSectionFormatted ' mutationProtocol' currentProtocolSectionFormatted ';']);
                    eval(['clear mutationProtocol' currentProtocolSectionFormatted ';']);

                    eval(['matingProtocol' currentProtocolSectionFormatted ' = matingProtocol;']);
                    eval(['save protocols/matingProtocol' currentProtocolSectionFormatted ' matingProtocol' currentProtocolSectionFormatted ';']);
                    eval(['clear matingProtocol' currentProtocolSectionFormatted ';']);

                elseif totalSimulationSteps == protocolSplittingSize && silentIO == 0

                    save clustering/proficientRecombinationSitesProtocol proficientRecombinationSitesProtocol;
                    save clustering/geneClusterSizesProtocol geneClusterSizesProtocol;
                    save protocols/starvationProtocol starvationProtocol;
                    save protocols/recombinationProtocol recombinationProtocol;
                    save protocols/rearrangementProtocol rearrangementProtocol;
                    save protocols/mutationProtocol mutationProtocol;
                    save protocols/matingProtocol matingProtocol;

                end;

                if frontendFlag == 0
                    if ~clusterMode
                        currentDirectory = 'protocols';
                    else
                        currentDirectory = masterDirectory;
                    end;
                    eval(['save ' currentDirectory '/simulationProtocol simulationProtocol;']);
                    % save protocols/simulationProtocol.txt simulationProtocol -ascii;

                    %unix('cd ..');
                    if ~clusterMode
                        cd('..');
                    end;
                end;
                
                if analyzeClustering && silentIO ~= 1
                    if fittingScheme == 0
                        clusterAnalysisMatrix(a + 1, b + 1) = clusterAnalysisResults(1, 7);
                        if clusterAnalysisResults(1, 7) ~= 0
                            clusterAnalysisMatrixUnfitted(a + 1, b + 1) = mean(clusterAnalysis((length(clusterAnalysis) - parameterAveragingRange):length(clusterAnalysis), 1));
                            averageRecombinationSitesAnalysis(a + 1, b + 1) = mean(averageRecombinationSitesArray((length(clusterAnalysis) - parameterAveragingRange):length(clusterAnalysis), 1));
                            averageFunctionalGenesAnalysis(a + 1, b + 1) = mean(averageFunctionalGenesArray((length(clusterAnalysis) - parameterAveragingRange):length(clusterAnalysis), 1));
                        else
                            clusterAnalysisMatrixUnfitted(a + 1, b + 1) = 0;
                            averageRecombinationSitesAnalysis(a + 1, b + 1) = NaN;
                            averageFunctionalGenesAnalysis(a + 1, b + 1) = NaN;
                        end;
                    elseif fittingScheme == 1
                        clusterAnalysisMatrix(a + 1, b + 1) = clusterAnalysisResults(1, 10);
                        if clusterAnalysisResults(1, 10) ~= 0
                            clusterAnalysisMatrixUnfitted(a + 1, b + 1) = mean(clusterAnalysis((length(clusterAnalysis) - parameterAveragingRange):length(clusterAnalysis), 1));
                            averageRecombinationSitesAnalysis(a + 1, b + 1) = mean(averageRecombinationSitesArray((length(clusterAnalysis) - parameterAveragingRange):length(clusterAnalysis), 1));
                            averageFunctionalGenesAnalysis(a + 1, b + 1) = mean(averageFunctionalGenesArray((length(clusterAnalysis) - parameterAveragingRange):length(clusterAnalysis), 1));
                        else
                            clusterAnalysisMatrixUnfitted(a + 1, b + 1) = 0;
                            averageRecombinationSitesAnalysis(a + 1, b + 1) = NaN;
                            averageFunctionalGenesAnalysis(a + 1, b + 1) = NaN;
                        end;
                    end;

                    if performSurvivalScreen == 2
                        totalClusterAnalysisResults(horizontalParameters * (a - 1) + b, 1) = seedingKnockoutArray(1, a);
                        totalClusterAnalysisResults(horizontalParameters * (a - 1) + b, 2) = clusterFrequencyArray(1, b);
                        totalSlidingWindowResults((2 * (horizontalParameters * (a - 1) + b) - 1):(2 * (horizontalParameters * (a - 1) + b)), 1) = seedingKnockoutArray(1, a);
                        totalSlidingWindowResults((2 * (horizontalParameters * (a - 1) + b) - 1):(2 * (horizontalParameters * (a - 1) + b)), 2) = clusterFrequencyArray(1, b);
                        totalSlidingWindowDeviationResults((2 * (horizontalParameters * (a - 1) + b) - 1):(2 * (horizontalParameters * (a - 1) + b)), 1) = seedingKnockoutArray(1, a);
                        totalSlidingWindowDeviationResults((2 * (horizontalParameters * (a - 1) + b) - 1):(2 * (horizontalParameters * (a - 1) + b)), 2) = clusterFrequencyArray(1, b);
                    else
                        totalClusterAnalysisResults(horizontalParameters * (a - 1) + b, 1) = verticalParametersList(1, a);
                        totalClusterAnalysisResults(horizontalParameters * (a - 1) + b, 2) = horizontalParametersList(1, b);
                        totalSlidingWindowResults((2 * (horizontalParameters * (a - 1) + b) - 1):(2 * (horizontalParameters * (a - 1) + b)), 1) = verticalParametersList(1, a);
                        totalSlidingWindowResults((2 * (horizontalParameters * (a - 1) + b) - 1):(2 * (horizontalParameters * (a - 1) + b)), 2) = horizontalParametersList(1, b);
                        totalSlidingWindowDeviationResults((2 * (horizontalParameters * (a - 1) + b) - 1):(2 * (horizontalParameters * (a - 1) + b)), 1) = verticalParametersList(1, a);
                        totalSlidingWindowDeviationResults((2 * (horizontalParameters * (a - 1) + b) - 1):(2 * (horizontalParameters * (a - 1) + b)), 2) = horizontalParametersList(1, b);
                    end;

                    if fittingScheme == 0
                        totalClusterAnalysisResults(horizontalParameters * (a - 1) + b, 3:9) = clusterAnalysisResults(1, :);
                    elseif fittingScheme == 1
                        totalClusterAnalysisResults(horizontalParameters * (a - 1) + b, 3:12) = clusterAnalysisResults(1, :);
                    end;

                    totalSlidingWindowResults(2 * (horizontalParameters * (a - 1) + b) - 1, 3:13) = slidingWindowEssentialGenesHistogram(:, :);
                    totalSlidingWindowResults(2 * (horizontalParameters * (a - 1) + b), 3:13) = slidingWindowRecombinationSitesHistogram(:, :);
                    totalSlidingWindowDeviationResults(2 * (horizontalParameters * (a - 1) + b) - 1, 3:13) = slidingWindowEssentialGenesHistogramDeviations(:, :);
                    totalSlidingWindowDeviationResults(2 * (horizontalParameters * (a - 1) + b), 3:13) = slidingWindowRecombinationSitesHistogramDeviations(:, :);
                end;
                
            end;
            
        end;
    end;
    
    % ---------------------
    % | Saving of results |
    % ---------------------

    if silentIO == 0
        save results/clusterAnalysisMatrix clusterAnalysisMatrix;
        % save results/clusterAnalysisMatrix.txt clusterAnalysisMatrix -ascii;
        save results/clusterAnalysisMatrixUnfitted clusterAnalysisMatrixUnfitted;
        % save results/clusterAnalysisMatrixUnfitted.txt clusterAnalysisMatrixUnfitted -ascii;
        save results/totalClusterAnalysisResults totalClusterAnalysisResults;
        % save results/totalClusterAnalysisResults.txt totalClusterAnalysisResults -ascii;
        save results/totalSlidingWindowResults totalSlidingWindowResults;
        % save results/totalSlidingWindowResults.txt totalSlidingWindowResults -ascii;
        save results/totalSlidingWindowDeviationResults totalSlidingWindowDeviationResults;
        % save results/totalSlidingWindowDeviationResults.txt totalSlidingWindowDeviationResults -ascii;
        save results/averageRecombinationSitesAnalysis averageRecombinationSitesAnalysis;
        % save results/averageRecombinationSitesAnalysis.txt averageRecombinationSitesAnalysis -ascii;
        save results/averageFunctionalGenesAnalysis averageFunctionalGenesAnalysis;
        % save results/averageFunctionalGenesAnalysis.txt averageFunctionalGenesAnalysis -ascii;
        save results/groupingMatrix groupingMatrix;
        save results/groupingMatrix.txt groupingMatrix -ascii;

        totalSlidingWindowResults
        totalSlidingWindowDeviationResults
        totalClusterAnalysisResults
        clusterAnalysisMatrix
        clusterAnalysisMatrixUnfitted
        groupingMatrix
        averageRecombinationSitesAnalysis
        averageFunctionalGenesAnalysis    

        figure;
        imagesc(averageRecombinationSitesAnalysis(2:(verticalParameters + 1), 2:(horizontalParameters + 1)));

        h = gca;
        colorbar;
        set(h, 'FontSize', 14, 'LineWidth', 1.5);
        xlabel(horizontalClass);
        ylabel(verticalClass);
        axis off;
        axis equal;
        currentFile = ['results/averageRecombinationSitesAnalysis.jpg'];
        saveas(h, currentFile);
        currentFrame = imread(currentFile);
        imwrite(currentFrame, currentFile, 'jpg', 'Quality', 100);
        close;

        figure;
        imagesc(averageFunctionalGenesAnalysis(2:(verticalParameters + 1), 2:(horizontalParameters + 1)));

        h = gca;
        colorbar;
        set(h, 'FontSize', 14, 'LineWidth', 1.5);
        xlabel(horizontalClass);
        ylabel(verticalClass);
        axis off;
        axis equal;
        currentFile = ['results/averageFunctionalGenesAnalysis.jpg'];
        saveas(h, currentFile);
        currentFrame = imread(currentFile);
        imwrite(currentFrame, currentFile, 'jpg', 'Quality', 100);
        close;

        figure;
        imagesc(clusterAnalysisMatrix(2:(verticalParameters + 1), 2:(horizontalParameters + 1)));

        h = gca;
        colorbar;
        set(h, 'FontSize', 14, 'LineWidth', 1.5);
        xlabel(horizontalClass);
        ylabel(verticalClass);
        axis off;
        axis equal;
        currentFile = ['results/clusterAnalysisMatrix.jpg'];
        saveas(h, currentFile);
        currentFrame = imread(currentFile);
        imwrite(currentFrame, currentFile, 'jpg', 'Quality', 100);
        close;

        figure;
        imagesc(clusterAnalysisMatrixUnfitted(2:(verticalParameters + 1), 2:(horizontalParameters + 1)));

        h = gca;
        colorbar;
        set(h, 'FontSize', 14, 'LineWidth', 1.5);
        xlabel(horizontalClass);
        ylabel(verticalClass);
        axis off;
        axis equal;
        currentFile = ['results/clusterAnalysisMatrixUnfitted.jpg'];
        saveas(h, currentFile);
        currentFrame = imread(currentFile);
        imwrite(currentFrame, currentFile, 'jpg', 'Quality', 100);
        close;

        figure;
        imagesc(groupingMatrix(2:(verticalParameters + 1), 2:(horizontalParameters + 1)));

        h = gca;
        colorbar;
        set(h, 'FontSize', 14, 'LineWidth', 1.5);
        xlabel(horizontalClass);
        ylabel(verticalClass);
        axis off;
        axis equal;
        currentFile = ['results/groupingMatrix.jpg'];
        saveas(h, currentFile);
        currentFrame = imread(currentFile);
        imwrite(currentFrame, currentFile, 'jpg', 'Quality', 100);
        close;

        statisticalAnalysisMatrix(:, :, simulationSession) = clusterAnalysisMatrix(:, :);
        statisticalAnalysisMatrixUnfitted(:, :, simulationSession) = clusterAnalysisMatrixUnfitted(:, :);
        statisticalClusterAnalysisResults(:, :, simulationSession) = totalClusterAnalysisResults(:, :);
        statisticalSlidingWindowResults(:, :, simulationSession) = totalSlidingWindowResults(:, :);
        statisticalSlidingWindowDeviationResults(:, :, simulationSession) = totalSlidingWindowDeviationResults(:, :);
        statisticalAnalysisRecombinationSites(:, :, simulationSession) = averageRecombinationSitesAnalysis(:, :);
        statisticalAnalysisFunctionalGenes(:, :, simulationSession) = averageFunctionalGenesAnalysis(:, :);
        statisticalAnalysisGrouping(:, :, simulationSession) = groupingMatrix(:, :);
    end;
    
    if frontendFlag == 0 && ~clusterMode
        %unix('cd ..');
        cd('..');
    end;
end;

% -----------------------
% | Statistical results |
% -----------------------

if silentIO == 0
    if displayInfo
        disp(' '); disp(' '); disp(' ');
        disp('---------------------------------');
        disp('Statistical results of simulation');
        disp('---------------------------------');
        disp(' ');
    end;
    
    save averaged_results/statisticalSlidingWindowResults statisticalSlidingWindowResults;
    save averaged_results/statisticalSlidingWindowDeviationResults statisticalSlidingWindowDeviationResults;
    save averaged_results/statisticalAnalysisMatrix statisticalAnalysisMatrix;
    save averaged_results/statisticalClusterAnalysisResults statisticalClusterAnalysisResults;
    save averaged_results/statisticalAnalysisMatrixUnfitted statisticalAnalysisMatrixUnfitted;
    save averaged_results/statisticalAnalysisRecombinationSites statisticalAnalysisRecombinationSites;
    save averaged_results/statisticalAnalysisFunctionalGenes statisticalAnalysisFunctionalGenes;
    save averaged_results/statisticalAnalysisGrouping statisticalAnalysisGrouping;

    meanAnalysisMatrix = zeros(verticalParameters + 1, horizontalParameters + 1);
    meanAnalysisMatrix(1, 1) = NaN;

    if performSurvivalScreen == 2
        meanAnalysisMatrix(2:(verticalParameters + 1), 1) = seedingKnockoutArray';
        meanAnalysisMatrix(1, 2:(horizontalParameters + 1)) = clusterFrequencyArray';
    else
        meanAnalysisMatrix(2:(verticalParameters + 1), 1) = verticalParametersList';
        meanAnalysisMatrix(1, 2:(horizontalParameters + 1)) = horizontalParametersList';
    end;

    deviationAnalysisMatrix = meanAnalysisMatrix;

    meanAnalysisMatrixUnfitted = meanAnalysisMatrix;
    deviationAnalysisMatrixUnfitted = meanAnalysisMatrix;
    meanRecombinationSitesAnalysisMatrix = meanAnalysisMatrix;
    deviationRecombinationSitesAnalysisMatrix = meanAnalysisMatrix;
    meanFunctionalGenesAnalysisMatrix = meanAnalysisMatrix;
    deviationFunctionalGenesAnalysisMatrix = meanAnalysisMatrix;
    meanGroupingMatrix = meanAnalysisMatrix;
    deviationGroupingMatrix = meanAnalysisMatrix;

    [xSize ySize zSize] = size(statisticalAnalysisMatrix);

    for n = 2:xSize
        for p = 2:ySize
            successfulRuns = [];
            for m = 1:zSize
                if statisticalAnalysisMatrix(n, p, m) ~= 0
                    successfulRuns = cat(2, successfulRuns, m);
                end;
            end;
            if ~isempty(successfulRuns)
                meanAnalysisMatrix(n, p) = mean(statisticalAnalysisMatrix(n, p, successfulRuns), 3);
                deviationAnalysisMatrix(n, p) = std(statisticalAnalysisMatrix(n, p, successfulRuns), 0, 3);
                meanAnalysisMatrixUnfitted(n, p) = mean(statisticalAnalysisMatrixUnfitted(n, p, successfulRuns), 3);
                deviationAnalysisMatrixUnfitted(n, p) = std(statisticalAnalysisMatrixUnfitted(n, p, successfulRuns), 0, 3);
                meanRecombinationSitesAnalysisMatrix(n, p) = mean(statisticalAnalysisRecombinationSites(n, p, successfulRuns), 3);
                deviationRecombinationSitesAnalysisMatrix(n, p) = std(statisticalAnalysisRecombinationSites(n, p, successfulRuns), 0, 3);
                meanFunctionalGenesAnalysisMatrix(n, p) = mean(statisticalAnalysisFunctionalGenes(n, p, successfulRuns), 3);
                deviationFunctionalGenesAnalysisMatrix(n, p) = std(statisticalAnalysisFunctionalGenes(n, p, successfulRuns), 0, 3);
                meanGroupingMatrix(n, p) = mean(statisticalAnalysisGrouping(n, p, successfulRuns), 3);
                deviationGroupingMatrix(n, p) = std(statisticalAnalysisGrouping(n, p, successfulRuns), 0, 3);
            else
                meanRecombinationSitesAnalysisMatrix(n, p) = NaN;
                deviationRecombinationSitesAnalysisMatrix(n, p) = NaN;
                meanFunctionalGenesAnalysisMatrix(n, p) = NaN;
                deviationFunctionalGenesAnalysisMatrix(n, p) = NaN;
            end;
        end;
    end;

    save averaged_results/meanAnalysisMatrix meanAnalysisMatrix;
    % save averaged_results/meanAnalysisMatrix.txt meanAnalysisMatrix -ascii;
    save averaged_results/deviationAnalysisMatrix deviationAnalysisMatrix;
    % save averaged_results/deviationAnalysisMatrix.txt deviationAnalysisMatrix -ascii;

    save averaged_results/meanAnalysisMatrixUnfitted meanAnalysisMatrixUnfitted;
    % save averaged_results/meanAnalysisMatrixUnfitted.txt meanAnalysisMatrixUnfitted -ascii;
    save averaged_results/deviationAnalysisMatrixUnfitted deviationAnalysisMatrixUnfitted;
    % save averaged_results/deviationAnalysisMatrixUnfitted.txt deviationAnalysisMatrixUnfitted -ascii;

    save averaged_results/meanRecombinationSitesAnalysisMatrix meanRecombinationSitesAnalysisMatrix;
    % save averaged_results/meanRecombinationSitesAnalysisMatrix.txt meanRecombinationSitesAnalysisMatrix -ascii;
    save averaged_results/deviationRecombinationSitesAnalysisMatrix deviationRecombinationSitesAnalysisMatrix;
    % save averaged_results/deviationRecombinationSitesAnalysisMatrix.txt deviationRecombinationSitesAnalysisMatrix -ascii;

    save averaged_results/meanFunctionalGenesAnalysisMatrix meanFunctionalGenesAnalysisMatrix;
    % save averaged_results/meanFunctionalGenesAnalysisMatrix.txt meanFunctionalGenesAnalysisMatrix -ascii;
    save averaged_results/deviationFunctionalGenesAnalysisMatrix deviationFunctionalGenesAnalysisMatrix;
    % save averaged_results/deviationFunctionalGenesAnalysisMatrix.txt deviationFunctionalGenesAnalysisMatrix -ascii;

    save averaged_results/meanGroupingMatrix meanGroupingMatrix;
    % save averaged_results/meanGroupingMatrix.txt meanGroupingMatrix -ascii;
    save averaged_results/deviationGroupingMatrix deviationGroupingMatrix;
    % save averaged_results/deviationGroupingMatrix.txt deviationGroupingMatrix -ascii;

    meanAnalysisMatrix
    deviationAnalysisMatrix
    meanAnalysisMatrixUnfitted
    deviationAnalysisMatrixUnfitted
    meanRecombinationSitesAnalysisMatrix
    deviationRecombinationSitesAnalysisMatrix
    meanFunctionalGenesAnalysisMatrix
    deviationFunctionalGenesAnalysisMatrix
    meanGroupingMatrix
    deviationGroupingMatrix

    figure;
    imagesc(deviationRecombinationSitesAnalysisMatrix(2:(verticalParameters + 1), 2:(horizontalParameters + 1)));

    h = gca;
    colorbar;
    set(h, 'FontSize', 14, 'LineWidth', 1.5);
    xlabel(horizontalClass);
    ylabel(verticalClass);
    axis off;
    axis equal;
    currentFile = ['averaged_results/deviationRecombinationSitesAnalysisMatrix.jpg'];
    saveas(h, currentFile);
    currentFrame = imread(currentFile);
    imwrite(currentFrame, currentFile, 'jpg', 'Quality', 100);
    close;

    figure;
    imagesc(meanRecombinationSitesAnalysisMatrix(2:(verticalParameters + 1), 2:(horizontalParameters + 1)));

    h = gca;
    colorbar;
    set(h, 'FontSize', 14, 'LineWidth', 1.5);
    xlabel(horizontalClass);
    ylabel(verticalClass);
    axis off;
    axis equal;
    currentFile = ['averaged_results/meanRecombinationSitesAnalysisMatrix.jpg'];
    saveas(h, currentFile);
    currentFrame = imread(currentFile);
    imwrite(currentFrame, currentFile, 'jpg', 'Quality', 100);
    close;

    figure;
    imagesc(deviationFunctionalGenesAnalysisMatrix(2:(verticalParameters + 1), 2:(horizontalParameters + 1)));

    h = gca;
    colorbar;
    set(h, 'FontSize', 14, 'LineWidth', 1.5);
    xlabel(horizontalClass);
    ylabel(verticalClass);
    axis off;
    axis equal;
    currentFile = ['averaged_results/deviationFunctionalGenesAnalysisMatrix.jpg'];
    saveas(h, currentFile);
    currentFrame = imread(currentFile);
    imwrite(currentFrame, currentFile, 'jpg', 'Quality', 100);
    close;

    figure;
    imagesc(meanFunctionalGenesAnalysisMatrix(2:(verticalParameters + 1), 2:(horizontalParameters + 1)));

    h = gca;
    colorbar;
    set(h, 'FontSize', 14, 'LineWidth', 1.5);
    xlabel(horizontalClass);
    ylabel(verticalClass);
    axis off;
    axis equal;
    currentFile = ['averaged_results/meanFunctionalGenesAnalysisMatrix.jpg'];
    saveas(h, currentFile);
    currentFrame = imread(currentFile);
    imwrite(currentFrame, currentFile, 'jpg', 'Quality', 100);
    close;

    figure;
    imagesc(deviationAnalysisMatrix(2:(verticalParameters + 1), 2:(horizontalParameters + 1)));

    h = gca;
    colorbar;
    set(h, 'FontSize', 14, 'LineWidth', 1.5);
    xlabel(horizontalClass);
    ylabel(verticalClass);
    axis off;
    axis equal;
    currentFile = ['averaged_results/deviationAnalysisMatrix.jpg'];
    saveas(h, currentFile);
    currentFrame = imread(currentFile);
    imwrite(currentFrame, currentFile, 'jpg', 'Quality', 100);
    close;

    figure;
    imagesc(meanAnalysisMatrix(2:(verticalParameters + 1), 2:(horizontalParameters + 1)));

    h = gca;
    colorbar;
    set(h, 'FontSize', 14, 'LineWidth', 1.5);
    xlabel(horizontalClass);
    ylabel(verticalClass);
    axis off;
    axis equal;
    currentFile = ['averaged_results/meanAnalysisMatrix.jpg'];
    saveas(h, currentFile);
    currentFrame = imread(currentFile);
    imwrite(currentFrame, currentFile, 'jpg', 'Quality', 100);
    close;

    figure;
    imagesc(deviationAnalysisMatrixUnfitted(2:(verticalParameters + 1), 2:(horizontalParameters + 1)));

    h = gca;
    colorbar;
    set(h, 'FontSize', 14, 'LineWidth', 1.5);
    xlabel(horizontalClass);
    ylabel(verticalClass);
    axis off;
    axis equal;
    currentFile = ['averaged_results/deviationAnalysisMatrixUnfitted.jpg'];
    saveas(h, currentFile);
    currentFrame = imread(currentFile);
    imwrite(currentFrame, currentFile, 'jpg', 'Quality', 100);
    close;

    figure;
    imagesc(meanAnalysisMatrixUnfitted(2:(verticalParameters + 1), 2:(horizontalParameters + 1)));

    h = gca;
    colorbar;
    set(h, 'FontSize', 14, 'LineWidth', 1.5);
    xlabel(horizontalClass);
    ylabel(verticalClass);
    axis off;
    axis equal;
    currentFile = ['averaged_results/meanAnalysisMatrixUnfitted.jpg'];
    saveas(h, currentFile);
    currentFrame = imread(currentFile);
    imwrite(currentFrame, currentFile, 'jpg', 'Quality', 100);
    close;

    figure;
    imagesc(deviationGroupingMatrix(2:(verticalParameters + 1), 2:(horizontalParameters + 1)));

    h = gca;
    colorbar;
    set(h, 'FontSize', 14, 'LineWidth', 1.5);
    xlabel(horizontalClass);
    ylabel(verticalClass);
    axis off;
    axis equal;
    currentFile = ['averaged_results/deviationGroupingMatrix.jpg'];
    saveas(h, currentFile);
    currentFrame = imread(currentFile);
    imwrite(currentFrame, currentFile, 'jpg', 'Quality', 100);
    close;

    figure;
    imagesc(meanGroupingMatrix(2:(verticalParameters + 1), 2:(horizontalParameters + 1)));

    h = gca;
    colorbar;
    set(h, 'FontSize', 14, 'LineWidth', 1.5);
    xlabel(horizontalClass);
    ylabel(verticalClass);
    axis off;
    axis equal;
    currentFile = ['averaged_results/meanGroupingMatrix.jpg'];
    saveas(h, currentFile);
    currentFrame = imread(currentFile);
    imwrite(currentFrame, currentFile, 'jpg', 'Quality', 100);
    close;

    [xSize ySize zSize] = size(statisticalSlidingWindowResults);

    meanSlidingWindowHistograms = zeros(xSize, ySize);
    deviationSlidingWindowHistograms = zeros(xSize, ySize);
    meanSlidingWindowHistograms(:, 1:2) = statisticalSlidingWindowResults(:, 1:2, 1);
    deviationSlidingWindowHistograms(:, 1:2) = statisticalSlidingWindowResults(:, 1:2, 1);

    meanSlidingWindowDeviationHistograms = zeros(xSize, ySize);
    deviationSlidingWindowDeviationHistograms = zeros(xSize, ySize);
    meanSlidingWindowDeviationHistograms(:, 1:2) = statisticalSlidingWindowDeviationResults(:, 1:2, 1);
    deviationSlidingWindowDeviationHistograms(:, 1:2) = statisticalSlidingWindowDeviationResults(:, 1:2, 1);

    for n = 1:xSize
        successfulRuns = [];
        for m = 1:zSize
            if sum(statisticalSlidingWindowResults(n, 3:ySize, m)) ~= 0
                successfulRuns = cat(2, successfulRuns, m);
            end;
        end;
        if ~isempty(successfulRuns)
            meanSlidingWindowHistograms(n, 3:ySize) = mean(statisticalSlidingWindowResults(n, 3:ySize, successfulRuns), 3);
            deviationSlidingWindowHistograms(n, 3:ySize) = std(statisticalSlidingWindowResults(n, 3:ySize, successfulRuns), 0, 3);
            meanSlidingWindowDeviationHistograms(n, 3:ySize) = mean(statisticalSlidingWindowDeviationResults(n, 3:ySize, successfulRuns), 3);
            deviationSlidingWindowDeviationHistograms(n, 3:ySize) = std(statisticalSlidingWindowDeviationResults(n, 3:ySize, successfulRuns), 0, 3);
        end;
    end;

    save averaged_results/meanSlidingWindowHistograms meanSlidingWindowHistograms;
    % save averaged_results/meanSlidingWindowHistograms.txt meanSlidingWindowHistograms -ascii;
    save averaged_results/deviationSlidingWindowHistograms deviationSlidingWindowHistograms;
    % save averaged_results/deviationSlidingWindowHistograms.txt deviationSlidingWindowHistograms -ascii;
    save averaged_results/meanSlidingWindowDeviationHistograms meanSlidingWindowDeviationHistograms;
    % save averaged_results/meanSlidingWindowDeviationHistograms.txt meanSlidingWindowDeviationHistograms -ascii;
    save averaged_results/deviationSlidingWindowDeviationHistograms deviationSlidingWindowDeviationHistograms;
    % save averaged_results/deviationSlidingWindowDeviationHistograms.txt deviationSlidingWindowDeviationHistograms -ascii;

    meanSlidingWindowHistograms
    deviationSlidingWindowHistograms
    meanSlidingWindowDeviationHistograms
    deviationSlidingWindowDeviationHistograms

    for a = 1:verticalParameters
        for b = 1:horizontalParameters
            if performSurvivalScreen == 2
                currentExperimentString = ['_F' num2str(clusterFrequencyArray(1, b), '%.3d') '_K' num2str(seedingKnockoutArray(1, a), '%.4f')];
            else
                currentExperimentString = ['_M' num2str(verticalParametersList(1, a), '%.6f') '_R' num2str(horizontalParametersList(1, b), '%.6f')];
            end;

            slidingWindowEssentialGenesHistogram = meanSlidingWindowHistograms(2 * (horizontalParameters * (a - 1) + b) - 1, 3:13);
            slidingWindowRecombinationSitesHistogram = meanSlidingWindowHistograms(2 * (horizontalParameters * (a - 1) + b), 3:13);
            slidingWindowEssentialGenesDeviationHistogram = meanSlidingWindowDeviationHistograms(2 * (horizontalParameters * (a - 1) + b) - 1, 3:13);
            slidingWindowRecombinationSitesDeviationHistogram = meanSlidingWindowDeviationHistograms(2 * (horizontalParameters * (a - 1) + b), 3:13);

            if sum(slidingWindowEssentialGenesHistogram) ~= 0

                figure;
                bar(0:slidingWindowSize, slidingWindowEssentialGenesHistogram, 'grouped');
                h = gca;
                set(h, 'FontSize', 14, 'LineWidth', 1.5);
                xlabel('clustered essential genes');
                ylabel('frequency');
                axis([-1 slidingWindowSize + 1 0 1]);
                currentFile = ['averaged_results/slidingWindowEssentialGenes' currentExperimentString '.jpg'];
                saveas(h, currentFile);
                currentFrame = imread(currentFile);
                imwrite(currentFrame, currentFile, 'jpg', 'Quality', 100);
                close;

                figure;
                bar(0:slidingWindowSize, slidingWindowRecombinationSitesHistogram, 'grouped');
                h = gca;
                set(h, 'FontSize', 14, 'LineWidth', 1.5);
                xlabel('clustered recombination sites');
                ylabel('frequency');
                axis([-1 slidingWindowSize + 1 0 1]);
                currentFile = ['averaged_results/slidingWindowRecombinationSites' currentExperimentString '.jpg'];
                saveas(h, currentFile);
                currentFrame = imread(currentFile);
                imwrite(currentFrame, currentFile, 'jpg', 'Quality', 100);
                close;

                figure;
                bar(0:slidingWindowSize, [slidingWindowEssentialGenesHistogram' slidingWindowRecombinationSitesHistogram'], 'grouped');
                h = gca;
                set(h, 'FontSize', 14, 'LineWidth', 1.5);
                xlabel('clustered units');
                ylabel('frequency');
                axis([-1 slidingWindowSize + 1 0 1]);
                currentFile = ['averaged_results/slidingWindowGrouped' currentExperimentString '.jpg'];
                saveas(h, currentFile);
                currentFrame = imread(currentFile);
                imwrite(currentFrame, currentFile, 'jpg', 'Quality', 100);
                close;

                figure;
                bar(0:slidingWindowSize, slidingWindowEssentialGenesDeviationHistogram, 'grouped');
                h = gca;
                set(h, 'FontSize', 14, 'LineWidth', 1.5);
                xlabel('clustered essential genes');
                ylabel('frequency');
                axis([-1 slidingWindowSize + 1 -1 1]);
                currentFile = ['averaged_results/slidingWindowDeviationEssentialGenes' currentExperimentString '.jpg'];
                saveas(h, currentFile);
                currentFrame = imread(currentFile);
                imwrite(currentFrame, currentFile, 'jpg', 'Quality', 100);
                close;

                figure;
                bar(0:slidingWindowSize, slidingWindowRecombinationSitesDeviationHistogram, 'grouped');
                h = gca;
                set(h, 'FontSize', 14, 'LineWidth', 1.5);
                xlabel('clustered recombination sites');
                ylabel('frequency');
                axis([-1 slidingWindowSize + 1 -1 1]);
                currentFile = ['averaged_results/slidingWindowDeviationRecombinationSites' currentExperimentString '.jpg'];
                saveas(h, currentFile);
                currentFrame = imread(currentFile);
                imwrite(currentFrame, currentFile, 'jpg', 'Quality', 100);
                close;

                figure;
                bar(0:slidingWindowSize, [slidingWindowEssentialGenesDeviationHistogram' slidingWindowRecombinationSitesDeviationHistogram'], 'grouped');
                h = gca;
                set(h, 'FontSize', 14, 'LineWidth', 1.5);
                xlabel('clustered units');
                ylabel('frequency');
                axis([-1 slidingWindowSize + 1 -1 1]);
                currentFile = ['averaged_results/slidingWindowDeviationGrouped' currentExperimentString '.jpg'];
                saveas(h, currentFile);
                currentFrame = imread(currentFile);
                imwrite(currentFrame, currentFile, 'jpg', 'Quality', 100);
                close;

            end;
        end;
    end;
end;

jobCompleted = 1;
if frontendFlag == 0 && ~clusterMode
    save jobCompleted.txt jobCompleted -ascii;
end;



% The following section provides Matlab core routines,
% required for compilation on the Unix cluster:



function p = betacdf(x,a,b)

if nargin<3, 
   error('stats:betacdf:TooFewInputs','Requires three input arguments.'); 
end

[errorcode x a b] = distchck(3,x,a,b);

if errorcode > 0
   error('stats:betacdf:InputSizeMismatch',...
         'Requires non-scalar arguments to match in size.');
end

% Initialize P to 0.
if isa(x,'single') || isa(a,'single') || isa(b,'single')
   p = zeros(size(x),'single');
else
   p = zeros(size(x));
end

p(a<=0 | b<=0) = NaN;

% If is X >= 1 the cdf of X is 1. 
p(x >= 1) = 1;

k = find(x > 0 & x < 1 & a > 0 & b > 0);
if any(k)
   p(k) = betainc(x(k),a(k),b(k));
end

% Make sure that round-off errors never make P greater than 1.
p(p > 1) = 1;



function x = betainv(p,a,b)

if nargin <  3, 
    error('stats:betainv:TooFewInputs','Requires three input arguments.'); 
end

[errorcode p a b] = distchck(3,p,a,b);

if errorcode > 0
    error('stats:betainv:InputSizeMismatch',...
          'Requires non-scalar arguments to match in size.');
end

%   Initialize x to zero.
if isa(p,'single') || isa(a,'single') || isa(b,'single')
   x = zeros(size(p),'single');
   seps = sqrt(eps('single'));
else
   x = zeros(size(p));
   seps = sqrt(eps);
end

%   Return NaN if the arguments are outside their respective limits.
x(p < 0 | p > 1 | a <= 0 | b <= 0) = NaN;

% The inverse cdf of 0 is 0, and the inverse cdf of 1 is 1.  
k0 = find(p == 0 & a > 0 & b > 0);
if any(k0), 
    x(k0) = 0;
end

k1 = find(p==1);
if any(k1), 
    x(k1) = 1;
end

% Newton's Method.
% Permit no more than count_limit interations.
count_limit = 100;
count = 0;


k = find(p > 0 & p < 1 & a > 0 & b > 0);
if isempty(k)
   return;
end
pk = p(k);

%   Use the mean as a starting guess. 
xk = a(k) ./ (a(k) + b(k));
if isa(p,'single')
   xk = single(xk);
end

% Move starting values away from the boundaries.
xk(xk==0) = seps;
xk(xk==1) = 1 - seps;

h = ones(size(pk));
crit = seps;

% Break out of the iteration loop for the following:
%  1) The last update is very small (compared to x).
%  2) The last update is very small (compared to 100*eps).
%  3) There are more than 100 iterations. This should NEVER happen. 

while(any(abs(h) > crit * abs(xk)) && max(abs(h)) > crit    ...
                                   && count < count_limit), 
                                 
    count = count+1;    
    h = (betacdf(xk,a(k),b(k)) - pk) ./ betapdf(xk,a(k),b(k));
    xnew = xk - h;

% Make sure that the values stay inside the bounds.
% Initially, Newton's Method may take big steps.
    ksmall = find(xnew <= 0);
    klarge = find(xnew >= 1);
    if any(ksmall) || any(klarge)
        xnew(ksmall) = xk(ksmall) /10;
        xnew(klarge) = 1 - (1 - xk(klarge))/10;
    end

    xk = xnew;  
end

% Return the converged value(s).
x(k) = xk;

if count==count_limit, 
    fprintf('\nWarning: BETAINV did not converge.\n');
    str = 'The last step was:  ';
    outstr = sprintf([str,'%13.8f\n'],max(h(:)));
    fprintf(outstr);
end



function y = betapdf(x,a,b)

if nargin < 3
   error('stats:betapdf:TooFewInputs','Requires three input arguments.');
end

% Return NaN for out of range parameters.
a(a<=0) = NaN;
b(b<=0) = NaN;

% Out of range x could create a spurious NaN*i part to y, prevent that.
% These entries will get set to zero later.
xOutOfRange = (x<0) | (x>1);
x(xOutOfRange) = .5;

try
    % When a==1, the density has a limit of beta(a,b) at x==0, and
    % similarly when b==1 at x==1.  Force that, instead of 0*log(0) = NaN.
    warn = warning('off','MATLAB:log:logOfZero');
    logkerna = (a-1).*log(x);   logkerna(a==1 & x==0) = 0;
    logkernb = (b-1).*log(1-x); logkernb(b==1 & x==1) = 0;
    warning(warn);
    y = exp(logkerna+logkernb - betaln(a,b));
catch
    warning(warn);
    error('stats:betapdf:InputSizeMismatch',...
          'Non-scalar arguments must match in size.');
end

% Fill in for the out of range x values, but don't overwrite NaNs from
% nonpositive params.
y(xOutOfRange & ~isnan(a) & ~isnan(b)) = 0;



function p = chi2cdf(x,v)

if   nargin < 2, 
    error('stats:chi2cdf:TooFewInputs','Requires two input arguments.');
end

[errorcode x v] = distchck(2,x,v);

if errorcode > 0
    error('stats:chi2cdf:InputSizeMismatch',...
          'Requires non-scalar arguments to match in size.');
end
    
% Call the gamma distribution function. 
p = gamcdf(x,v/2,2);



function [dy,y,d2y] = dgammainc(x,a,tail)

if nargin < 3
    lower = true;
else
    switch tail
    case 'lower', lower = true;
    case 'upper', lower = false;
    otherwise, error('stats:dgammainc:InvalidTailArg', ...
                     'TAIL must be ''lower'' or ''upper''.');
    end
end

% x and a must be compatible for addition.
try
   y = x + a;
   y(:) = NaN;
catch
   error('stats:dgammainc:InputSizeMismatch', ...
         'X and A must be the same size, or scalars.')
end
dy = y;
if nargout > 2
    d2y = y;
end

if any(a(:) < 0)
   error('stats:dgammainc:NegativeArg', 'A must be non-negative.')
end

% If a is a vector, make sure x is too.
ascalar = isscalar(a);
if ~ascalar && isscalar(x)
   x = repmat(x,size(a));
end

% Upper limit for series and continued fraction.
amax = 2^20;

% Approximation for a > amax.  Accurate to about 5.e-5.
k = find(a > amax);
if ~isempty(k)
   if ascalar
      x = max(amax-1/3 + sqrt(amax/a).*(x-(a-1/3)),0);
      a = amax;
   else
      x(k) = max(amax-1/3 + sqrt(amax./a(k)).*(x(k)-(a(k)-1/3)),0);
      a(k) = amax;
   end
end

%
% Series expansion for lower incomplete gamma when x < a+1
%
k = find(x < a+1 & x ~= 0);
if ~isempty(k)
    xk = x(k);
    if ascalar, ak = a; else ak = a(k); end
    aplusn = ak;
    del = 1;
    ddel = 0;
    d2del = 0;
    sum = del;
    dsum = ddel;
    d2sum = d2del;
    while norm(del,'inf') >= 100*eps(norm(sum,'inf'))
        aplusn = aplusn + 1;
        del = del .* xk ./ aplusn;
        ddel = (ddel .* xk - del) ./ aplusn;
        d2del = (d2del .* xk - 2 .* ddel) ./ aplusn;
        sum = sum + del;
        dsum = dsum + ddel;
        d2sum = d2sum + d2del;
    end
    fac = exp(-xk + ak.*log(xk) - gammaln(ak+1));
    yk = fac.*sum;
    % For very small a, the series may overshoot very slightly.
    yk(xk > 0 & yk > 1) = 1;
    if lower, y(k) = yk; else y(k) = 1 - yk; end
    
    dlogfac = (log(xk) - psi(ak+1));
    dfac = fac .* dlogfac;
    dyk = dfac.*sum + fac.*dsum;
    if lower, dy(k) = dyk; else dy(k) = -dyk; end
    
    if nargout > 2
        d2fac = dfac.*dlogfac - fac.*psi(1,ak+1);
        d2yk = d2fac.*sum + 2.*dfac.*dsum + fac.*d2sum;
        if lower, d2y(k) = d2yk; else d2y(k) = -d2yk; end
    end
end

%
% Continued fraction for upper incomplete gamma when x >= a+1
%
k = find(x >= a+1); % & x ~= 0
if ~isempty(k)
    xk = x(k);
    if ascalar, ak = a; else ak = a(k); end
    n = 0;
    a0 = 0;
    a1 = ak;
    b0 = 1;
    b1 = xk;
    da0 = 0; db0 = 0; da1 = 1; db1 = 0;
    d2a0 = 0; d2b0 = 0; d2a1 = 0; d2b1 = 0;
    g = ak ./ xk;
    dg = 1 ./ xk;
    d2g = 0;
    d2gold = 1; % force one iteration, any nonzero value will do
    % Testing d2g is the more stringent than testing g or dg.  d2g may
    % be zero, so use a strict inequality
    while norm(d2g-d2gold,'inf') > 100*eps(norm(d2g,'inf'))
        rescale = 1 ./ b1; % keep terms from overflowing
        n = n + 1;
        nminusa = n - ak;
        d2a0 = (d2a1 + d2a0 .* nminusa - 2 .* da0) .* rescale;
        d2b0 = (d2b1 + d2b0 .* nminusa - 2 .* db0) .* rescale;
        da0 = (da1 + da0 .* nminusa - a0) .* rescale;
        db0 = (db1 + db0 .* nminusa - b0) .* rescale;
        a0 = (a1 + a0 .* nminusa) .* rescale;
        b0 = 1 + (b0 .* nminusa) .* rescale; % (b1 + b0 .* nminusa) .* rescale
        nrescale = n .* rescale;
        d2a1 = d2a0 .* xk + d2a1 .* nrescale;
        d2b1 = d2b0 .* xk + d2b1 .* nrescale;
        da1 = da0 .* xk + da1 .* nrescale;
        db1 = db0 .* xk + db1 .* nrescale;
        a1 = a0 .* xk + a1 .* nrescale;
        b1 = b0 .* xk + n; % b0 .* xk + b1 .* nrescale
        d2gold = d2g;
        g = a1 ./ b1;
        dg = (da1 - g.*db1) ./ b1;
        d2g = (d2a1 - dg.*db1 - g.*d2b1 - dg.*db1) ./ b1;
    end
    fac = exp(-xk + ak.*log(xk) - gammaln(ak+1));
    yk = fac.*g;
    if lower, y(k) = 1 - yk; else y(k) = yk; end
    
    dlogfac = (log(xk) - psi(ak+1));
    dfac = fac .* dlogfac;
    dyk = dfac.*g + fac.*dg;
    if lower, dy(k) = -dyk; else dy(k) = dyk; end
    
    if nargout > 2
        d2fac = dfac.*dlogfac - fac.*psi(1,ak+1);
        d2yk = d2fac.*g + 2.*dfac.*dg + fac.*d2g;
        if lower, d2y(k) = -d2yk; else d2y(k) = d2yk; end
    end
end

% Handle x == 0 separately to get it exactly correct.
kx0 = find(x == 0);
if ~isempty(kx0)
    if lower, y(kx0) = 0; else y(kx0) = 1; end
    dy(kx0) = 0;
    if nargout > 2
        d2y(kx0) = 0;
    end
end

% a == 0, x ~= 0 is already handled by the power series or continued
% fraction, now fill in dgammainc(0,0).  While we're at it, make
% gammainc(x,0) or 1-gammainc(x,0) exact for any x, not just x == 0.
ka0 = find(a == 0);
if ~isempty(ka0)
    if ascalar
        if lower
            y(:) = 1;
            dy(kx0) = -Inf;
            if nargout > 2, d2y(kx0) = Inf; end
        else
            y(:) = 0; 
            dy(kx0) = Inf;
            if nargout > 2, d2y(kx0) = -Inf; end
        end
    else
        ka0x0 = find(a == 0 & x == 0);
        if lower
            y(ka0) = 1;
            dy(ka0x0) = -Inf;
            if nargout > 2, d2y(ka0x0) = Inf; end
        else
            y(ka0) = 0;
            dy(ka0x0) = Inf;
            if nargout > 2, d2y(ka0x0) = -Inf; end
        end
    end
end



function [errorcode,varargout] = distchck(nparms,varargin)

errorcode = 0;
varargout = varargin;

if nparms == 1
    return;
end

% Get size of each input, check for scalars, copy to output
isscalar = (cellfun('prodofsize',varargin) == 1);

% Done if all inputs are scalars.  Otherwise fetch their common size.
if (all(isscalar)), return; end

n = nparms;

for j=1:n
   sz{j} = size(varargin{j});
end
t = sz(~isscalar);
size1 = t{1};

% Scalars receive this size.  Other arrays must have the proper size.
for j=1:n
   sizej = sz{j};
   if (isscalar(j))
      vj = varargin{j};
      if isnumeric(vj)
         t = zeros(size1,class(vj));
      else
         t = zeros(size1);
      end
      t(:) = varargin{j};
      varargout{j} = t;
   elseif (~isequal(sizej,size1))
      errorcode = 1;
      return;
   end
end



function p = fcdf(x,v1,v2)

if nargin < 3, 
    error('stats:fcdf:TooFewInputs','Requires three input arguments.'); 
end

[errorcode x v1 v2] = distchck(3,x,v1,v2);

if errorcode > 0
    error('stats:fcdf:InputSizeMismatch',...
          'Requires non-scalar arguments to match in size.');
end

% Initialize P to zero.
if isa(x,'single') || isa(v1,'single') || isa(v2,'single')
   p = zeros(size(x),'single');
else
   p = zeros(size(x));
end

t = (v1 <= 0 | v2 <= 0 | isnan(x) | isnan(v1) | isnan(v2));
p(t) = NaN;
s = (x==Inf) & ~t;
if any(s(:))
   p(s) = 1;
   t = t | s;
end

% Compute P when X > 0.
k = find(x > 0 & ~t & isfinite(v1) & isfinite(v2));
if any(k), 
% use A&S formula 26.6.2 to relate to incomplete beta function 
    % Also use 26.5.2 to avoid cancellation by subtracting from 1
    xx = x(k)./(x(k) + v2(k)./v1(k));
    p(k) = betainc(xx, v1(k)/2, v2(k)/2);
end

if any(~isfinite(v1(:)) | ~isfinite(v2(:)))
   k = find(x > 0 & ~t & isfinite(v1) & ~isfinite(v2) & v2>0);
   if any(k)
      p(k) = chi2cdf(v1(k).*x(k),v1(k));
   end
   k = find(x > 0 & ~t & ~isfinite(v1) & v1>0 & isfinite(v2));
   if any(k)
      p(k) = 1 - chi2cdf(v2(k)./x(k),v2(k));
   end
   k = find(x > 0 & ~t & ~isfinite(v1) & v1>0 & ~isfinite(v2) & v2>0);
   if any(k)
      p(k) = (x(k)>=1);
   end
end



function [p,plo,pup] = gamcdf(x,a,b,pcov,alpha)

if nargin < 2
    error('stats:gamcdf:TooFewInputs',...
          'Requires at least two input arguments.');
elseif nargin < 3
    b = 1;
end

% More checking if we need to compute confidence bounds.
if nargout > 1
    if nargin < 4
        error('stats:gamcdf:TooFewInputs',...
              'Must provide covariance matrix to compute confidence bounds.');
    end
    if ~isequal(size(pcov),[2 2])
        error('stats:gamcdf:BadCovariance',...
              'Covariance matrix must have 2 rows and columns.');
    end
    if nargin < 5
        alpha = 0.05;
    elseif ~isnumeric(alpha) || numel(alpha) ~= 1 || alpha <= 0 || alpha >= 1
        error('stats:gamcdf:BadAlpha',...
              'ALPHA must be a scalar between 0 and 1.');
    end
end

% Return NaN for out of range parameters.
a(a <= 0) = NaN;
b(b <= 0) = NaN;
x(x < 0) = 0;

try
    z = x ./ b;
    p = gammainc(z, a);
catch
    error('stats:gamcdf:InputSizeMismatch',...
          'Non-scalar arguments must match in size.');
end
p(z == Inf) = 1;

% Compute confidence bounds if requested.
if nargout >= 2
    % Approximate the variance of p on the logit scale
    logitp = log(p./(1-p));
    dp = 1 ./ (p.*(1-p)); % derivative of logit(p) w.r.t. p
    da = dgammainc(z,a) .* dp; % dlogitp/da = dp/da * dlogitp/dp
    db = -exp(a.*log(z)-z-gammaln(a)-log(b)) .* dp; % dlogitp/db = dp/db * dlogitp/dp
    varLogitp = pcov(1,1).*da.^2 + 2.*pcov(1,2).*da.*db + pcov(2,2).*db.^2;
    if any(varLogitp(:) < 0)
        error('stats:gamcdf:BadCovariance',...
              'PCOV must be a positive semi-definite matrix.');
    end
    
    % Use a normal approximation on the logit scale, then transform back to
    % the original CDF scale
    halfwidth = -norminv(alpha/2) * sqrt(varLogitp);
    explogitplo = exp(logitp - halfwidth);
    explogitpup = exp(logitp + halfwidth);
    plo = explogitplo ./ (1 + explogitplo);
    pup = explogitpup ./ (1 + explogitpup);
end



function [x,xlo,xup] = norminv(p,mu,sigma,pcov,alpha)

if nargin<1
    error('stats:norminv:TooFewInputs','Input argument P is undefined.');
end
if nargin < 2
    mu = 0;
end
if nargin < 3
    sigma = 1;
end

% More checking if we need to compute confidence bounds.
if nargout>2
   if nargin<4
      error('stats:norminv:TooFewInputs',...
            'Must provide covariance matrix to compute confidence bounds.');
   end
   if ~isequal(size(pcov),[2 2])
      error('stats:norminv:BadCovariance',...
            'Covariance matrix must have 2 rows and columns.');
   end
   if nargin<5
      alpha = 0.05;
   elseif ~isnumeric(alpha) || numel(alpha)~=1 || alpha<=0 || alpha>=1
      error('stats:norminv:BadAlpha',...
            'ALPHA must be a scalar between 0 and 1.');
   end
end

% Return NaN for out of range parameters or probabilities.
sigma(sigma <= 0) = NaN;
p(p < 0 | 1 < p) = NaN;

x0 = -sqrt(2).*erfcinv(2*p);
try
    x = sigma.*x0 + mu;
catch
    error('stats:norminv:InputSizeMismatch',...
          'Non-scalar arguments must match in size.');
end

% Compute confidence bounds if requested.
if nargout>=2
   xvar = pcov(1,1) + 2*pcov(1,2)*x0 + pcov(2,2)*x0.^2;
   if any(xvar<0)
      error('stats:norminv:BadCovariance',...
            'PCOV must be a positive semi-definite matrix.');
   end
   normz = -norminv(alpha/2);
   halfwidth = normz * sqrt(xvar);
   xlo = x - halfwidth;
   xup = x + halfwidth;
end



function x = tinv(p,v)

if nargin < 2, 
    error('stats:tinv:TooFewInputs','Requires two input arguments.'); 
end

[errorcode p v] = distchck(2,p,v);

if errorcode > 0
    error('stats:tinv:InputSizeMismatch',...
          'Requires non-scalar arguments to match in size.');
end

% Initialize Y to zero, or NaN for invalid d.f.
if isa(p,'single') || isa(v,'single')
    x = zeros(size(p),'single');
else
    x = zeros(size(p));
end
x(v <= 0) = NaN;

k = find(v == 1 & ~isnan(x));
if any(k)
  x(k) = tan(pi * (p(k) - 0.5));
end

% The inverse cdf of 0 is -Inf, and the inverse cdf of 1 is Inf.
k0 = find(p == 0 & ~isnan(x));
if any(k0)
    tmp   = Inf;
    x(k0) = -tmp(ones(size(k0)));
end
k1 = find(p ==1 & ~isnan(x));
if any(k1)
    tmp   = Inf;
    x(k1) = tmp(ones(size(k1)));
end

% For small d.f., call betainv which uses Newton's method
k = find(p >= 0.5 & p < 1 & ~isnan(x) & v < 1000);
if any(k)
    z = betainv(2*(1-p(k)),v(k)/2,0.5);
    x(k) = sqrt(v(k) ./ z - v(k));
end

k = find(p < 0.5 & p > 0 & ~isnan(x) & v < 1000);
if any(k)
    z = betainv(2*(p(k)),v(k)/2,0.5);
    x(k) = -sqrt(v(k) ./ z - v(k));
end

% For large d.f., use Abramowitz & Stegun formula 26.7.5
k = find(p>0 & p<1 & ~isnan(x) & v >= 1000);
if any(k)
   xn = norminv(p(k));
   df = v(k);
   x(k) = xn + (xn.^3+xn)./(4*df) + ...
           (5*xn.^5+16.*xn.^3+3*xn)./(96*df.^2) + ...
           (3*xn.^7+19*xn.^5+17*xn.^3-15*xn)./(384*df.^3) +...
           (79*xn.^9+776*xn.^7+1482*xn.^5-1920*xn.^3-945*xn)./(92160*df.^4);
end



function [pbest, perror, nchi2] = fitting(mfunc, x, y, sy, pt, v)

% Levenberg-Marquardt non-linear regression
% Written 4/12/95 by Michael Fleming, University Of Auckland

chiCut = 0.01;
lambda = 0.001;
stepSize = 0.001;
chiOld = Inf;

y = y(:);
x = x(:);
sy = sy(:);
pt = pt(:);
v = v(:);
maxIter = 30;

m = length(pt);
n = length(x);

evalstr = [mfunc '(x,[']; % Create string containing function call
count = 0;
for i = 1:m             
    if v(i) == 1 % Extract vector p of parameters which will be varied
        count = count + 1; % from vector pt which contains all input parameters
        p(count) = pt(i);
        evalstr = [evalstr 'p(' int2str(count) ') '];
    else
        evalstr = [evalstr num2str(pt(i)) ' '];
    end;
    if (pt(i) == 0) % Check all parameters are non-zero
        error('You have entered a zero parameter. All parameters must be nonzero.');
    end;
end;

evalstr = [evalstr '])'];

dof = n - count; % dof is the number of degrees of freedom
p = p(:);
delta = p * stepSize; % delta is the size of the small increment used for calculating numerical derivatives

chiSqr = sum((abs(y - eval(evalstr)) ./ sy) .^ 2); % Calc ChiSqr from initial parameters
% if chiSqr / dof > 5000;
%     disp('You have made a bad choice of initial parameters.');
% end;

iter = 0;
while (abs(chiOld - chiSqr) > chiCut) && (iter < maxIter);
    iter = iter + 1;
    chiOld = chiSqr;
    [alpha, beta] = subsidiary(evalstr, x, y, sy, p, delta, lambda);
    % if det(alpha) == 0;
    %     error('No convergence - try a different set of parameters.');
    % end;
    dp = alpha \ beta; % Evaluate parameter increments
    p = p + dp;
    chiSqr = sum((abs(y - eval(evalstr)) ./ sy) .^ 2);
    while (chiSqr > (chiOld + chiCut));
        p = p - dp;
        iter = iter + 1;
        lambda = lambda * 10;
        [alpha, beta] = subsidiary(evalstr, x, y, sy, p, delta, lambda);
        % if det(alpha) == 0;
        %     error('No convergence - try a different set of parameters.');
        % end;
        dp = alpha \ beta; % Evaluate parameter increments
        p = p + dp;
        chiSqr = sum((abs(y - eval(evalstr)) ./ sy) .^ 2);
    end;
    lambda = 0.1 * lambda;
end;

% if iter == maxIter,
%     disp('Maximum number of iterations exceeded - convergence not achieved');
% end;

% xfine = linspace(x(1), x(n), 201); % Print graph of function
% yfitted = model(xfine, p);
% errorbar(x, y, sy);
% Calculate correlation coefficicent R^2
% f = eval(evalstr);   
% r = corrcoef(y, f);
% r2 = r(1, 2) .^ 2;

% Calculate standard errors in parameters
[alpha, beta] = subsidiary(evalstr, x, y, sy, p, delta, 0); % Calculate final alpha matrix
sp = sqrt(diag(inv(alpha))); % Evaluate standard errors in parameters.
% disp('Parameters');
count = 0;
for i = 1:m
    if v(i) == 1
        count = count + 1;
        pbest(i) = p(count);
        perror(i) = sp(count);
        % disp(['p(', int2str(i), ') = ' sprintf('%5.3e', p(count)), '+-' sprintf('%5.3e', sp(count))]);
    else
        % disp(['p(', int2str(i), ') = ' sprintf('%5.3e', pt(i)), '+-0']);
        pbest(i) = pt(i);
        perror(i) = 0;
    end;
end;

format compact
% disp('Correlation Coefficient R^2');
% disp(r2);
% disp('Normalised chi-squared value');
nchi2 = chiSqr / dof;
% disp(nchi2);
format;



function [b,bint,r,rint,stats] = regress(y,X,alpha)

if  nargin < 2
    error('stats:regress:TooFewInputs', ...
          'REGRESS requires at least two input arguments.');
elseif nargin == 2
    alpha = 0.05;
end

% Check that matrix (X) and left hand side (y) have compatible dimensions
[n,ncolX] = size(X);
if ~isvector(y)
    error('stats:regress:InvalidData', 'Y must be a vector.');
elseif numel(y) ~= n
    error('stats:regress:InvalidData', ...
          'The number of rows in Y must equal the number of rows in X.');
end

% Remove missing values, if any
wasnan = (isnan(y) | any(isnan(X),2));
havenans = any(wasnan);
if havenans
   y(wasnan) = [];
   X(wasnan,:) = [];
   n = length(y);
end

% Use the rank-revealing QR to remove dependent columns of X.
[Q,R,perm] = qr(X,0);
p = sum(abs(diag(R)) > max(n,ncolX)*eps(R(1)));
if p < ncolX
    warning('stats:regress:RankDefDesignMat', ...
            'X is rank deficient to within machine precision.');
    R = R(1:p,1:p);
    Q = Q(:,1:p);
    perm = perm(1:p);
end

% Compute the LS coefficients, filling in zeros in elements corresponding
% to rows of X that were thrown out.
b = zeros(ncolX,1);
b(perm) = R \ (Q'*y);

if nargout >= 2
    % Find a confidence interval for each component of x
    % Draper and Smith, equation 2.6.15, page 94
    RI = R\eye(p);
    nu = max(0,n-p);                % Residual degrees of freedom
    yhat = X*b;                     % Predicted responses at each data point.
    r = y-yhat;                     % Residuals.
    normr = norm(r);
    if nu ~= 0
        rmse = normr/sqrt(nu);    % Root mean square error.
        tval = tinv((1-alpha/2),nu);
    else
        rmse = NaN;
        tval = 0;
    end
    s2 = rmse^2;                    % Estimator of error variance.
    se = zeros(ncolX,1);
    se(perm,:) = rmse*sqrt(sum(abs(RI).^2,2));
    bint = [b-tval*se, b+tval*se];

    % Find the standard errors of the residuals.
    % Get the diagonal elements of the "Hat" matrix.
    % Calculate the variance estimate obtained by removing each case (i.e. sigmai)
    % see Chatterjee and Hadi p. 380 equation 14.
    if nargout >= 4
        hatdiag = sum(abs(Q).^2,2);
        ok = ((1-hatdiag) > sqrt(eps(class(hatdiag))));
        hatdiag(~ok) = 1;
        if nu > 1
            denom = (nu-1) .* (1-hatdiag);
            sigmai = zeros(length(denom),1);
            sigmai(ok) = sqrt(max(0,(nu*s2/(nu-1)) - (r(ok) .^2 ./ denom(ok))));
            ser = sqrt(1-hatdiag) .* sigmai;
            ser(~ok) = Inf;
        elseif nu == 1
            ser = sqrt(1-hatdiag) .* rmse;
            ser(~ok) = Inf;
        else % if nu == 0
            ser = rmse*ones(length(y),1); % == Inf
        end

        % Create confidence intervals for residuals.
        rint = [(r-tval*ser) (r+tval*ser)];
    end

    % Calculate R-squared and the other statistics.
    if nargout == 5
        % There are several ways to compute R^2, all equivalent for a
        % linear model where X includes a constant term, but not equivalent
        % otherwise.  R^2 can be negative for models without an intercept.
        % This indicates that the model is inappropriate.
        SSE = normr.^2;              % Error sum of squares.
        RSS = norm(yhat-mean(y))^2;  % Regression sum of squares.
        TSS = norm(y-mean(y))^2;     % Total sum of squares.
        r2 = 1 - SSE/TSS;            % R-square statistic.
        if p > 1
            F = (RSS/(p-1))/s2;      % F statistic for regression
        else
            F = NaN;
        end
        prob = 1 - fcdf(F,p-1,nu);   % Significance probability for regression
        stats = [r2 F prob s2];

        % All that requires a constant.  Do we have one?
        if ~any(all(X==1,1))
            % Apparently not, but look for an implied constant.
            b0 = R\(Q'*ones(n,1));
            if (sum(abs(1-X(:,perm)*b0))>n*sqrt(eps(class(X))))
                warning('stats:regress:NoConst',...
                        ['R-square and the F statistic are not well-defined' ...
                         ' unless X has a column of ones.\nType "help' ...
                         ' regress" for more information.']);
            end
        end
    end

    % Restore NaN so inputs and outputs conform
    if havenans
        if nargout >= 3
            tmp = repmat(NaN,length(wasnan),1);
            tmp(~wasnan) = r;
            r = tmp;
            if nargout >= 4
                tmp = repmat(NaN,length(wasnan),2);
                tmp(~wasnan,:) = rint;
                rint = tmp;
            end
        end
    end

end % nargout >= 2



function [alpha, beta] = subsidiary(evalstr, x, y, sy, p, delta, lambda)

% subsidiary is a subsidiary function of fitting. It must be called through fitting.

m = length(p);
n = length(x);
% Compute beta = -0.5 * grad(chiSqr)
% Numerical partial derivatives of chi-squared are taken in an interval around point p
for i = 1:m;
    p(i) = p(i) + 0.5 * delta(i);
    XiSqPlus = sum((abs(y - eval(evalstr)) ./ sy) .^ 2);
    p(i) = p(i) - delta(i);
    XiSqMinus = sum((abs(y - eval(evalstr)) ./ sy) .^ 2);
    p(i) = p(i) + 0.5 * delta(i);
    dXiSq_da(i) = (XiSqPlus - XiSqMinus) / delta(i);
end;
beta = -0.5 * dXiSq_da;
beta = beta(:);

%make alpha
alpha = zeros(m);

y0 = eval(evalstr);
for i = 1:m;
    p(i) = p(i) + delta(i);
    y1 = eval(evalstr);
    p(i) = p(i) - delta(i);
    dYda(:, i) = (y1 - y0) ./ (sy * delta(i));
end;
for j = 1:n
    jdYda = dYda(j, :);
    alpha = alpha + jdYda' * jdYda;
end;

for i = 1:m
    alpha(i, i) = alpha(i, i) * (1 + lambda);
end;



function y = erlang(x)
shapeFactor = 4;
y = 2 .* shapeFactor .* exp(-(2 .* shapeFactor .* x)) .* (2 .* shapeFactor .* x) .^ (shapeFactor - 1) ./ factorial(shapeFactor - 1);



function y = exponential(x, parameters)
y = parameters(1) .* exp(- parameters(2) .* x) + parameters(3);